@@ -24,6 +24,7 @@ Application module.
 Handles application initialization, shutdown, opening projects, autosave and changing
 sequences.
 """
+
 try:
     import pgi
     pgi.install_as_gi()
@@ -39,8 +40,8 @@ from gi.repository import Gtk
 from gi.repository import Gdk
 
 import locale
-import md5
 import mlt
+import hashlib
 import os
 import sys
 import time
@@ -52,17 +53,23 @@ import audiowaveformrenderer
 import clipeffectseditor
 import clipmenuaction
 import compositeeditor
+import containerclip
+import containeractions
 import dialogs
 import dialogutils
 import dnd
+import diskcachemanagement
 import edit
 import editevent
+import editorlayout
 import editorpersistance
 import editorstate
 import editorwindow
 import gmic
 import gui
+import jobs
 import keyevents
+import keyframeeditor
 import keyframeeditcanvas
 import kftoolmode
 import medialog
@@ -73,6 +80,7 @@ import mltprofiles
 import mlttransitions
 import modesetting
 import movemodes
+import multitrimmode
 import persistance
 import positionbar
 import preferenceswindow
@@ -86,21 +94,24 @@ import render
 import renderconsumer
 import respaths
 import resync
+import rotomask
 import sequence
 import shortcuts
 import snapping
 import threading
 import titler
+import tlinerender
 import tlinewidgets
 import toolsintegration
-import toolnatron
 import trimmodes
 import translations
 import undo
 import updater
+import userfolders
 import utils
 import workflow
 
+
 AUTOSAVE_DIR = appconsts.AUTOSAVE_DIR
 AUTOSAVE_FILE = "autosave/autosave"
 instance_autosave_id_str = None
@@ -108,6 +119,7 @@ PID_FILE = "flowbladepidfile"
 BATCH_DIR = "batchrender/"
 autosave_timeout_id = -1
 recovery_dialog_id = -1
+disk_cache_timeout_id = -1
 sdl2_timeout_id = -1
 loaded_autosave_file = None
 
@@ -118,6 +130,7 @@ window_resize_id = -1
 window_state_id = -1
 resize_timeout_id = -1
 
+
 _log_file = None
 
 assoc_file_path = None
@@ -132,7 +145,9 @@ def main(root_path):
     if _log_file != None:
         log_print_output_to_file()
 
-    print "Application version: " + editorstate.appversion
+    set_quiet_if_requested()
+
+    print("Application version: " + editorstate.appversion)
 
     # Print OS, Python version and GTK+ version
     try:
@@ -140,54 +155,29 @@ def main(root_path):
         os_text = os_release_file.read()
         s_index = os_text.find("PRETTY_NAME=")
         e_index = os_text.find("\n", s_index)
-        print "OS: " + os_text[s_index + 13:e_index - 1]
+        print("OS: " + os_text[s_index + 13:e_index - 1])
     except:
         pass
 
-    print "Python", sys.version
+    print("Python", sys.version)
 
     gtk_version = "%s.%s.%s" % (Gtk.get_major_version(), Gtk.get_minor_version(), Gtk.get_micro_version())
-    print "GTK+ version:", gtk_version
+    print("GTK+ version:", gtk_version)
     editorstate.gtk_version = gtk_version
     try:
         editorstate.mlt_version = mlt.LIBMLT_VERSION
     except:
         editorstate.mlt_version = "0.0.99" # magic string for "not found"
 
-    #print "SDL version:", str(editorstate.get_sdl_version())
+    # Create user folders if needed and determine if we're using xdg or dotfile user folders.
+    userfolders.init()
     
-    # passing -xdg as a flag will change the user_dir location with XDG_CONFIG_HOME
-    # For full xdg-app support all the launch processes need to add this too, currently not impl.
+    # Flatpak still needs to use standard home XDG cache folder for Blender.
+    # Flatpak only uses XDG cache folder for Blender and we are keeping this around if we ever
+    # succeed in getting Blender going for Flatpak.
+    if editorstate.app_running_from == editorstate.RUNNING_FROM_FLATPAK:
+        userfolders.init_user_cache_for_flatpak()
 
-    for arg in sys.argv:
-        if arg.lower() == "-xdg":
-            editorstate.use_xdg = True
-
-    # Create hidden folders if not present
-    user_dir = utils.get_hidden_user_dir_path()
-
-    print "User dir:",user_dir
-    if not os.path.exists(user_dir):
-        os.mkdir(user_dir)
-    if not os.path.exists(user_dir + mltprofiles.USER_PROFILES_DIR):
-        os.mkdir(user_dir + mltprofiles.USER_PROFILES_DIR)
-    if not os.path.exists(user_dir + AUTOSAVE_DIR):
-        os.mkdir(user_dir + AUTOSAVE_DIR)
-    if not os.path.exists(user_dir + BATCH_DIR):
-        os.mkdir(user_dir + BATCH_DIR)
-    if not os.path.exists(user_dir + appconsts.AUDIO_LEVELS_DIR):
-        os.mkdir(user_dir + appconsts.AUDIO_LEVELS_DIR)
-    if not os.path.exists(utils.get_hidden_screenshot_dir_path()):
-        os.mkdir(utils.get_hidden_screenshot_dir_path())
-    if not os.path.exists(user_dir + appconsts.GMIC_DIR):
-        os.mkdir(user_dir + appconsts.GMIC_DIR)
-    if not os.path.exists(user_dir + appconsts.MATCH_FRAME_DIR):
-        os.mkdir(user_dir + appconsts.MATCH_FRAME_DIR)
-    if not os.path.exists(user_dir + appconsts.TRIM_VIEW_DIR):
-        os.mkdir(user_dir + appconsts.TRIM_VIEW_DIR)
-    if not os.path.exists(user_dir + appconsts.NATRON_DIR):
-        os.mkdir(user_dir + appconsts.NATRON_DIR)
-       
     # Set paths.
     respaths.set_paths(root_path)
 
@@ -197,8 +187,6 @@ def main(root_path):
         respaths.apply_dark_theme()
     if editorpersistance.prefs.display_all_audio_levels == False:
         editorstate.display_all_audio_levels = False
-    editorpersistance.create_thumbs_folder_if_needed(user_dir)
-    editorpersistance.create_rendered_clips_folder_if_needed(user_dir)
 
     editorpersistance.save()
 
@@ -207,11 +195,18 @@ def main(root_path):
     translations.load_filters_translations()
     mlttransitions.init_module()
 
-    # Apr-2017 - SvdB - Keyboard shortcuts
+    # Keyboard shortcuts
     shortcuts.load_shortcut_files()
     shortcuts.load_shortcuts()
 
-    # We respaths and translations data available so we need to init in a function.
+    # The test for len != 4 is to make sure that if we change the number of values below the prefs are reset to the correct list
+    # So when we add or remove a value, make sure we also change the len test
+    # Only use positive numbers.
+    if( not editorpersistance.prefs.AUTO_SAVE_OPTS or len(editorpersistance.prefs.AUTO_SAVE_OPTS) != 4):
+        print("Initializing Auto Save Options")
+        editorpersistance.prefs.AUTO_SAVE_OPTS = ((0, _("No Autosave")),(1, _("1 min")),(2, _("2 min")),(5, _("5 min")))
+
+    # We need respaths and translations data available so we need to do init in a function.
     workflow.init_data()
 
     # RHEL7/CentOS compatibility fix
@@ -222,10 +217,23 @@ def main(root_path):
     Gdk.threads_init()
     Gdk.threads_enter()
 
-    # Themes
-    if editorpersistance.prefs.theme == appconsts.FLOWBLADE_THEME:
-        success = gui.apply_gtk_css()
+    # Handle userfolders init error and quit.
+    if userfolders.get_init_error() != None:
+        _xdg_error_exit(userfolders.get_init_error())
+        return
+
+    # After moving to Python 3 we need at least MLT 6.18
+    if editorstate.mlt_version_is_greater_correct("6.17.99") == False:
+        _too_low_mlt_version_exit()
+        return
+
+    # Apply custom themes.
+    if editorpersistance.prefs.theme == appconsts.FLOWBLADE_THEME \
+        or editorpersistance.prefs.theme == appconsts.FLOWBLADE_THEME_GRAY \
+        or editorpersistance.prefs.theme == appconsts.FLOWBLADE_THEME_NEUTRAL:
+        success = gui.apply_gtk_css(editorpersistance.prefs.theme)
         if not success:
+            print("Applying custom theme failed.")
             editorpersistance.prefs.theme = appconsts.LIGHT_THEME
             editorpersistance.save()
 
@@ -235,16 +243,8 @@ def main(root_path):
     # Load drag'n'drop images
     dnd.init()
 
-    # Adjust gui parameters for smaller screens
-    scr_w = Gdk.Screen.width()
-    scr_h = Gdk.Screen.height()
-    editorstate.SCREEN_WIDTH = scr_w
-    editorstate.SCREEN_HEIGHT = scr_h
-
-    print "Screen size:", scr_w, "x", scr_h
-    print "Small height:", editorstate.screen_size_small_height()
-    print "Small width:",  editorstate.screen_size_small_width()
-
+    # Save screen size data and modify rendering based on screen size/s and number of monitors. 
+    scr_w, scr_h = _set_screen_size_data()
     _set_draw_params()
 
     # Refuse to run on too small screen.
@@ -276,7 +276,10 @@ def main(root_path):
 
     # Create list of available mlt profiles.
     mltprofiles.load_profile_list()
-    
+
+    # If we have crashed we could have large amount of disk space wasted unless we delete all files here.
+    tlinerender.app_launch_clean_up()
+
     # Save assoc file path if found in arguments.
     global assoc_file_path
     assoc_file_path = get_assoc_file_path()
@@ -294,10 +297,9 @@ def main(root_path):
 
     # Check for tools and init tools integration.
     gmic.test_availablity()
-    toolnatron.init()
+    containerclip.test_blender_availebility()
     toolsintegration.init()
-    #toolsintegration.test()
-    
+
     # Create player object.
     create_player()
 
@@ -343,8 +345,11 @@ def main(root_path):
         else:
             GObject.timeout_add(10, autosaves_many_recovery_dialog)
     else:
+        tlinerender.init_session()
         start_autosave()
 
+    projectaction.clear_changed_since_last_save_flags()
+    
     # We prefer to monkeypatch some callbacks into some modules, usually to
     # maintain a simpler and/or non-circular import structure.
     monkeypatch_callbacks()
@@ -352,12 +357,9 @@ def main(root_path):
     # File in assoc_file_path is opened after very short delay.
     if not(check_crash == True and len(autosave_files) > 0):
         if assoc_file_path != None:
-            print "Launch assoc file:", assoc_file_path
+            print("Launch assoc file:", assoc_file_path)
             global assoc_timeout_id
             assoc_timeout_id = GObject.timeout_add(10, open_assoc_file)
-
-    if editorpersistance.prefs.theme == appconsts.FLOWBLADE_THEME:
-        gui.apply_flowblade_theme_fixes()
         
     # SDL 2 consumer needs to created after Gtk.main() has run enough for window to be visble
     #if editorstate.get_sdl_version() == editorstate.SDL_2: # needs more state considerion still
@@ -372,6 +374,15 @@ def main(root_path):
     if editorstate.runtime_version_greater_then_test_version(editorpersistance.prefs.workflow_dialog_last_version_shown, editorstate.appversion):
         GObject.timeout_add(500, show_worflow_info_dialog)
         
+    # Copy to XDG.
+    if userfolders.data_copy_needed():
+        GObject.timeout_add(500, show_user_folders_copy_dialog)
+    else:
+        print("No user folders actions needed.")
+
+    global disk_cache_timeout_id
+    disk_cache_timeout_id = GObject.timeout_add(2500, check_disk_cache_size)
+
     # Launch gtk+ main loop
     Gtk.main()
 
@@ -379,13 +390,11 @@ def main(root_path):
 
 # ----------------------------------- callback setting
 def monkeypatch_callbacks():
-    # Prefences setting
-    preferenceswindow.select_thumbnail_dir_callback = projectaction.select_thumbnail_dir_callback
-    preferenceswindow.select_render_clips_dir_callback = projectaction.select_render_clips_dir_callback
 
     # We need to do this on app start-up or
     # we'll get circular imports with projectaction->mltplayer->render->projectaction
     render.open_media_file_callback = projectaction.open_rendered_file
+    jobs.open_media_file_callback = projectaction.open_rendered_file
 
     # Set callback for undo/redo ops, batcherrender app does not need this 
     undo.set_post_undo_redo_callback(modesetting.set_post_undo_redo_edit_mode)
@@ -412,13 +421,25 @@ def monkeypatch_callbacks():
     snapping._get_x_for_frame_func = tlinewidgets._get_frame_x
 
     # Callback to reinit to change slider <-> kf editor
-    propertyeditorbuilder.re_init_editors_for_slider_type_change_func = clipeffectseditor.effect_selection_changed
+    propertyeditorbuilder.re_init_editors_for_slider_type_change_func = clipeffectseditor.refresh_clip
 
+    propertyeditorbuilder.show_rotomask_func = rotomask.show_rotomask
+    
+    multitrimmode.set_default_mode_func = modesetting.set_default_edit_mode
+    
+    keyframeeditor._get_current_edited_compositor = compositeeditor.get_compositor
+
+    # Not callbacks but tlinerender needs this data and we do this instead of copypaste.
+    tlinerender._get_frame_for_x_func = tlinewidgets.get_frame
+    tlinerender._get_x_for_frame_func = tlinewidgets._get_frame_x
+    tlinerender._get_last_tline_view_frame_func = tlinewidgets.get_last_tline_view_frame
+
+    #keyframeeditor.add_fade_out_func = compositeeditor._add_fade_out_pressed
     # These provide clues for further module refactoring 
 
 # ---------------------------------- SDL2 consumer
 #def create_sdl_2_consumer():
-#    GObject.source_remove(sdl2_timeout_id)
+#    GObject.source_remove(disk_cache_timeout_id)
 #    print "Creating SDL2 consumer..."
 #    editorstate.PLAYER().create_sdl2_video_consumer()
 
@@ -439,24 +460,28 @@ def get_assoc_file_path():
         return arg_str
 
 def open_assoc_file():
-    GObject.source_remove(assoc_timeout_id)
+    GLib.source_remove(assoc_timeout_id)
     projectaction.actually_load_project(assoc_file_path, block_recent_files=False)
-    
+
+def set_quiet_if_requested():
+    for arg in sys.argv:
+        if arg == "--quiet":
+            global _log_file
+            _log_file = "/dev/null"
+            log_print_output_to_file()
+            
 def create_gui():
     """
     Called at app start to create gui objects and handles for them.
     """
-    tlinewidgets.load_icons()
+    tlinewidgets.load_icons_and_set_colors()
     kftoolmode.load_icons()
 
     updater.set_clip_edit_mode_callback = modesetting.set_clip_monitor_edit_mode
     updater.load_icons()
 
-    # Notebook indexes are differn for 1 and 2 window layouts
-    if editorpersistance.prefs.global_layout != appconsts.SINGLE_WINDOW:
-        medialog.range_log_notebook_index = 0
-        compositeeditor.compositor_notebook_index = 2
-        clipeffectseditor.filters_notebook_index = 1
+    # Make layout data available
+    editorlayout.init_layout_data()
 
     # Create window and all child components
     editor_window = editorwindow.EditorWindow()
@@ -464,6 +489,9 @@ def create_gui():
     # Make references to various gui components available via gui module
     gui.capture_references(editor_window)
 
+    # Unused frames take 3 pixels so hide those.
+    editorlayout.set_positions_frames_visibility()
+        
     # All widgets are now realized and references captured so can find out theme colors
     gui.set_theme_colors()
     tlinewidgets.set_dark_bg_color()
@@ -534,7 +562,7 @@ def init_project_gui():
     projectinfogui.update_project_info()
 
     titler.reset_titler()
-
+    
     # Set render folder selector to last render if prefs require 
     folder_path = editorstate.PROJECT().get_last_render_folder()
     if folder_path != None and editorpersistance.prefs.remember_last_render_dir == True:
@@ -545,8 +573,21 @@ def init_sequence_gui():
     Called after project load or changing current sequence 
     to initialize interface.
     """
+    # Set correct compositing mode menu item selected
+    gui.editor_window.init_compositing_mode_menu()
+    gui.editor_window.init_timeline_rendering_menu()
+    gui.editor_window.tline_render_mode_launcher.set_pixbuf(editorstate.tline_render_mode) 
+    gui.comp_mode_launcher.set_pixbuf(editorstate.get_compositing_mode())
+
     # Set initial timeline scale draw params
     editorstate.current_sequence().update_length()
+    
+    # Handle timeline rendering GUI and data
+    tlinerender.init_for_sequence()
+    gui.editor_window.hide_tline_render_strip()
+    if editorstate.get_tline_rendering_mode() != appconsts.TLINE_RENDERING_OFF: 
+        gui.editor_window.show_tline_render_strip()
+        
     updater.update_pix_per_frame_full_view()
     updater.init_tline_scale()
     updater.repaint_tline()
@@ -573,11 +614,10 @@ def init_editor_state():
 
     # Clear editors 
     clipeffectseditor.clear_clip()
-    clipeffectseditor.effect_selection_changed() # to get No Clip text
     compositeeditor.clear_compositor()
 
     # Show first pages on notebooks
-    gui.middle_notebook.set_current_page(0)
+    gui.editor_window.notebook.set_current_page(0)
 
     # Clear clip selection.
     movemodes.clear_selection_values()
@@ -613,8 +653,8 @@ def open_project(new_project):
 
     editorstate.project = new_project
     editorstate.media_view_filter = appconsts.SHOW_ALL_FILES
-    editorstate.auto_follow = editorstate.project.get_project_property(appconsts.P_PROP_AUTO_FOLLOW)
-
+    editorstate.tline_render_mode = appconsts.TLINE_RENDERING_OFF
+    
     # Inits widgets with project data
     init_project_gui()
     
@@ -633,7 +673,7 @@ def open_project(new_project):
     # Delete autosave file after it has been loaded
     global loaded_autosave_file
     if loaded_autosave_file != None:
-        print "Deleting", loaded_autosave_file
+        print("Deleting", loaded_autosave_file)
         os.remove(loaded_autosave_file)
         loaded_autosave_file = None
 
@@ -643,6 +683,7 @@ def open_project(new_project):
     editorstate.clear_trim_clip_cache()
     audiomonitoring.init_for_project_load()
 
+    tlinerender.init_session()
     start_autosave()
 
     if new_project.update_media_lengths_on_load == True:
@@ -659,14 +700,23 @@ def open_project(new_project):
     global resize_timeout_id
     resize_timeout_id = GLib.timeout_add(500, _do_window_resized_update)
 
+    projectaction.clear_changed_since_last_save_flags()
+
+    # Set scrubbing
+    editorstate.player.set_scrubbing(editorpersistance.prefs.audio_scrubbing)
+    
 def _do_window_resized_update():
-    GObject.source_remove(resize_timeout_id)
+    GLib.source_remove(resize_timeout_id)
     updater.window_resized()
     
 def change_current_sequence(index):
+    edit.do_gui_update = False  # This should not be necessery but we are doing this signal intention that GUI updates are disabled
+    
     stop_autosave()
     editorstate.project.c_seq = editorstate.project.sequences[index]
 
+    editorstate.tline_render_mode = appconsts.TLINE_RENDERING_OFF
+
     # Inits widgets with current sequence data
     init_sequence_gui()
     
@@ -685,6 +735,8 @@ def change_current_sequence(index):
     selected_index = editorstate.project.sequences.index(editorstate.current_sequence())
     selection.select_path(str(selected_index))
 
+    audiomonitoring.recreate_master_meter_filter_for_new_sequence()
+    
     start_autosave()
 
     updater.set_timeline_height()
@@ -709,12 +761,13 @@ def autosave_recovery_dialog():
 
 def autosave_dialog_callback(dialog, response):
     dialog.destroy()
-    autosave_file = utils.get_hidden_user_dir_path() + AUTOSAVE_DIR + get_autosave_files()[0]
+    autosave_file = userfolders.get_cache_dir() + AUTOSAVE_DIR + get_autosave_files()[0]
     if response == Gtk.ResponseType.OK:
         global loaded_autosave_file
         loaded_autosave_file = autosave_file
-        projectaction.actually_load_project(autosave_file, True)
+        projectaction.actually_load_project(autosave_file, True, False, True)
     else:
+        tlinerender.init_session()  # didn't do this in main and not going to do app-open_project
         os.remove(autosave_file)
         start_autosave()
 
@@ -723,7 +776,7 @@ def autosaves_many_recovery_dialog():
     now = time.time()
     autosaves = []
     for a_file_name in autosaves_file_names:
-        autosave_path = utils.get_hidden_user_dir_path() + AUTOSAVE_DIR + a_file_name
+        autosave_path = userfolders.get_cache_dir() + AUTOSAVE_DIR + a_file_name
         autosave_object = utils.EmptyClass()
         autosave_object.age = now - os.stat(autosave_path).st_mtime
         autosave_object.path = autosave_path
@@ -736,45 +789,57 @@ def autosaves_many_recovery_dialog():
 def autosaves_many_dialog_callback(dialog, response, autosaves_view, autosaves):
     if response == Gtk.ResponseType.OK:
         autosave_file = autosaves[autosaves_view.get_selected_indexes_list()[0]].path # Single selection, 1 quaranteed to exist
-        print "autosave_file", autosave_file
+        print("autosave_file", autosave_file)
         global loaded_autosave_file
         loaded_autosave_file = autosave_file
         dialog.destroy()
-        projectaction.actually_load_project(autosave_file, True)
+        projectaction.actually_load_project(autosave_file, True, False, True)
     else:
         dialog.destroy()
+        tlinerender.init_session()
         start_autosave()
 
 def set_instance_autosave_id():
     global instance_autosave_id_str
-    instance_autosave_id_str = "_" + md5.new(str(os.urandom(32))).hexdigest()
-
+    instance_autosave_id_str = "_" + hashlib.md5(str(os.urandom(32)).encode('utf-8')).hexdigest()
+    
 def get_instance_autosave_file():
     return AUTOSAVE_FILE + instance_autosave_id_str
 
 def start_autosave():
     global autosave_timeout_id
-    time_min = 1 # hard coded, probably no need to make configurable
+
+    # Aug-2019 - SvdB - AS - Made changes to use the value stored in prefs, with Default=1 minute, rather than hardcoding
+    try:
+        time_min, desc = editorpersistance.prefs.AUTO_SAVE_OPTS[editorpersistance.prefs.auto_save_delay_value_index]
+    except:
+    	  time_min = 1
+
     autosave_delay_millis = time_min * 60 * 1000
 
-    print "Autosave started..."
-    autosave_timeout_id = GObject.timeout_add(autosave_delay_millis, do_autosave)
-    autosave_file = utils.get_hidden_user_dir_path() + get_instance_autosave_file()
-    persistance.save_project(editorstate.PROJECT(), autosave_file)
+    # Aug-2019 - SvdB - AS - put in code to stop or not start autosave depending on user selection
+    if autosave_delay_millis > 0:
+        print("Autosave started...")
+        autosave_timeout_id = GObject.timeout_add(autosave_delay_millis, do_autosave)
+        autosave_file = userfolders.get_cache_dir() + get_instance_autosave_file()
+        persistance.save_project(editorstate.PROJECT(), autosave_file)
+    else:
+        print("Autosave disabled...")
+        stop_autosave()
 
 def get_autosave_files():
-    autosave_dir = utils.get_hidden_user_dir_path() + AUTOSAVE_DIR
+    autosave_dir = userfolders.get_cache_dir() + AUTOSAVE_DIR
     return os.listdir(autosave_dir)
 
 def stop_autosave():
     global autosave_timeout_id
     if autosave_timeout_id == -1:
         return
-    GObject.source_remove(autosave_timeout_id)
+    GLib.source_remove(autosave_timeout_id)
     autosave_timeout_id = -1
 
 def do_autosave():
-    autosave_file = utils.get_hidden_user_dir_path() + get_instance_autosave_file()
+    autosave_file = userfolders.get_cache_dir() + get_instance_autosave_file()
     persistance.save_project(editorstate.PROJECT(), autosave_file)
     return True
 
@@ -798,7 +863,7 @@ def show_splash_screen():
 
 def destroy_splash_screen():
     splash_screen.destroy()
-    GObject.source_remove(splash_timeout_id)
+    GLib.source_remove(splash_timeout_id)
 
 def show_worflow_info_dialog():
     editorpersistance.prefs.workflow_dialog_last_version_shown = editorstate.appversion
@@ -806,8 +871,69 @@ def show_worflow_info_dialog():
     
     worflow_info_dialog = workflow.WorkflowDialog()
     return False
+
+# ------------------------------------------------------- disk cahce size check
+def check_disk_cache_size():
+    GLib.source_remove(disk_cache_timeout_id)
+    diskcachemanagement.check_disk_cache_size()
     
-# ------------------------------------------------------- small screens
+# ------------------------------------------------------- userfolders dialogs
+def show_user_folders_init_error_dialog(error_msg):
+    # not done
+    print(error_msg, " user folder XDG init error")
+    return False
+
+def show_user_folders_copy_dialog():
+    dialog = dialogs.xdg_copy_dialog()
+    copy_thread = userfolders.XDGCopyThread(dialog, _xdg_copy_completed_callback)
+    copy_thread.start()
+    return False
+
+def _xdg_copy_completed_callback(dialog):
+    Gdk.threads_enter()
+    dialog.destroy()
+    Gdk.threads_leave()
+
+# ------------------------------------------------------- small and multiple screens
+# We need a bit more stuff because having multiple monitors can mix up setting draw parameters.
+def _set_screen_size_data():
+    monitor_data = utils.get_display_monitors_size_data()
+    monitor_data_index = editorpersistance.prefs.layout_display_index
+
+    display = Gdk.Display.get_default()
+    num_monitors = display.get_n_monitors() # Get number of monitors.
+    if monitor_data_index == 0:
+        scr_w = Gdk.Screen.width()
+        scr_h = Gdk.Screen.height()
+        print("Using Full Screen size for layout:", scr_w, "x", scr_h)
+    elif monitor_data_index > num_monitors:
+        print("Specified layout monitor not present.")
+        scr_w = Gdk.Screen.width()
+        scr_h = Gdk.Screen.height()
+        print("Using Full Screen size for layout:", scr_w, "x", scr_h)
+        editorpersistance.prefs.layout_display_index = 0
+    else:
+
+        scr_w, scr_h = monitor_data[monitor_data_index]
+        if scr_w < 1151 or scr_h < 767:
+            print("Selected layout monitor too small.")
+            scr_w = Gdk.Screen.width()
+            scr_h = Gdk.Screen.height()
+            print("Using Full Screen size for layout:", scr_w, "x", scr_h)
+            editorpersistance.prefs.layout_display_index = 0
+        else:
+            # Selected monitor data is available and monitor is usable as layout monitor.
+            print("Using monitor " + str(monitor_data_index) + " for layout: " + str(scr_w) + " x " + str(scr_h))
+    
+    editorstate.SCREEN_WIDTH = scr_w
+    editorstate.SCREEN_HEIGHT = scr_h
+    
+    print("Small height:", editorstate.screen_size_small_height())
+    print("Small width:",  editorstate.screen_size_small_width())
+
+    return (scr_w, scr_h)
+
+# Adjust gui parameters for smaller screens
 def _set_draw_params():
     if editorstate.screen_size_small_width() == True:
         appconsts.NOTEBOOK_WIDTH = 400
@@ -842,7 +968,7 @@ def _too_small_screen_exit():
     Gtk.main()
 
 def _show_too_small_info():
-    GObject.source_remove(exit_timeout_id)
+    GLib.source_remove(exit_timeout_id)
     primary_txt = _("Too small screen for this application.")
     scr_w = Gdk.Screen.width()
     scr_h = Gdk.Screen.height()
@@ -850,6 +976,29 @@ def _show_too_small_info():
                     _("Your screen dimensions are ") + str(scr_w) + " x " + str(scr_h) + "."
     dialogutils.warning_message_with_callback(primary_txt, secondary_txt, None, False, _early_exit)
 
+def _xdg_error_exit(error_str):
+    global exit_timeout_id
+    exit_timeout_id = GObject.timeout_add(200, _show_xdg_error_info, error_str)
+    # Launch gtk+ main loop
+    Gtk.main()
+
+def _show_xdg_error_info(error_str):
+    GLib.source_remove(exit_timeout_id)
+    primary_txt = _("Cannot launch application because XDG folders init error.")
+    secondary_txt = error_str + "."
+    dialogutils.warning_message_with_callback(primary_txt, secondary_txt, None, False, _early_exit)
+
+def _too_low_mlt_version_exit():
+    global exit_timeout_id
+    exit_timeout_id = GObject.timeout_add(200, _show_mlt_version_exit_info)
+    # Launch gtk+ main loop
+    Gtk.main()
+
+def _show_mlt_version_exit_info():
+    primary_txt = _("Flowblade version 2.6 (or later) requires MLT version 6.18 to run")
+    secondary_txt = _("Your MLT version is: ") + editorstate.mlt_version + ".\n\n" + _("Install MLT 6.18 or higher to run Flowblade.")
+    dialogutils.warning_message_with_callback(primary_txt, secondary_txt, None, False, _early_exit)
+
 def _early_exit(dialog, response):
     dialog.destroy()
     # Exit gtk main loop.
@@ -857,38 +1006,45 @@ def _early_exit(dialog, response):
 
 # ------------------------------------------------------- logging
 def log_print_output_to_file():
-    so = se = open(_log_file, 'w', 0)
+    so = se = open(_log_file, 'w', buffering=1)
 
-    sys.stdout = os.fdopen(sys.stdout.fileno(), 'w', 0)
+    sys.stdout = os.fdopen(sys.stdout.fileno(), 'w', buffering=1)
 
     os.dup2(so.fileno(), sys.stdout.fileno())
     os.dup2(se.fileno(), sys.stderr.fileno())
 
 # ------------------------------------------------------ shutdown
 def shutdown():
-    dialogs.exit_confirm_dialog(_shutdown_dialog_callback, get_save_time_msg(), gui.editor_window.window, editorstate.PROJECT().name)
-    return True # Signal that event is handled, otherwise it'll destroy window anyway
+    if projectaction.was_edited_since_last_save() == False:
+        _shutdown_dialog_callback(None, None, True)
+        return True
+    else:
+        dialogs.exit_confirm_dialog(_shutdown_dialog_callback, get_save_time_msg(), gui.editor_window.window, editorstate.PROJECT().name)
+        return True # Signal that event is handled, otherwise it'll destroy window anyway
 
 def get_save_time_msg():
     return projectaction.get_save_time_msg()
 
-def _shutdown_dialog_callback(dialog, response_id):
-    dialog.destroy()
-    if response_id == Gtk.ResponseType.CLOSE:# "Don't Save"
-        pass
-    elif response_id ==  Gtk.ResponseType.YES:# "Save"
-        if editorstate.PROJECT().last_save_path != None:
-            persistance.save_project(editorstate.PROJECT(), editorstate.PROJECT().last_save_path)
-        else:
-            dialogutils.warning_message(_("Project has not been saved previously"), 
-                                    _("Save project with File -> Save As before closing."),
-                                    gui.editor_window.window)
+def _shutdown_dialog_callback(dialog, response_id, no_dialog_shutdown=False):
+    if no_dialog_shutdown == False:
+        dialog.destroy()
+        if response_id == Gtk.ResponseType.CLOSE:# "Don't Save"
+            pass
+        elif response_id ==  Gtk.ResponseType.YES:# "Save"
+            if editorstate.PROJECT().last_save_path != None:
+                persistance.save_project(editorstate.PROJECT(), editorstate.PROJECT().last_save_path)
+            else:
+                dialogutils.warning_message(_("Project has not been saved previously"), 
+                                        _("Save project with File -> Save As before closing."),
+                                        gui.editor_window.window)
+                return
+        else: # "Cancel"
             return
-    else: # "Cancel"
-        return
+    else:
+        print("Nothing changed since last save.")
 
     # --- APP SHUT DOWN --- #
-    print "Exiting app..."
+    print("Exiting app...")
     # Sep-2018 - SvdB - Stop wave form threads
     for thread_termination in threading.enumerate():
         # We only terminate threads with a 'process', as these are launched
@@ -901,6 +1057,12 @@ def _shutdown_dialog_callback(dialog, response_id):
     # No more auto saving
     stop_autosave()
 
+    tlinerender.delete_session()
+
+    clipeffectseditor.shutdown_polling()
+    compositeeditor.shutdown_polling()
+
+    
     # Save window dimensions on exit
     alloc = gui.editor_window.window.get_allocation()
     x, y, w, h = alloc.x, alloc.y, alloc.width, alloc.height 
@@ -912,7 +1074,7 @@ def _shutdown_dialog_callback(dialog, response_id):
     editorpersistance.prefs.app_v_paned_position = gui.editor_window.app_v_paned.get_position()
     editorpersistance.prefs.top_paned_position = gui.editor_window.top_paned.get_position()
     try: # This fails if preference for top row layout changed, we just ignore saving these values then.
-        if editorwindow.top_level_project_panel() == True:
+        if editorlayout.top_level_project_panel() == True:
             editorpersistance.prefs.mm_paned_position = 200  # This is not used until user sets preference to not have top level project panel
         else:
             editorpersistance.prefs.mm_paned_position = gui.editor_window.mm_paned.get_position()
@@ -940,9 +1102,16 @@ def _app_destroy():
         pass
     # Delete autosave file
     try:
-        os.remove(utils.get_hidden_user_dir_path() + get_instance_autosave_file())
+        os.remove(userfolders.get_cache_dir() + get_instance_autosave_file())
     except:
-        print "Delete autosave file FAILED"
+        print("Delete autosave file FAILED!")
 
-    # Exit gtk main loop.
-    Gtk.main_quit()
+    do_gtk_main_quit = jobs.handle_shutdown(get_instance_autosave_file())
+    
+    # Exit gtk main loop if no jobs unfinished.
+    if do_gtk_main_quit == True:
+        Gtk.main_quit()
+    else:
+        # Jobs lauches its own top level window to show progress on unfinished jobs renders
+        # and does Gtk.main_quit() later when done.
+        pass
