@@ -74,8 +74,7 @@ TRACK_HEIGHT_SMALLEST = 25 # maybe remove this as it is no longer used
 TLINE_HEIGHT = 260
 
 # Notebook widths
-NOTEBOOK_WIDTH = 600 # defines app min width together with MONITOR_AREA_WIDTH
-NOTEBOOK_WIDTH_WIDESCREEN = 500
+NOTEBOOK_WIDTH = 400 # currently has no effect, panels widgets size requests combined width > 400 (about 488), make bigger if needed.
 TOP_ROW_HEIGHT = 500
 
 # Property editing gui consts
@@ -169,6 +168,7 @@ CONTAINER_CLIPS_UNRENDERED = "container_clips/unrendered"
 CC_CLIP_FRAMES_DIR = "/clip_frames"
 CC_RENDERED_FRAMES_DIR = "/rendered_frames"
 CC_PREVIEW_RENDER_DIR = "/preview_frames"
+USER_SHORTCUTS_DIR =  "user_shortcuts/"
 
 # Luma bands
 SHADOWS = 0
@@ -201,6 +201,7 @@ TRIM_VIEW_OFF = 2
 MIDBAR_TC_LEFT = 0
 MIDBAR_TC_CENTER = 1
 MIDBAR_COMPONENTS_CENTERED = 2
+MIDBAR_TC_FREE = 3
 
 # Windows mode
 SINGLE_WINDOW = 1
@@ -252,6 +253,8 @@ MONITOR_CLIP_BUTTON_PRESSED = 2
 FLOWBLADE_THEME = 0
 DARK_THEME = 1
 LIGHT_THEME = 2
+FLOWBLADE_THEME_GRAY = 3
+FLOWBLADE_THEME_NEUTRAL = 4
 
 # DND actions
 DND_ALWAYS_OVERWRITE = 0
@@ -262,6 +265,10 @@ DND_ALWAYS_INSERT = 2
 THREE_PANELS_IF_POSSIBLE = 0
 ALWAYS_TWO_PANELS = 1
 
+# Tool selection optiokns
+TOOL_SELECTOR_IS_MENU = 0
+TOOL_SELECTOR_IS_LEFT_DOCK = 1
+
 # Copypaste data tyoe
 COPY_PASTE_DATA_CLIPS = 1
 COPY_PASTE_DATA_COMPOSITOR_PROPERTIES = 2
@@ -287,7 +294,6 @@ PROXY_SIZE_FULL = 0
 PROXY_SIZE_HALF = 1
 PROXY_SIZE_QUARTER = 2
 
-
 # Container clip types
 CONTAINER_CLIP_GMIC = 0
 CONTAINER_CLIP_MLT_XML = 1
@@ -295,3 +301,37 @@ CONTAINER_CLIP_CAIRO_SCRIPT = 2
 CONTAINER_CLIP_BLENDER = 3
 
 CONTAINER_CLIP_VIDEO_CLIP_NAME = "container_clip"
+
+# Middlebar button groups.
+BUTTON_GROUP_TOOLS = "tool_buttons"
+BUTTON_GROUP_UNDO = "undo_redo"
+BUTTON_GROUP_ZOOM = "zoom_buttons"
+BUTTON_GROUP_EDIT = "edit_buttons"
+BUTTON_GROUP_SYNC_SPLIT = "edit_buttons_2"
+BUTTON_GROUP_DELETE = "edit_buttons_3"
+BUTTON_GROUP_MONITOR_ADD = "monitor_insert_buttons"
+BIG_TIME_CODE = "big_TC"
+WORKFLOW_LAUNCH = "worflow_launch" 
+TOOL_SELECT = "tool_selector"
+
+# Panel placement options
+PANEL_PLACEMENT_TOP_ROW_DEFAULT = 0
+PANEL_PLACEMENT_TOP_ROW_RIGHT = 1
+PANEL_PLACEMENT_LEFT_COLUMN = 2
+PANEL_PLACEMENT_BOTTOM_ROW_LEFT = 3
+PANEL_PLACEMENT_BOTTOM_ROW_RIGHT = 4
+PANEL_PLACEMENT_TOP_ROW_PROJECT_DEFAULT = 5
+PANEL_PLACEMENT_NOT_VISIBLE = 6
+PANEL_PLACEMENT_TWO_WINDOWS_MEDIA_PANEL_POS = 7
+
+# Panels
+PANEL_MEDIA = 0
+PANEL_FILTERS = 1
+PANEL_COMPOSITORS = 2
+PANEL_RANGE_LOG = 3
+PANEL_RENDERING = 4
+PANEL_JOBS = 5
+PANEL_PROJECT = 6
+PANEL_PROJECT_SMALL_SCREEN = 7
+PANEL_MEDIA_AND_BINS_SMALL_SCREEN = 8
+PANEL_FILTER_SELECT = 9
\ No newline at end of file
