@@ -38,6 +38,7 @@ import audiosync
 import appconsts
 import clipeffectseditor
 import compositeeditor
+import containerclip
 import dialogs
 import dialogutils
 import gui
@@ -56,6 +57,7 @@ import syncsplitevent
 import tlinewidgets
 import tlineaction
 import updater
+import userfolders
 import utils
 
 _match_frame_writer = None
@@ -110,7 +112,15 @@ def _compositor_menu_item_activated(widget, data):
     elif action_id == "set auto follow":
         compositor.obey_autofollow = widget.get_active()
         updater.repaint_tline()
-        
+
+def _delete_compositors(data):
+    clip, track, item_id, x = data
+    compositors = current_sequence().get_clip_compositors(clip)
+    for compositor in compositors:
+        data = {"compositor":compositor}
+        action = edit.delete_compositor_action(data)
+        action.do_edit()
+    
 def _open_clip_in_effects_editor(data):
     updater.open_clip_in_effects_editor(data)
     
@@ -201,12 +211,15 @@ def _add_filter(data):
     clip, track, item_id, item_data = data
     x, filter_info = item_data
     action = clipeffectseditor.get_filter_add_action(filter_info, clip)
+    clipeffectseditor.set_stack_update_blocked() # We update stack on set_clip below
     action.do_edit()
-    
+    clipeffectseditor.set_stack_update_unblocked()
+
     # (re)open clip in editor
     frame = tlinewidgets.get_frame(x)
     index = track.get_clip_index_at(frame)
     clipeffectseditor.set_clip(clip, track, index)
+    clipeffectseditor.set_filter_item_expanded(len(clip.filters) - 1)
 
 def _add_compositor(data):
     clip, track, item_id, item_data = data
@@ -217,15 +230,21 @@ def _add_compositor(data):
 
     target_track_index = track.id - 1
 
+    if current_sequence().compositing_mode == appconsts.COMPOSITING_MODE_STANDARD_AUTO_FOLLOW:
+        target_track_index = current_sequence().first_video_index
+
     compositor_in = current_sequence().tracks[track.id].clip_start(clip_index)
     clip_length = clip.clip_out - clip.clip_in
     compositor_out = compositor_in + clip_length
 
+    a_track = target_track_index
+    b_track = track.id
+
     edit_data = {"origin_clip_id":clip.id,
                 "in_frame":compositor_in,
                 "out_frame":compositor_out,
-                "a_track":target_track_index,
-                "b_track":track.id,
+                "a_track":a_track,
+                "b_track":b_track,
                 "compositor_type":compositor_type,
                 "clip":clip}
     action = edit.add_compositor_action(edit_data)
@@ -234,6 +253,8 @@ def _add_compositor(data):
     updater.repaint_tline()
 
 def _add_autofade(data):
+    # NOTE: These stay synced only in "Top Down Auto Follow" mode, see: edit.get_full_compositor_sync_data()
+    
     clip, track, item_id, item_data = data
     x, compositor_type = item_data
 
@@ -246,7 +267,7 @@ def _add_autofade(data):
     if compositor_type == "##auto_fade_in":
         compositor_in = current_sequence().tracks[track.id].clip_start(clip_index)
         compositor_out = compositor_in + int(utils.fps()) - 1
-    else:
+    else: # fade out
         clip_start = current_sequence().tracks[track.id].clip_start(clip_index)
         compositor_out = clip_start + clip_length
         compositor_in = compositor_out - int(utils.fps()) + 1
@@ -513,7 +534,7 @@ class MatchFrameWriter:
         clip_path = self.clip.path
 
         # Create consumer
-        matchframe_new_path = utils.get_hidden_user_dir_path() + appconsts.MATCH_FRAME_NEW
+        matchframe_new_path = userfolders.get_cache_dir() + appconsts.MATCH_FRAME_NEW
         consumer = mlt.Consumer(PROJECT().profile, "avformat", matchframe_new_path)
         consumer.set("real_time", 0)
         consumer.set("vcodec", "png")
@@ -538,7 +559,7 @@ class MatchFrameWriter:
             time.sleep(0.1)
 
         # Copy to match frame
-        matchframe_path = utils.get_hidden_user_dir_path() + appconsts.MATCH_FRAME
+        matchframe_path = userfolders.get_cache_dir() + appconsts.MATCH_FRAME
         shutil.copyfile(matchframe_new_path, matchframe_path)
 
         # Update timeline data           
@@ -630,6 +651,26 @@ def _brightness_keyframes(data):
     clip, track, item_id, item_data = data
     modesetting.kftool_mode_from_popup_menu(clip, track, kftoolmode.BRIGHTNESS_KF_EDIT)
 
+def _reload_clip_media(data):
+    clip, track, item_id, item_data = data
+    
+    # TODO: This ain't doing the clip icon update as wished.
+    media_item = PROJECT().get_media_file_for_path(clip.path)
+    media_item.create_icon()
+    
+    clip_index = track.clips.index(clip)
+    new_clip = current_sequence().create_clone_clip(clip)
+    
+    data = {"old_clip":clip,
+            "new_clip":new_clip,
+            "track":track,
+            "index":clip_index}
+    action = edit.reload_replace(data)
+    action.do_edit()
+    
+    
+    
+    
 # Functions to handle popup menu selections for strings 
 # set as activation messages in guicomponents.py
 # activation_message -> _handler_func
@@ -674,4 +715,11 @@ POPUP_HANDLERS = {"set_master":syncsplitevent.init_select_master_clip,
                   "delete_clip_marker":_delete_clip_marker,
                   "deleteall_clip_markers":_delete_all_clip_markers,
                   "volumekf":_volume_keyframes,
-                  "brightnesskf":_brightness_keyframes}
+                  "brightnesskf":_brightness_keyframes,
+                  "delete_compositors":_delete_compositors,
+                  "reload_media":_reload_clip_media,
+                  "cc_render_full_media":containerclip.render_full_media,
+                  "cc_render_clip":containerclip.render_clip_length,
+                  "cc_go_to_underdered":containerclip.switch_to_unrendered_media,
+                  "cc_render_settings":containerclip.set_render_settings,
+                  "cc_edit_program":containerclip.edit_program}
