@@ -96,7 +96,7 @@ SCREEN_WIDTH = -1
 # Runtime environment data
 gtk_version = None
 mlt_version = None
-appversion = "2.6.0"
+appversion = "2.8.0"
 RUNNING_FROM_INSTALLATION = 0
 RUNNING_FROM_DEV_VERSION = 1
 RUNNING_FROM_FLATPAK = 2
@@ -126,6 +126,7 @@ display_clip_media_thumbnails = True
 
 # Flag for window being in fullscreen mode
 fullscreen = False
+fullscreen_second_window = False
 
 # Trim view mode
 show_trim_view = appconsts.TRIM_VIEW_OFF
@@ -141,6 +142,10 @@ tline_render_mode = appconsts.TLINE_RENDERING_OFF
 # Trim clips cache for quicker inits, path -> clip
 _trim_clips_cache = {}
 
+# Normal installs and Flatpaks have G'Mic bin in different locations.
+# Value set in gmic.py module. 
+gmic_path = None
+
 def current_is_move_mode():
     if ((edit_mode == INSERT_MOVE) or (edit_mode == OVERWRITE_MOVE) or (edit_mode == MULTI_MOVE)):
         return True
