@@ -26,10 +26,12 @@ from gi.repository import Pango
 import copy
 import os
 import subprocess
+import sys
 import time
 import threading
 
 import appconsts
+import editorlayout
 import editorpersistance
 from editorstate import PROJECT
 import gui
@@ -59,15 +61,11 @@ open_media_file_callback = None
 _status_polling_thread = None
 
 _hamburger_menu = Gtk.Menu()
-
 _jobs_list_view = None
 
 _jobs = [] # proxy objects that represent background renders and provide info on render status.
-
 _remove_list = [] # objects are removed from GUI with delay to give user time to notice copmpletion
 
-jobs_notebook_index = 4 # 4 for single window, app.py sets to 3 for two windows
-
 _jobs_render_progress_window = None
 
 
@@ -134,7 +132,7 @@ def add_job(job_proxy):
     _jobs.append(job_proxy)
     _jobs_list_view.fill_data_model()
     if editorpersistance.prefs.open_jobs_panel_on_add == True:
-        gui.middle_notebook.set_current_page(jobs_notebook_index)
+        editorlayout.show_panel(appconsts.PANEL_JOBS)
     
     if editorpersistance.prefs.render_jobs_sequentially == False: # Feature not active for first release 2.6.
         job_proxy.start_render()
@@ -244,6 +242,9 @@ def get_jobs_panel():
     panel.pack_start(row2, False, True, 0)
     panel.set_size_request(400, 10)
 
+    if editorpersistance.prefs.theme == appconsts.FLOWBLADE_THEME_GRAY:
+        panel.override_background_color(Gtk.StateFlags.NORMAL, gui.get_light_gray_light_color())
+            
     return panel
 
 
@@ -490,14 +491,14 @@ class MotionRenderJobQueueObject(AbstractJobQueueObject):
         job_msg.status = RENDERING
         update_job_queue(job_msg)
         
-        # Run with nice to lower priority if requested (currently hard coded to lower)
-        nice_command = "nice -n " + str(10) + " " + respaths.LAUNCH_DIR + "flowblademotionheadless"
+        # Create command list and launch process.
+        command_list = [sys.executable]
+        command_list.append(respaths.LAUNCH_DIR + "flowblademotionheadless")
         for arg in self.args:
-            nice_command += " "
-            nice_command += arg
-
-        subprocess.Popen([nice_command], shell=True)
+            command_list.append(arg)
 
+        subprocess.Popen(command_list)
+        
     def update_render_status(self):
 
         Gdk.threads_enter()
@@ -560,25 +561,23 @@ class ProxyRenderJobQueueObject(AbstractJobQueueObject):
         job_msg.status = RENDERING
         update_job_queue(job_msg)
         
-        # Run with nice to lower priority if requested (currently hard coded to lower)
-        nice_command = "nice -n " + str(10) + " " + respaths.LAUNCH_DIR + "flowbladeproxyheadless"
+        # Create command list and launch process.
+        command_list = [sys.executable]
+        command_list.append(respaths.LAUNCH_DIR + "flowbladeproxyheadless")
         args = self.render_data.get_data_as_args_tuple()
         for arg in args:
-            nice_command += " "
-            nice_command += arg
-
+            command_list.append(arg)
+            
         session_arg = "session_id:" + str(self.session_id)
-        nice_command += " "
-        nice_command += session_arg
-                
-        subprocess.Popen([nice_command], shell=True)
+        command_list.append(session_arg)
 
+        subprocess.Popen(command_list)
+    
     def update_render_status(self):
 
         Gdk.threads_enter()
                     
         if proxyheadless.session_render_complete(self.get_session_id()) == True:
-            #remove_as_status_polling_object(self)
             
             job_msg = self.get_completed_job_message()
             update_job_queue(job_msg)
