@@ -24,9 +24,11 @@ Module handles Keyframe tool functionality
 from gi.repository import Pango, PangoCairo, Gtk
 
 import cairo
+import math
 
 import appconsts
 import cairoarea
+import clipeffectseditor
 import dialogutils
 import edit
 from editorstate import current_sequence
@@ -64,15 +66,22 @@ KF_HIT_WIDTH = 8
 KF_DRAG_THRESHOLD = 3
 
 # Colors
-FRAME_SCALE_LINES = (0.07, 0.22, 0.07)
-FRAME_SCALE_LINES_BRIGHT = (0.2, 0.6, 0.2)
+FRAME_SCALE_LINES = (0.4, 0.4, 0.6) #(0.07, 0.07, 0.32)
+FRAME_SCALE_LINES_BRIGHT = (0.2, 0.2, 0.6)
 TEXT_COLOR = (0.6, 0.6, 0.6) 
-CURVE_COLOR = (0.71, 0.13, 0.64)
+CURVE_COLOR = (0.97, 0.97, 0.30, 1)#(0.71, 0.13, 0.64, 1.0) # (0.19, 0.69, 0.15, 1) #
 OVERLAY_BG = (0.0, 0.0, 0.0, 0.8)
+VALUE_AREA_COLOR = (0.27, 0.27, 0.62, 0.85)
+SOURCE_TRIANGLE_COLOR = (0.19, 0.32, 0.57)
+SOURCE_TRIANGLE_OUTLINE_COLOR = (0.9, 0.9, 0.9)
+SCALE_LINES_TEXT_COLOR = (0.9, 0.9, 0.9)
+CLIP_OUTLINE_COLOR = (0.7, 0.7, 0.5, 0.22)
+AUDIO_LEVELS_COLOR = (0.4, 0.4, 0.68, 0.20)
 
 # Edit types
 VOLUME_KF_EDIT = 0
 BRIGHTNESS_KF_EDIT = 1
+PARAM_KF_EDIT = 2
 
 # Editor states
 KF_DRAG = 0
@@ -85,22 +94,31 @@ DRAG_MIN_Y = 4 # to make strt value slightly macnetic, makes easier to move posi
 hamburger_menu = Gtk.Menu()
 oor_before_menu = Gtk.Menu()
 oor_after_menu = Gtk.Menu()
-
+value_snapping_menu = Gtk.Menu()
+params_menu =  Gtk.Menu()
+ 
 edit_data = None
 enter_mode = None
 _kf_editor = None
 
 _playhead_follow_kf = True
 
+_snapping = 1
+
 # -------------------------------------------------- init
 def load_icons():
     global HAMBURGER_ICON, ACTIVE_KF_ICON, NON_ACTIVE_KF_ICON
 
-    HAMBURGER_ICON = cairo.ImageSurface.create_from_png(respaths.IMAGE_PATH + "hamburger.png")
-    ACTIVE_KF_ICON = cairo.ImageSurface.create_from_png(respaths.IMAGE_PATH + "kf_active.png")
-    NON_ACTIVE_KF_ICON = cairo.ImageSurface.create_from_png(respaths.IMAGE_PATH + "kf_not_active_tool.png")    
+    # Aug-2019 - SvdB - BB
+    HAMBURGER_ICON = guiutils.get_cairo_image("hamburger")
+    ACTIVE_KF_ICON = guiutils.get_cairo_image("kf_active")
+    NON_ACTIVE_KF_ICON = guiutils.get_cairo_image("kf_not_active_tool")    
+
+def init_tool_for_clip(clip, track, edit_type=VOLUME_KF_EDIT, param_data=None):
+    # These can produce data for same objects and we are not (currently) updating
+    # clipeffectseditor/kftool with events from each other.
+    clipeffectseditor.clear_clip()
 
-def init_tool_for_clip(clip, track, edit_type=VOLUME_KF_EDIT):
     clip_index = track.clips.index(clip)
 
     # Save data needed to do the keyframe edits.
@@ -112,13 +130,18 @@ def init_tool_for_clip(clip, track, edit_type=VOLUME_KF_EDIT):
                  "track":track,
                  "initializing":True}
 
-    # Always brightness keyframes for media types that contain no audio.
-    if edit_data["clip"].media_type != appconsts.VIDEO and edit_data["clip"].media_type != appconsts.AUDIO:
-        edit_type = BRIGHTNESS_KF_EDIT
-    
-    # Volume keyframes on audio track for video and audio
-    if track.type == appconsts.AUDIO and not(edit_data["clip"].media_type != appconsts.VIDEO and edit_data["clip"].media_type != appconsts.AUDIO):
-        edit_type = VOLUME_KF_EDIT
+    if edit_type == PARAM_KF_EDIT:
+        pass # We are not trying to decide based on track what to edit
+    else:
+        # Always brightness keyframes for media types that contain no audio.
+        if edit_data["clip"].media_type != appconsts.VIDEO and edit_data["clip"].media_type != appconsts.AUDIO:
+            edit_type = BRIGHTNESS_KF_EDIT
+        
+        # Volume keyframes on audio track for video and audio
+        if track.type == appconsts.AUDIO and not(edit_data["clip"].media_type != appconsts.VIDEO and edit_data["clip"].media_type != appconsts.AUDIO):
+            edit_type = VOLUME_KF_EDIT
+
+    global _kf_editor
         
     # Init for edit type
     if edit_type == VOLUME_KF_EDIT:
@@ -128,12 +151,12 @@ def init_tool_for_clip(clip, track, edit_type=VOLUME_KF_EDIT):
             data = {"clip":clip, 
                     "filter_info":filter_info,
                     "filter_edit_done_func":_filter_create_dummy_func}
-            action = edit.add_multipart_filter_action(data)
+            action = edit.add_filter_action(data)
             action.do_edit()
             ep = _get_volume_editable_property(clip, track, clip_index)
 
         edit_data["editable_property"] = ep
-        global _kf_editor
+
         _kf_editor = TLineKeyFrameEditor(ep, True, VOLUME_KF_EDIT)
         
     elif edit_type == BRIGHTNESS_KF_EDIT:
@@ -147,11 +170,31 @@ def init_tool_for_clip(clip, track, edit_type=VOLUME_KF_EDIT):
             action = edit.add_filter_action(data)
             action.do_edit()
             ep = _get_brightness_editable_property(clip, track, clip_index)
-            
+
         edit_data["editable_property"] = ep
-        global _kf_editor
+
         _kf_editor = TLineKeyFrameEditor(ep, True, BRIGHTNESS_KF_EDIT)
         
+    else: #  edit_type == PARAM_KF_EDIT 
+        property_name, filter_object, filter_index, disp_name = param_data
+        ep = _get_param_editable_property(property_name, clip, track, clip_index, filter_object, filter_index)
+
+        # create kf in frame 0 if value PROP_INT or PROP_FLOAT and kf expression
+        eq_index = ep.value.find("=")
+        if eq_index == -1:
+            new_value = "0=" + ep.value
+            ep.value = new_value
+            ep.write_filter_object_property(new_value)
+
+        # Turn into keyframe property
+        ep = ep.get_as_KeyFrameHCSFilterProperty()
+        
+        edit_data["editable_property"] = ep
+
+        filter_param_name = filter_object.info.name + ":" + disp_name
+
+        _kf_editor = TLineKeyFrameEditor(ep, True, PARAM_KF_EDIT, filter_param_name)
+    
     tlinewidgets.set_edit_mode_data(edit_data)
     updater.repaint_tline()
 
@@ -161,12 +204,31 @@ def update_clip_frame(tline_frame):
         _kf_editor.set_and_display_clip_frame(clip_frame)
 
 def _get_volume_editable_property(clip, track, clip_index):
-    return _get_multipart_keyframe_ep_from_service(clip, track, clip_index, "volume")
-
+    return _get_param_editable_property_with_filter_search("volume", "level", clip, track, clip_index)
+    
 def _get_brightness_editable_property(clip, track, clip_index):
+    return _get_param_editable_property_with_filter_search("brightness", "level", clip, track, clip_index)
+
+def _get_param_editable_property(property_name, clip, track, clip_index, filter_object, filter_index):
+    editable_properties = propertyedit.get_filter_editable_properties(
+                                                   clip, 
+                                                   filter_object,
+                                                   filter_index,
+                                                   track,
+                                                   clip_index)
+    for ep in editable_properties:          
+        try:
+            if ep.name == property_name:
+                return ep
+        except:
+            pass
+                    
+    return None
+
+def _get_param_editable_property_with_filter_search(mlt_service_id, property_name, clip, track, clip_index):
     for i in range(0, len(clip.filters)):
         filter_object = clip.filters[i]
-        if filter_object.info.mlt_service_id == "brightness":
+        if filter_object.info.mlt_service_id == mlt_service_id:
             editable_properties = propertyedit.get_filter_editable_properties(
                                                            clip, 
                                                            filter_object,
@@ -174,13 +236,12 @@ def _get_brightness_editable_property(clip, track, clip_index):
                                                            track,
                                                            clip_index)
             for ep in editable_properties:          
-                # Volume is one of these MLT multipart filters, so we chose this way to find the editable property in filter.
                 try:
-                    if ep.name == "level":
+                    if ep.name == property_name:
                         return ep
                 except:
                     pass
-                    
+        
     return None
     
 def _get_multipart_keyframe_ep_from_service(clip, track, clip_index, mlt_service_id):
@@ -202,7 +263,14 @@ def _get_multipart_keyframe_ep_from_service(clip, track, clip_index, mlt_service
                     
     return None
 
+def _has_deprecated_volume_filter(clip):
+    for i in range(0, len(clip.filters)):
+        filter_object = clip.filters[i]
+        if filter_object.info.multipart_filter == True and filter_object.info.mlt_service_id == "volume":
+            return True 
 
+    return False
+            
 def exit_tool():
     set_no_clip_edit_data()
     global enter_mode
@@ -250,6 +318,14 @@ def mouse_press(event, frame):
 
     clip = track.clips[clip_index]
 
+
+    if _has_deprecated_volume_filter(clip) == True:
+        set_no_clip_edit_data()
+        primary_txt = _("This Clip has a deprecated Volume filter and cannot be edited with Keyframe Tool!")
+        secondary_txt = _("Flowblade 2.8 changed to use Volume filtes with dB values.\n\nOld style Volume filters will continue to function but they need be replaced\nto edit this Clip with Keyframe Tool.")
+        dialogutils.info_message(primary_txt, secondary_txt, gui.editor_window.window)
+        return
+
     init_tool_for_clip(clip, track)
 
 def _handle_edit_mouse_press(event):
@@ -303,7 +379,7 @@ def _tline_overlay(cr):
     
     # Get y position for clip's track
     ty_bottom = tlinewidgets._get_track_y(1) + current_sequence().tracks[1].height
-    ty_top = tlinewidgets._get_track_y(len(current_sequence().tracks) - 2) - 6 # -6 is hand correction, no idea why the math isn't getting correct pos top most track
+    ty_top = tlinewidgets._get_track_y(len(current_sequence().tracks) - 2) - 6 # -6 is hand correction, no idea why the math isn't getting correct pos for top most track
     ty_top_bottom_edge = ty_top + EDIT_AREA_HEIGHT
     off_step = float(ty_bottom - ty_top_bottom_edge) / float(len(current_sequence().tracks) - 2)
     ty_off = off_step * float(track.id - 1)
@@ -319,7 +395,7 @@ def _tline_overlay(cr):
 # ----------------------------------------------------- editor object
 class TLineKeyFrameEditor:
 
-    def __init__(self, editable_property, use_clip_in, edit_type):
+    def __init__(self, editable_property, use_clip_in, edit_type, filter_param_name=None):
         
         self.clip_length = editable_property.get_clip_length() - 1
         self.edit_type = edit_type
@@ -350,6 +426,7 @@ class TLineKeyFrameEditor:
         self.media_frame_txt = _("Media Frame: ")
         self.volume_kfs_text = _("Volume Keyframes")
         self.brightness_kfs_text = _("Brightness Keyframes")
+        self.filter_param_name_txt = filter_param_name
         
         self.current_mouse_action = None
         self.drag_min = -1
@@ -382,9 +459,9 @@ class TLineKeyFrameEditor:
 
     # ------------------------------------------------------- debug
     def print_keyframes(self):
-        print "clip edit keyframes:"
+        print("clip edit keyframes:")
         for i in range(0, len(self.keyframes)):
-            print self.keyframes[i]
+            print(self.keyframes[i])
             
     # ----------------------------------------------------------------- Draw
     def draw(self, cr):
@@ -405,19 +482,89 @@ class TLineKeyFrameEditor:
         cr.set_source_surface(HAMBURGER_ICON, x + 4.5, y + 4)
         cr.paint()
 
+        # Draw clip out line
+        clip = edit_data["clip"]
+        track = edit_data["track"]
+
+        pix_per_frame = tlinewidgets.pix_per_frame
+        pos = tlinewidgets.pos
+        
+        clip_start_in_tline = edit_data["clip_start_in_timeline"]
+        clip_start_frame = clip_start_in_tline - pos
+    
+        clip_in = clip.clip_in
+        clip_out = clip.clip_out
+        clip_length = clip_out - clip_in + 1 # +1 because in and out both inclusive
+        scale_length = clip_length * pix_per_frame
+        scale_in = clip_start_frame * pix_per_frame
+        
+        clip_outline_y = tlinewidgets._get_track_y(track.id)
+         
+        self.create_round_rect_path(cr, scale_in,
+                                     clip_outline_y, scale_length, 
+                                     track.height)
+        cr.set_source_rgba(*CLIP_OUTLINE_COLOR)
+        cr.set_line_width(2.0)
+        cr.fill()
+            
         # Frame scale and value lines
         self.frame_scale.draw(cr, edit_data["clip_start_in_timeline"], self.clip_length, self._get_upper_y(), self._get_lower_y())
         self._draw_value_lines(cr, x, w)
 
         kf_positions = self.get_clip_kfs_and_positions()
 
-        # Draw value curves,they need to be clipped into edit area
-        cr.set_source_rgb(*CURVE_COLOR)
-        cr.set_line_width(1.0)
-
+        # Draw value curves, area fill and audio levels,they need to be clipped into edit area
         cr.save()
-        cr.set_line_width(2.0)
+
         ex, ey, ew, eh = self._get_edit_area_rect()
+        
+        # Maybe draw audio levels
+        if self.edit_type == VOLUME_KF_EDIT and clip.is_blanck_clip == False and clip.waveform_data != None:
+
+            cr.set_source_rgba(*AUDIO_LEVELS_COLOR)
+        
+            y_pad = TOP_PAD
+            bar_height = eh
+            
+            # Draw all frames only if pixels per frame > 2, otherwise
+            # draw only every other or fewer frames
+            draw_pix_per_frame = tlinewidgets.pix_per_frame
+            if draw_pix_per_frame < 2:
+                draw_pix_per_frame = 2
+                step = int(2 / pix_per_frame)
+                if step < 1:
+                    step = 1
+            else:
+                step = 1
+
+            # Draw only frames in display
+            draw_first = clip_in
+            draw_last = clip_out + 1
+            if clip_start_frame < 0:
+                draw_first = int(draw_first - clip_start_frame)
+
+            # Get media frame 0 position in screen pixels
+            media_start_pos_pix = scale_in - clip_in * pix_per_frame
+            mid_y = y + y_pad + eh / 2.0
+            
+            # Draw level bar for each frame in draw range
+            for f in range(draw_first, draw_last, step):
+                try:
+                    xf = media_start_pos_pix + f * pix_per_frame
+                    hf = bar_height * clip.waveform_data[f] * 0.5
+                    if h < 1:
+                        h = 1
+                    cr.rectangle(xf, mid_y - hf, draw_pix_per_frame, hf * 2.0)
+                except:
+                    # This is just dirty fix a when 23.98 fps does not work
+                    break
+
+            cr.fill()
+
+        # Draw value curve and area fill
+        cr.set_source_rgba(*CURVE_COLOR)
+        cr.set_line_width(3.0)
+
         cr.rectangle(ex, ey, ew, eh)
         cr.clip() 
         for i in range(0, len(kf_positions)):
@@ -431,14 +578,16 @@ class TLineKeyFrameEditor:
                 cr.move_to(kf_pos_x, kf_pos_y)
             else:
                 cr.line_to(kf_pos_x, kf_pos_y)
-        cr.stroke()
         # If last kf before clip end, continue value curve to end
         kf, frame, kf_index, kf_pos_x, kf_pos_y = kf_positions[-1]
         if kf_pos_x < ex + ew:
             cr.move_to(kf_pos_x, kf_pos_y)
             cr.line_to(ex + ew, kf_pos_y)
-            cr.stroke()
-
+        else:
+            cr.line_to(ex + ew, kf_pos_y)
+    
+        cr.stroke()
+        
         cr.restore()
 
         # Draw keyframes
@@ -474,12 +623,25 @@ class TLineKeyFrameEditor:
                 self._draw_text(cr, str(after_kfs), x + w - OUT_OF_RANGE_NUMBER_X_END_PAD, kfy + KF_TEXT_PAD)
         
         # Draw source triangle
-        cr.move_to(x - 6, self.source_track_center - 8)
-        cr.line_to(x + 3, self.source_track_center)
-        cr.line_to(x - 6, self.source_track_center + 8)
+        cr.set_line_width(2.0)
+        cr.move_to(x - 8, self.source_track_center - 8)
+        cr.line_to(x + 1, self.source_track_center)
+        cr.line_to(x - 8, self.source_track_center + 8)
         cr.close_path()
-        cr.set_source_rgb(*TEXT_COLOR)
-        cr.fill()
+        cr.set_source_rgb(*SOURCE_TRIANGLE_COLOR)
+        cr.fill_preserve()
+        cr.set_source_rgb(*SOURCE_TRIANGLE_OUTLINE_COLOR)
+        cr.stroke()
+
+        cr.move_to(x + w + 8, self.source_track_center - 8)
+        cr.line_to(x + w - 1, self.source_track_center)
+        cr.line_to(x + w + 8, self.source_track_center + 8)
+        cr.close_path()
+        cr.set_source_rgb(*SOURCE_TRIANGLE_COLOR)
+        cr.set_source_rgb(*SOURCE_TRIANGLE_COLOR)
+        cr.fill_preserve()
+        cr.set_source_rgb(*SOURCE_TRIANGLE_OUTLINE_COLOR)
+        cr.stroke()
         
         # Draw frame pointer
         try:
@@ -496,11 +658,16 @@ class TLineKeyFrameEditor:
         if w > 55: # dont draw on too small editors
             if self.edit_type == VOLUME_KF_EDIT:
                 text = edit_data["editable_property"].clip.name + " - " + self.volume_kfs_text
-            else:
+            elif self.edit_type == BRIGHTNESS_KF_EDIT:
                 text = edit_data["editable_property"].clip.name + " - " + self.brightness_kfs_text
+            else: # PARAM_KF_EDIT
+                text = edit_data["editable_property"].clip.name + " - " + self.filter_param_name_txt
             self._draw_text(cr, text, -1, y + 4, True, x, w)
             self._draw_text(cr, self.media_frame_txt + str(self.current_clip_frame), -1, kfy - 8, True, x, w)
 
+        # Value texts
+        self._draw_value_texts(cr, x, w)
+     
         # Value value info
         if self.edit_value != None:
             self._draw_value_text_box(cr, self.mouse_x,self.mouse_y, str(self.edit_value))
@@ -512,6 +679,81 @@ class TLineKeyFrameEditor:
         cr.stroke()
 
     def _draw_value_lines(self, cr, x, w):
+        # Audio hard coded value lines
+        active_width = w - 2 * END_PAD
+        xs = x + END_PAD
+        xe = xs + active_width
+
+        if self.edit_type == VOLUME_KF_EDIT:
+            # 0
+            y = self._get_panel_y_for_value(0.0)
+            cr.set_line_width(1.0)
+            cr.set_source_rgb(*FRAME_SCALE_LINES)
+            cr.move_to(xs, y)
+            cr.line_to(xe, y)
+            cr.stroke()
+            
+            # -20
+            y = self._get_panel_y_for_value(-20.0)
+            cr.set_source_rgb(*FRAME_SCALE_LINES)
+            cr.move_to(xs, y)
+            cr.line_to(xe, y)
+            cr.stroke()
+
+        elif self.edit_type == BRIGHTNESS_KF_EDIT:
+            # 0
+            y = self._get_panel_y_for_value(0.0)
+            cr.set_line_width(1.0)
+            cr.set_source_rgb(*FRAME_SCALE_LINES)
+            cr.move_to(xs, y)
+            cr.line_to(xe, y)
+            cr.stroke()
+
+            # 50
+            y = self._get_panel_y_for_value(50)
+            cr.set_line_width(1.0)
+            cr.set_source_rgb(*FRAME_SCALE_LINES)
+            cr.move_to(xs, y)
+            cr.line_to(xe, y)
+            cr.stroke()
+            
+            # 100
+            y = self._get_panel_y_for_value(100) 
+            cr.set_source_rgb(*FRAME_SCALE_LINES)
+            cr.move_to(xs, y)
+            cr.line_to(xe, y)
+            cr.stroke()
+            
+        else:
+            editable_property = edit_data["editable_property"] 
+            adjustment = editable_property.get_input_range_adjustment()
+            lower = adjustment.get_lower()
+            upper = adjustment.get_upper()
+            half = (upper - lower) / 2 + lower
+            
+            # Min
+            y = self._get_panel_y_for_value(lower)
+            cr.set_line_width(1.0)
+            cr.set_source_rgb(*FRAME_SCALE_LINES)
+            cr.move_to(xs, y)
+            cr.line_to(xe, y)
+            cr.stroke()
+
+            # Half
+            y = self._get_panel_y_for_value(half)
+            cr.set_source_rgb(*FRAME_SCALE_LINES)
+            cr.move_to(xs, y)
+            cr.line_to(xe, y)
+            cr.stroke()
+            
+            # Max
+            y = self._get_panel_y_for_value(upper)
+            cr.set_source_rgb(*FRAME_SCALE_LINES)
+            cr.move_to(xs, y)
+            cr.line_to(xe, y)
+            cr.stroke()
+
+    def _draw_value_texts(self, cr, x, w):
         # Audio hard coded value lines
         TEXT_X_OFF = 4
         TEXT_X_OFF_END = -28
@@ -525,62 +767,41 @@ class TLineKeyFrameEditor:
                               cairo.FONT_SLANT_NORMAL,
                               cairo.FONT_WEIGHT_NORMAL)
         cr.set_font_size(12)
-
-        # TODO: Do this in a more general way if we ever use this tool to edit other than "volume" and "brightness".
+        cr.set_source_rgb(*SCALE_LINES_TEXT_COLOR)
+            
         if self.edit_type == VOLUME_KF_EDIT:
             # 0
             y = self._get_panel_y_for_value(0.0)
-            cr.set_line_width(1.0)
-            cr.set_source_rgb(*FRAME_SCALE_LINES)
-            cr.move_to(xs, y)
-            cr.line_to(xe, y)
-            cr.stroke()
             
-            cr.set_source_rgb(*FRAME_SCALE_LINES_BRIGHT)
-            text = "0"
+            text = "0 dB"
             cr.move_to(xs + TEXT_X_OFF, y - TEXT_Y_OFF)
             cr.show_text(text)
             cr.move_to(xe + TEXT_X_OFF_END + 16, y - TEXT_Y_OFF)
             cr.show_text(text)
 
-            # 50
-            y = self._get_panel_y_for_value(50)
-            cr.set_source_rgb(*FRAME_SCALE_LINES)
-            cr.move_to(xs, y)
-            cr.line_to(xe, y)
-            cr.stroke()
+            # -20
+            y = self._get_panel_y_for_value(-20.0)
 
-            cr.set_source_rgb(*FRAME_SCALE_LINES_BRIGHT)
-            text = "50"
+            text = "-20 dB"
             cr.move_to(xs + TEXT_X_OFF, y - TEXT_Y_OFF + 8)
             cr.show_text(text)
             cr.move_to(xe + TEXT_X_OFF_END + 6, y - TEXT_Y_OFF + 8)
             cr.show_text(text)
             
-            # 100
-            y = self._get_panel_y_for_value(100)
-            cr.set_source_rgb(*FRAME_SCALE_LINES)
-            cr.move_to(xs, y)
-            cr.line_to(xe, y)
-            cr.stroke()
+        
+            # -70
+            y = self._get_panel_y_for_value(-70.0)
             
-            cr.set_source_rgb(*FRAME_SCALE_LINES_BRIGHT)
-            text = "100"
-            cr.move_to(xs + TEXT_X_OFF, y - TEXT_Y_OFF + 17)
+            text = "-70 dB"
+            cr.move_to(xs + TEXT_X_OFF, y - TEXT_Y_OFF)
             cr.show_text(text)
-            cr.move_to(xe + TEXT_X_OFF_END, y - TEXT_Y_OFF + 17)
+            cr.move_to(xe + TEXT_X_OFF_END, y - TEXT_Y_OFF)
             cr.show_text(text)
+
             
         elif self.edit_type == BRIGHTNESS_KF_EDIT:
             # 0
             y = self._get_panel_y_for_value(0.0)
-            cr.set_line_width(1.0)
-            cr.set_source_rgb(*FRAME_SCALE_LINES)
-            cr.move_to(xs, y)
-            cr.line_to(xe, y)
-            cr.stroke()
-            
-            cr.set_source_rgb(*FRAME_SCALE_LINES_BRIGHT)
             text = "0"
             cr.move_to(xs + TEXT_X_OFF, y - TEXT_Y_OFF)
             cr.show_text(text)
@@ -588,15 +809,8 @@ class TLineKeyFrameEditor:
             cr.show_text(text)
 
             # 50
-            #XOFF_END_50 = -5
             y = self._get_panel_y_for_value(50)
-            cr.set_line_width(1.0)
-            cr.set_source_rgb(*FRAME_SCALE_LINES)
-            cr.move_to(xs, y)
-            cr.line_to(xe, y)
-            cr.stroke()
             
-            cr.set_source_rgb(*FRAME_SCALE_LINES_BRIGHT)
             text = "50"
             cr.move_to(xs + TEXT_X_OFF, y + 4)
             cr.show_text(text)
@@ -605,17 +819,46 @@ class TLineKeyFrameEditor:
             
             # 100
             y = self._get_panel_y_for_value(100) 
-            cr.set_source_rgb(*FRAME_SCALE_LINES)
-            cr.move_to(xs, y)
-            cr.line_to(xe, y)
-            cr.stroke()
             
-            cr.set_source_rgb(*FRAME_SCALE_LINES_BRIGHT)
             text = "100"
             cr.move_to(xs + TEXT_X_OFF, y + 13)
             cr.show_text(text)
             cr.move_to(xe + TEXT_X_OFF_END, y + 13)
             cr.show_text(text)
+        else:
+
+            editable_property = edit_data["editable_property"] 
+            adjustment = editable_property.get_input_range_adjustment()
+            lower = adjustment.get_lower()
+            upper = adjustment.get_upper()
+            half = (upper - lower) / 2 + lower
+            
+            # Min
+            y = self._get_panel_y_for_value(lower)
+            
+            text = str(lower)
+            cr.move_to(xs + TEXT_X_OFF, y - TEXT_Y_OFF)
+            cr.show_text(text)
+            cr.move_to(xe + TEXT_X_OFF_END + 16, y - TEXT_Y_OFF)
+            cr.show_text(text)
+
+            # Half
+            y = self._get_panel_y_for_value(half)
+
+            text = str(half)
+            cr.move_to(xs + TEXT_X_OFF, y - TEXT_Y_OFF + 8)
+            cr.show_text(text)
+            cr.move_to(xe + TEXT_X_OFF_END + 6, y - TEXT_Y_OFF + 8)
+            cr.show_text(text)
+            
+            # Max
+            y = self._get_panel_y_for_value(upper)
+            
+            text = str(upper)
+            cr.move_to(xs + TEXT_X_OFF, y - TEXT_Y_OFF + 17)
+            cr.show_text(text)
+            cr.move_to(xe + TEXT_X_OFF_END, y - TEXT_Y_OFF + 17)
+            cr.show_text(text)
             
     def _draw_text(self, cr, txt, x, y, centered=False, tline_x=-1, w=-1):
         layout = PangoCairo.create_layout(cr)
@@ -667,6 +910,16 @@ class TLineKeyFrameEditor:
         cr.set_source_rgb(0.8, 0.8, 0.8)
         cr.show_text(text) 
 
+    def create_round_rect_path(self, cr, x, y, width, height, radius=4.0):
+        degrees = math.pi / 180.0
+
+        cr.new_sub_path()
+        cr.arc(x + width - radius, y + radius, radius, -90 * degrees, 0 * degrees)
+        cr.arc(x + width - radius, y + height - radius, radius, 0 * degrees, 90 * degrees)
+        cr.arc(x + radius, y + height - radius, radius, 90 * degrees, 180 * degrees)
+        cr.arc(x + radius, y + radius, radius, 180 * degrees, 270 * degrees)
+        cr.close_path()
+        
     # ----------------------------------------------------------- mouse events
     def press_event(self, event):
         """
@@ -694,7 +947,7 @@ class TLineKeyFrameEditor:
         self.mouse_x = lx
         self.mouse_y = ly
 
-        if event.button == 3:
+        if event.button == 3: # right mouse
             self.current_mouse_action = POSITION_DRAG
             
             self.drag_min = self.clip_in
@@ -758,6 +1011,10 @@ class TLineKeyFrameEditor:
                 frame = 0
             if self.value_drag_on == True:
                 value = round(self._get_value_for_panel_y(ly))
+                if _snapping == 2:
+                    value = round(value / 2) * 2
+                elif  _snapping == 5:
+                    value = round(value / 5.0) * 5
                 self.edit_value = value
                 self.set_active_kf_frame_and_value(frame, value)
             else:
@@ -792,6 +1049,10 @@ class TLineKeyFrameEditor:
                 frame = 0
             if self.value_drag_on == True:
                 value = round(self._get_value_for_panel_y(ly))
+                if _snapping == 2:
+                    value = round(value / 2) * 2
+                elif  _snapping == 5:
+                    value = round(value / 5) * 5
                 self.set_active_kf_frame_and_value(frame, value)
                 self.hack_fix_for_zero_one_keyframe_problem()
             else:
@@ -857,7 +1118,7 @@ class TLineKeyFrameEditor:
                 self.keyframes.insert(i, (frame, value))
                 self.active_kf_index = i
                 return
-        #prev_frame, prev_value = self.keyframes[len(self.keyframes) - 1]
+
         self.keyframes.append((frame, value))
         self.active_kf_index = len(self.keyframes) - 1
 
@@ -870,7 +1131,8 @@ class TLineKeyFrameEditor:
         if self.active_kf_index < 0:
             self.active_kf_index = 0
         self._set_pos_to_active_kf()
-    
+        self.update_property_value()
+        
         updater.repaint_tline()
 
     def set_and_display_clip_frame(self, clip_frame):
@@ -1102,7 +1364,6 @@ class TLineKeyFrameEditor:
         if len(self.keyframes) > 1 and before_kfs > 0:
             menu.add(self._get_menu_item(_("Set Keyframe at Frame 0 to value of next Keyframe"), self._oor_menu_item_activated, "zero_next" ))
 
-
     def _show_oor_after_menu(self, widget, event):
         menu = oor_before_menu
         self._build_oor_after_menu(menu)
@@ -1144,6 +1405,45 @@ class TLineKeyFrameEditor:
                     edit_brightness.set_sensitive(False)
                 menu.add(edit_brightness)
 
+            editable_params_exist = False
+            params_menu_item = Gtk.MenuItem(_("Edit Other Filter Parameters"))
+            params_menu = Gtk.Menu()
+            guiutils.remove_children(params_menu)
+            for i in range(0, len(edit_data["clip"].filters)):
+                filt = edit_data["clip"].filters[i]
+                for prop in filt.properties:
+                    p_name, p_val, p_type = prop
+                    args_str = filt.info.property_args[p_name]
+
+                    args_dict = propertyparse.args_string_to_args_dict(args_str)
+                    try:
+                        editor = args_dict["editor"]
+                    except:
+                        editor = "slider"
+                        
+                    try:
+                        disp_name = args_dict["displayname"]
+                    except:
+                        disp_name = p_name
+
+                    try:
+                        range_in = args_dict["range_in"]
+                    except:
+                        range_in = None
+
+                    if (editor == "slider" or editor == "keyframe_editor") and range_in != None:
+                        param_data = (p_name, filt, i, disp_name)
+                        editable_params_exist = True
+                        item_text = filt.info.name  + ": " +  disp_name
+                        param_item = self._get_menu_item(item_text, self._param_edit_item_activated, param_data)
+                        params_menu.add(param_item)
+                        
+            if editable_params_exist == False:
+                params_menu_item.set_sensitive(False)
+            params_menu_item.set_submenu(params_menu)
+            params_menu_item.show_all()
+            menu.add(params_menu_item)
+        
             sep = Gtk.SeparatorMenuItem()
             sep.show()
             menu.add(sep)
@@ -1172,16 +1472,48 @@ class TLineKeyFrameEditor:
         playhead_follow_item.connect("activate", self._oor_menu_item_activated, "playhead_follows")
         playhead_follow_item.show()
         menu.add(playhead_follow_item)
+
+        value_snapping_item = Gtk.MenuItem(_("Value Snapping"))
+        value_snapping_menu = Gtk.Menu()
+        guiutils.remove_children(value_snapping_menu)
+
+        first_item = Gtk.RadioMenuItem()
+        first_item.set_label("1")
+        if _snapping == 1:
+            first_item.set_active(True)
+        first_item.show()
+        value_snapping_menu.append(first_item)
+        first_item.connect("activate", self._oor_menu_item_activated, "1")
+
+        radio_item = Gtk.RadioMenuItem.new_with_label([first_item], "2")
+        if _snapping == 2:
+            radio_item.set_active(True)
+        radio_item.connect("activate", self._oor_menu_item_activated, "2")
+        value_snapping_menu.append(radio_item)
+        radio_item.show()
+
+        radio_item = Gtk.RadioMenuItem.new_with_label([first_item], "5")
+        if _snapping == 5:
+            radio_item.set_active(True)
+        radio_item.connect("activate", self._oor_menu_item_activated, "5")
+        value_snapping_menu.append(radio_item)
+        radio_item.show()
+
+        value_snapping_item.set_submenu(value_snapping_menu)
+        value_snapping_item.show_all()
+        menu.add(value_snapping_item)
         
         sep = Gtk.SeparatorMenuItem()
         sep.show()
         menu.add(sep)
         
         menu.add(self._get_menu_item(_("Exit Edit"), self._oor_menu_item_activated, "exit" ))
-
+        
         menu.popup(None, None, None, None, event.button, event.time)
 
     def _oor_menu_item_activated(self, widget, data):
+        global _snapping
+        
         if data == "delete_all_before":
             keep_doing = True
             while keep_doing:
@@ -1238,10 +1570,18 @@ class TLineKeyFrameEditor:
         elif data == "playhead_follows":
             global _playhead_follow_kf
             _playhead_follow_kf = widget.get_active()
-            
-            
+        elif data == "1":
+            _snapping = 1
+        elif data == "2":
+            _snapping = 2
+        elif data == "5":
+            _snapping = 5
+
         updater.repaint_tline()
-        
+
+    def _param_edit_item_activated(self,  widget, data):
+        init_tool_for_clip(edit_data["clip"],  edit_data["track"], PARAM_KF_EDIT, data)
+
     def _get_menu_item(self, text, callback, data):
         item = Gtk.MenuItem(text)
         item.connect("activate", callback, data)
