@@ -22,7 +22,6 @@
 Module contains an object that is used to do playback from mlt.Producers to
 a Xwindow of a GTK+ widget and os audiosystem using a SDL consumer.
 """
-
 from gi.repository import Gdk
 
 import mlt
@@ -31,6 +30,7 @@ import time
 
 import gui
 from editorstate import timeline_visible
+import editorpersistance
 import utils
 import updater
 
@@ -49,7 +49,7 @@ class Player:
     def init_for_profile(self, profile):
         # Get profile and create ticker for playback GUI updates
         self.profile = profile
-        print "Player initialized with profile: ", self.profile.description()
+        print("Player initialized with profile: ", self.profile.description())
         
         # Trim loop preview
         self.loop_start = -1
@@ -60,7 +60,7 @@ class Player:
         self.is_rendering = False
         self.render_stop_frame = -1
         self.render_start_frame = -1
-        self.render_callbacks = None
+        #self.render_callbacks = None
         self.wait_for_producer_end_stop = True
         self.render_gui_update_count = 0
 
@@ -76,7 +76,7 @@ class Player:
         #    print "refuse SDL1 consumer"
         #    return
 
-        print "Create SDL1 consumer..."
+        print("Create SDL1 consumer...")
         # Create consumer and set params
         self.consumer = mlt.Consumer(self.profile, "sdl")
         self.consumer.set("real_time", 1)
@@ -84,6 +84,7 @@ class Player:
         self.consumer.set("resize", 1)
         self.consumer.set("progressive", 1)
 
+
         # Hold ref to switch back from rendering
         self.sdl_consumer = self.consumer 
 
@@ -111,6 +112,12 @@ class Player:
         self.connect_and_start()
     """
     
+    def set_scrubbing(self, scrubbing_active):
+        if scrubbing_active == True:
+            self.consumer.set("scrub_audio", 1)
+        else:
+            self.consumer.set("scrub_audio", 0)
+            
     def set_sdl_xwindow(self, widget):
         """
         Connects SDL output to display widget's xwindow
@@ -266,17 +273,15 @@ class Player:
         self.producer.seek(frame) 
 
         # GUI update path starts here.
-        # All user or program initiated seeks go through this method.
         if update_gui:
             updater.update_frame_displayers(frame)
             
         frame = self.producer.get_frame()
-        # And make sure we deinterlace if input is interlaced
+        # And make sure we deinterlace if input is interlaced.
         frame.set("consumer_deinterlace", 1)
 
-        # Now we are ready to get the image and save it.
-        size = (self.profile.width(), self.profile.height())
-        rgb = frame.get_image(mlt.mlt_image_rgb24a, *size) 
+        # Now we are ready to get the image and save it.        
+        rgb = frame.get_image(int(mlt.mlt_image_rgb24a), int(self.profile.width()), int(self.profile.height()))
         return rgb
 
     def display_inside_sequence_length(self, new_seq_len):
@@ -287,12 +292,21 @@ class Player:
         return (self.producer.get_speed() != 0)
 
     def _ticker_event(self):
-            
+        
+        current_frame = self.producer.frame()
+        
+        loop_clips = editorpersistance.prefs.loop_clips
+        if loop_clips and current_frame >= self.get_active_length() and timeline_visible() == False: # Looping for clips
+            self.seek_frame(0, False) #NOTE: False==GUI not updated
+            self.producer.set_speed(1)
+            Gdk.threads_enter()
+            updater.update_frame_displayers(current_frame)
+            Gdk.threads_leave()
+            return
+
         # Stop ticker if playback has stopped.
         if (self.consumer.is_stopped() or self.producer.get_speed() == 0):
             self.ticker.stop_ticker()
-
-        current_frame = self.producer.frame()
         
         # Stop rendering if last frame reached.
         if self.is_rendering == True and current_frame >= self.render_stop_frame:
@@ -309,9 +323,6 @@ class Player:
             self.render_gui_update_count = self.render_gui_update_count + 1
             if self.render_gui_update_count % 8 == 0: # we need quick updates for stop accuracy, but slower gui updating
                 self.render_gui_update_count = 1
-                Gdk.threads_enter()
-                self.render_callbacks.set_render_progress_gui(render_fraction)
-                Gdk.threads_leave()
             return 
 
         # If we're out of active range seek end.
@@ -327,13 +338,17 @@ class Player:
             or (current_frame >= self.get_active_length()))):
             self.seek_frame(self.loop_start, False) #NOTE: False==GUI not updated
             self.producer.set_speed(1)
-        
+
+        # Frame displayers update
         Gdk.threads_enter()
-        # If prefs set and frame out tline view, move tline view
-        range_moved = updater.maybe_move_playback_tline_range(current_frame) # range_moved given just to avoid two updates
-        if range_moved == False:
-            # Just display tline
+        if timeline_visible() == False:
             updater.update_frame_displayers(current_frame)
+        else:
+            # If prefs set and frame out tline view, move tline view
+            range_moved = updater.maybe_move_playback_tline_range(current_frame) # range_moved flag returned just to avoid two updates
+            if range_moved == False:
+                # Just display tline
+                updater.update_frame_displayers(current_frame)
         Gdk.threads_leave()
         
     def get_active_length(self):
@@ -349,17 +364,7 @@ class Player:
             return float(self.producer.frame()) / float(self.producer.get_length() - 1)
         else:
             return float(self.producer.frame() - self.render_start_frame) / float(self.render_stop_frame - self.render_start_frame)
-    
-    def set_render_callbacks(self, callbacks):
-        # Callbacks object interface:
-        #
-        # callbacks = utils.EmptyClass()
-        # callbacks.set_render_progress_gui(fraction)
-        # callbacks.save_render_start_time()
-        # callbacks.exit_render_gui()
-        # callbacks.maybe_open_rendered_file_in_bin()
-        self.render_callbacks = callbacks
-
+   
     def start_rendering(self, render_consumer, start_frame=0, stop_frame=-1):
         if stop_frame == -1:
             stop_frame = self.producer.get_length() - 1
@@ -369,7 +374,7 @@ class Player:
         else:
             self.wait_for_producer_end_stop = False
                 
-        print "start_rendering(), start frame :" + str(start_frame) + ", stop_frame: " + str(stop_frame)
+        print("start_rendering(), start frame :" + str(start_frame) + ", stop_frame: " + str(stop_frame))
         self.ticker.stop_ticker()
         self.consumer.stop()
         self.producer.set_speed(0)
@@ -382,11 +387,11 @@ class Player:
         self.consumer.start()
         self.producer.set_speed(1)
         self.is_rendering = True
-        self.render_callbacks.save_render_start_time()
+        #self.render_callbacks.save_render_start_time()
         self.ticker.start_ticker(RENDER_TICKER_DELAY)
 
     def stop_rendering(self):
-        print "stop_rendering, producer frame: " + str(self.producer.frame())
+        print("stop_rendering, producer frame: " + str(self.producer.frame()))
         # Stop render
         # This method of stopping makes sure that whole producer is rendered and written to disk
         if self.wait_for_producer_end_stop:
@@ -416,46 +421,6 @@ class Player:
         Gdk.threads_leave()
         self.seek_frame(0)
 
-        Gdk.threads_enter()
-        self.render_callbacks.exit_render_gui()
-        self.render_callbacks.maybe_open_rendered_file_in_bin()
-        Gdk.threads_leave()
-
-    """
-    def jack_output_on(self):
-        # We're assuming that we are not rendering and consumer is SDL consumer
-        self.producer.set_speed(0)
-        self.ticker.stop_ticker()
-
-        self.consumer.stop()
-
-        self.create_sdl_consumer()
-
-        self.jack_output_filter = mlt.Filter(self.profile, "jackrack")
-        if editorpersistance.prefs.jack_output_type == appconsts.JACK_OUT_AUDIO:
-            self.jack_output_filter.set("out_1", "system:playback_1")
-            self.jack_output_filter.set("out_2", "system:playback_2")
-        self.consumer.attach(self.jack_output_filter)
-        self.consumer.set("audio_off", "1")
-        self.consumer.set("frequency", str(editorpersistance.prefs.jack_frequency))
-
-        self.consumer.connect(self.producer)
-        self.consumer.start()
-
-    def jack_output_off(self):
-        # We're assuming that we are not rendering and consumer is SDL consumer
-        self.producer.set_speed(0)
-        self.ticker.stop_ticker()
-
-        self.consumer.detach(self.jack_output_filter)
-        self.consumer.set("audio_off", "0")
-
-        self.consumer.stop()
-        self.consumer.start()
-        
-        self.jack_output_filter = None
-    """
-
     def shutdown(self):
         self.ticker.stop_ticker()
         self.producer.set_speed(0)
