@@ -23,14 +23,16 @@ MLT framework profiles.
 """
 import os
 import mlt
+import xml.dom.minidom
 
+import appconsts
 import editorpersistance
 import respaths
-import utils
+import userfolders
 
 # Inside hidden user folder
-USER_PROFILES_DIR = "user_profiles/"
-DEFAULT_DEFAULT_PROFILE = "DV/DVD PAL"
+USER_PROFILES_DIR = appconsts.USER_PROFILES_DIR
+DEFAULT_DEFAULT_PROFILE = "HD 1080p 30 fps"
 
 # List of mlt profiles
 _profile_list = []
@@ -45,16 +47,16 @@ def load_profile_list():
     """
     global _profile_list,_factory_profiles, _hidden_factory_profiles, _user_profiles, _proxy_profiles
 
-    user_profiles_dir = utils.get_hidden_user_dir_path() + USER_PROFILES_DIR
+    user_profiles_dir = userfolders.get_data_dir() + USER_PROFILES_DIR
     _user_profiles = _load_profiles_list(user_profiles_dir)
     _load_factory_profiles()
 
     _profile_list = _factory_profiles + _user_profiles
 
-    _profile_list.sort(_sort_profiles)
-    _factory_profiles.sort(_sort_profiles)
-    _hidden_factory_profiles.sort(_sort_profiles)
-    _user_profiles.sort(_sort_profiles)
+    _profile_list.sort(key=_sort_profiles)
+    _factory_profiles.sort(key=_sort_profiles)
+    _hidden_factory_profiles.sort(key=_sort_profiles)
+    _user_profiles.sort(key=_sort_profiles)
 
 def _load_profiles_list(dir_path):
     load_profiles = []
@@ -119,6 +121,14 @@ def get_profile_for_index(index):
 def get_profile_name_for_index(index):
     profile_name, profile = _profile_list[index]
     return profile_name
+
+def get_profile_index_for_profile(test_profile):
+    for i in range(0, len(_profile_list)):
+        fname, profile = _profile_list[i]
+        if profile.description() == test_profile.description():
+            return i
+    
+    return -1 # not found
     
 def get_default_profile():
     return get_profile_for_index(get_default_profile_index())
@@ -129,13 +139,13 @@ def get_default_profile_index():
     """
     def_profile_index = get_index_for_name(editorpersistance.prefs.default_profile_name)
     if def_profile_index == -1:
-        print "default profile from prefs not found"
+        print("default profile from prefs not found")
         def_profile_index = get_index_for_name(DEFAULT_DEFAULT_PROFILE)
         def_profile_name =  DEFAULT_DEFAULT_PROFILE
         if def_profile_index == -1:
             def_profile_index = 0
             def_profile_name, profile = _profile_list[def_profile_index]
-            print "DEFAULT_DEFAULT_PROFILE deleted returning first profile"
+            print("DEFAULT_DEFAULT_PROFILE deleted returning first profile")
         editorpersistance.prefs.default_profile_name = def_profile_name
         editorpersistance.save()
     return def_profile_index
@@ -167,6 +177,45 @@ def get_profile_node(profile):
 
     return node_str
 
+def is_mlt_xml_profile_match_to_profile(mlt_xml_path, profile):
+    mlt_xml_doc = xml.dom.minidom.parse(mlt_xml_path)
+    try:
+        profile_node = mlt_xml_doc.getElementsByTagName("profile")[0]
+    except:
+        print("no profile node")
+        return (False, "Unknown")
+    
+    match = True
+    if profile_node.getAttribute("description") != profile.description():
+        match = False
+    if profile_node.getAttribute("width") != str(profile.width()):
+        match = False
+    if profile_node.getAttribute("height") != str(profile.height()):
+        match = False
+    if profile.progressive() == True:
+        prog_val = "1"
+    else:
+        prog_val = "0"
+    if profile_node.getAttribute("progressive") != prog_val:
+        match = False
+    if profile_node.getAttribute("sample_aspect_num") != str(profile.sample_aspect_num()):
+        match = False
+    if profile_node.getAttribute("sample_aspect_den") != str(profile.sample_aspect_den()):
+        match = False
+    if profile_node.getAttribute("display_aspect_num") != str(profile.display_aspect_num()):
+        match = False
+    if profile_node.getAttribute("display_aspect_den") != str(profile.display_aspect_den()):
+        match = False
+    if profile_node.getAttribute("frame_rate_num") != str(profile.frame_rate_num()):
+        match = False
+    if profile_node.getAttribute("frame_rate_den") != str(profile.frame_rate_den()):
+        match = False
+    if profile_node.getAttribute("colorspace") != str(profile.colorspace()):
+        match = False
+
+    return (match, profile_node.getAttribute("description"))
+
+
 def get_closest_matching_profile_index(producer_info):
     # producer_info is dict from utils.get_file_producer_info
     width = producer_info["width"]
@@ -175,7 +224,7 @@ def get_closest_matching_profile_index(producer_info):
     fps_den = producer_info["fps_den"]
     progressive = producer_info["progressive"]
     fps = round(float(float(fps_num)/float(fps_den)), 1)
-    fps_2 = round(float(float(fps_num)/float(fps_den)), 2) # We added as a fix later for #290, fix might have been done some other way
+    fps_2 = round(float(float(fps_num)/float(fps_den)), 2) # We added as a fix later for #290
     
     # We calculate match score for all available profiles and return 
     # the one with the highest score
@@ -191,13 +240,15 @@ def get_closest_matching_profile_index(producer_info):
         prof_fps_den = profile.frame_rate_den()
         prof_progressive = profile.progressive()
         prof_fps = round(float(float(prof_fps_num)/float(prof_fps_den)), 1)
-        prof_fps_2 = round(float(float(prof_fps_num)/float(prof_fps_den)), 2) # We added this as a fix later for #290, fix might have been done some other way
+        prof_fps_2 = round(float(float(prof_fps_num)/float(prof_fps_den)), 2) # We added this as a fix later for #290
         
         if width == prof_width and height == prof_height:
             match_score = match_score + 1000
+        if (width * 2) < prof_width or (height * 2) < prof_height: # We some time got matches where given profile was hugely different size if other properties matched
+            match_score = match_score - 500
         if fps == prof_fps:
             match_score = match_score + 100
-        if fps_2 == prof_fps_2: # We added this as a fix later for #290, fix might have been done some other way
+        if fps_2 == prof_fps_2: # We added this as a fix later for #290
             match_score = match_score + 5
         if prof_progressive: # prefer progressive always
             match_score = match_score + 10
@@ -210,14 +261,6 @@ def get_closest_matching_profile_index(producer_info):
     
     return current_match_index
 
-def _sort_profiles(a, b):
-    a_desc, a_profile = a
-    b_desc, b_profile = b
-
-    if a_desc.lower() < b_desc.lower():
-        return -1
-    elif a_desc.lower() > b_desc.lower():
-        return 1
-    else:
-        return 0
-
+def _sort_profiles(profile_item):
+    a_desc, a_profile = profile_item
+    return a_desc.lower()
