@@ -18,6 +18,8 @@
     along with Flowblade Movie Editor.  If not, see <http://www.gnu.org/licenses/>.
 """
 
+import os
+
 from gi.repository import Gtk
 
 import appconsts
@@ -27,32 +29,25 @@ import editorpersistance
 import gui
 import guiutils
 import mltprofiles
-# Jan-2017 - SvdB - To get the number of CPU cores
 import multiprocessing
+import utils
 
 PREFERENCES_WIDTH = 730
 PREFERENCES_HEIGHT = 440
 PREFERENCES_LEFT = 410
 
-select_thumbnail_dir_callback = None # app.py sets at start up
-select_render_clips_dir_callback = None # app.py sets at start up
-
 def preferences_dialog():
 
-
     dialog = Gtk.Dialog(_("Editor Preferences"), None,
                     Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT,
-                    (_("Cancel").encode('utf-8'), Gtk.ResponseType.REJECT,
-                    _("OK").encode('utf-8'), Gtk.ResponseType.ACCEPT))
+                    (_("Cancel"), Gtk.ResponseType.REJECT,
+                    _("OK"), Gtk.ResponseType.ACCEPT))
 
-    gen_opts_panel, gen_opts_widgets = _general_options_panel(_thumbs_select_clicked, _renders_select_clicked)
+    gen_opts_panel, gen_opts_widgets = _general_options_panel()
     edit_prefs_panel, edit_prefs_widgets = _edit_prefs_panel()
     playback_prefs_panel, playback_prefs_widgets  = _playback_prefs_panel()
     view_pres_panel, view_pref_widgets = _view_prefs_panel()
-    # Jan-2017 - SvdB
     performance_panel, performance_widgets = _performance_panel()
-    # Apr-2017 - SvdB
-    #shortcuts_panel, shortcuts_widgets = _shortcuts_panel()
 
     notebook = Gtk.Notebook()
     notebook.set_size_request(PREFERENCES_WIDTH, PREFERENCES_HEIGHT)
@@ -61,7 +56,6 @@ def preferences_dialog():
     notebook.append_page(playback_prefs_panel, Gtk.Label(label=_("Playback")))
     notebook.append_page(view_pres_panel, Gtk.Label(label=_("View")))
     notebook.append_page(performance_panel, Gtk.Label(label=_("Performance")))
-    #notebook.append_page(shortcuts_panel, Gtk.Label(label=_("Shortcuts")))
     guiutils.set_margins(notebook, 4, 24, 6, 0)
 
     dialog.connect('response', _preferences_dialog_callback, (gen_opts_widgets, edit_prefs_widgets, playback_prefs_widgets, view_pref_widgets, \
@@ -69,15 +63,11 @@ def preferences_dialog():
     dialog.vbox.pack_start(notebook, True, True, 0)
     dialogutils.set_outer_margins(dialog.vbox)
     dialogutils.default_behaviour(dialog)
-    # Jul-2016 - SvdB - The next line is to get rid of the message "GtkDialog mapped without a transient parent. This is discouraged."
     dialog.set_transient_for(gui.editor_window.window)
     dialog.show_all()
 
-def _thumbs_select_clicked(widget):
-    dialogs.select_thumbnail_dir(select_thumbnail_dir_callback, gui.editor_window.window, editorpersistance.prefs.thumbnail_folder, False)
+    notebook.set_current_page(0) # gen_opts_widgets
 
-def _renders_select_clicked(widget):
-    dialogs.select_rendred_clips_dir(select_render_clips_dir_callback, gui.editor_window.window, editorpersistance.prefs.render_folder)
 
 def _preferences_dialog_callback(dialog, response_id, all_widgets):
     if response_id == Gtk.ResponseType.ACCEPT:
@@ -91,7 +81,7 @@ def _preferences_dialog_callback(dialog, response_id, all_widgets):
 
     dialog.destroy()
 
-def _general_options_panel(folder_select_clicked_cb, render_folder_select_clicked_cb):
+def _general_options_panel():
     prefs = editorpersistance.prefs
 
     # Widgets
@@ -106,23 +96,18 @@ def _general_options_panel(folder_select_clicked_cb, render_folder_select_clicke
     for profile in profiles:
         default_profile_combo.append_text(profile[0])
     default_profile_combo.set_active(mltprofiles.get_default_profile_index())
-
-    spin_adj = Gtk.Adjustment(prefs.undos_max, editorpersistance.UNDO_STACK_MIN, editorpersistance.UNDO_STACK_MAX, 1)
+    spin_adj = Gtk.Adjustment(value=prefs.undos_max, lower=editorpersistance.UNDO_STACK_MIN, upper=editorpersistance.UNDO_STACK_MAX, step_incr=1)
     undo_max_spin = Gtk.SpinButton.new_with_range(editorpersistance.UNDO_STACK_MIN, editorpersistance.UNDO_STACK_MAX, 1)
     undo_max_spin.set_adjustment(spin_adj)
     undo_max_spin.set_numeric(True)
 
-    folder_select = Gtk.Button(_("Select Folder")) # thumbnails
-    folder_select.connect("clicked" , folder_select_clicked_cb)
-
-    render_folder_select = Gtk.Button(_("Select Folder"))
-    render_folder_select.connect("clicked" , render_folder_select_clicked_cb)
-
     autosave_combo = Gtk.ComboBoxText()
-    AUTO_SAVE_OPTS = ((-1, _("No Autosave")),(1, _("1 min")),(2, _("2 min")),(5, _("5 min")))
+    # Aug-2019 - SvdB - AS - This is now initialized in app.main
+    # Using editorpersistance.prefs.AUTO_SAVE_OPTS as source
+    # AUTO_SAVE_OPTS = ((-1, _("No Autosave")),(1, _("1 min")),(2, _("2 min")),(5, _("5 min")))
 
-    for i in range(0, len(AUTO_SAVE_OPTS)):
-        time, desc = AUTO_SAVE_OPTS[i]
+    for i in range(0, len(editorpersistance.prefs.AUTO_SAVE_OPTS)):
+        time, desc = editorpersistance.prefs.AUTO_SAVE_OPTS[i]
         autosave_combo.append_text(desc)
     autosave_combo.set_active(prefs.auto_save_delay_value_index)
 
@@ -132,56 +117,60 @@ def _general_options_panel(folder_select_clicked_cb, render_folder_select_clicke
     load_order_combo.append_text(_("Absolute paths only"))
     load_order_combo.set_active(prefs.media_load_order)
 
+    render_folder_select = Gtk.FileChooserButton.new (_("Select Default Render Folder"), Gtk.FileChooserAction.SELECT_FOLDER)
+    if prefs.default_render_directory == None or prefs.default_render_directory == appconsts.USER_HOME_DIR \
+        or (not os.path.exists(prefs.default_render_directory)) \
+        or (not os.path.isdir(prefs.default_render_directory)):
+        render_folder_select.set_current_folder_uri(os.path.expanduser("~") + "/")
+    else:
+        render_folder_select.set_current_folder_uri(prefs.default_render_directory)
+
+    disk_cache_warning_combo  = Gtk.ComboBoxText()
+    disk_cache_warning_combo.append_text(_("Off"))
+    disk_cache_warning_combo.append_text(_("500 MB"))
+    disk_cache_warning_combo.append_text(_("1 GB"))
+    disk_cache_warning_combo.append_text(_("2 GB"))
+    disk_cache_warning_combo.set_active(prefs.disk_space_warning)
+    
     # Layout
     row1 = _row(guiutils.get_two_column_box(Gtk.Label(label=_("Default Profile:")), default_profile_combo, PREFERENCES_LEFT))
     row2 = _row(guiutils.get_checkbox_row_box(open_in_last_opened_check, Gtk.Label(label=_("Remember last media directory"))))
     row3 = _row(guiutils.get_two_column_box(Gtk.Label(label=_("Undo stack size:")), undo_max_spin, PREFERENCES_LEFT))
-    row4 = _row(guiutils.get_two_column_box(Gtk.Label(label=_("Thumbnail folder:")), folder_select, PREFERENCES_LEFT))
     row5 = _row(guiutils.get_checkbox_row_box(open_in_last_rendered_check, Gtk.Label(label=_("Remember last render directory"))))
     row6 = _row(guiutils.get_two_column_box(Gtk.Label(label=_("Autosave for crash recovery every:")), autosave_combo, PREFERENCES_LEFT))
-    row8 = _row(guiutils.get_two_column_box(Gtk.Label(label=_("Rendered Clips folder:")), render_folder_select, PREFERENCES_LEFT))
     row9 = _row(guiutils.get_two_column_box(Gtk.Label(label=_("Media look-up order on load:")), load_order_combo, PREFERENCES_LEFT))
+    row10 = _row(guiutils.get_two_column_box(Gtk.Label(label=_("Default render directory:")), render_folder_select, PREFERENCES_LEFT))
+    row11 = _row(guiutils.get_two_column_box(Gtk.Label(label=_("Warning on Disk Cache Size:")), disk_cache_warning_combo, PREFERENCES_LEFT))
 
     vbox = Gtk.VBox(False, 2)
     vbox.pack_start(row1, False, False, 0)
     vbox.pack_start(row6, False, False, 0)
     vbox.pack_start(row2, False, False, 0)
+    vbox.pack_start(row10, False, False, 0)
     vbox.pack_start(row5, False, False, 0)
     vbox.pack_start(row3, False, False, 0)
-    vbox.pack_start(row4, False, False, 0)
-    vbox.pack_start(row8, False, False, 0)
     vbox.pack_start(row9, False, False, 0)
+    vbox.pack_start(row11, False, False, 0)
     vbox.pack_start(Gtk.Label(), True, True, 0)
 
     guiutils.set_margins(vbox, 12, 0, 12, 12)
 
-    return vbox, (default_profile_combo, open_in_last_opened_check, open_in_last_rendered_check, undo_max_spin, load_order_combo)
+    # Aug-2019 - SvdB - AS - Added autosave_combo
+    return vbox, ( default_profile_combo, open_in_last_opened_check, open_in_last_rendered_check,
+                    undo_max_spin, load_order_combo, autosave_combo, render_folder_select, disk_cache_warning_combo)
 
 def _edit_prefs_panel():
     prefs = editorpersistance.prefs
 
     # Widgets
-    spin_adj = Gtk.Adjustment(prefs.default_grfx_length, 1, 15000, 1)
+    spin_adj = Gtk.Adjustment(value=prefs.default_grfx_length, lower=1, upper=15000, step_incr=1)
     gfx_length_spin = Gtk.SpinButton()
     gfx_length_spin.set_adjustment(spin_adj)
     gfx_length_spin.set_numeric(True)
 
-    """
-    overwrite_clip_drop = Gtk.ComboBoxText()
-    active = 0
-    if prefs.overwrite_clip_drop == False:
-        active = 1
-    overwrite_clip_drop.append_text(_("Overwrite blanks"))
-    overwrite_clip_drop.append_text(_("Always insert"))
-    overwrite_clip_drop.set_active(active)
-    """
-    
-    kf_edit_playhead_move = Gtk.CheckButton()
-    kf_edit_playhead_move.set_active(prefs.kf_edit_init_affects_playhead)
-
     cover_delete = Gtk.CheckButton()
     cover_delete.set_active(prefs.trans_cover_delete)
-        
+
     active = 0
     if prefs.mouse_scroll_action_is_zoom == False:
         active = 1
@@ -197,30 +186,36 @@ def _edit_prefs_panel():
     hor_scroll_dir.append_text(_("Scroll Up Forward"))
     hor_scroll_dir.append_text(_("Scroll Down Forward"))
     hor_scroll_dir.set_active(active)
-            
+
+    active = 0
+    if prefs.single_click_effects_editor_load == True:
+        active = 1
+    effects_editor_clip_load = Gtk.ComboBoxText()
+    effects_editor_clip_load.append_text(_("On Double Click"))
+    effects_editor_clip_load.append_text(_("On Single Click"))
+    effects_editor_clip_load.set_active(active)
+
     hide_file_ext_button = Gtk.CheckButton()
     if hasattr(prefs, 'hide_file_ext'):
         hide_file_ext_button.set_active(prefs.hide_file_ext)
-    
+
     # Layout
     row4 = _row(guiutils.get_two_column_box(Gtk.Label(label=_("Graphics default length:")), gfx_length_spin, PREFERENCES_LEFT))
-    #row8 = _row(guiutils.get_two_column_box(Gtk.Label(label=_("Media drag'n'drop action on non-V1 tracks:")), overwrite_clip_drop, PREFERENCES_LEFT))
     row9 = _row(guiutils.get_checkbox_row_box(cover_delete, Gtk.Label(label=_("Cover Transition/Fade clips on delete if possible"))))
     # Jul-2016 - SvdB - For play_pause button
     row11 = _row(guiutils.get_two_column_box(Gtk.Label(label=_("Mouse Middle Button Scroll Action:")), mouse_scroll_action, PREFERENCES_LEFT))
     row13 = _row(guiutils.get_two_column_box(Gtk.Label(label=_("Mouse Horizontal Scroll Direction:")), hor_scroll_dir, PREFERENCES_LEFT))
-    row14 = _row(guiutils.get_checkbox_row_box(kf_edit_playhead_move, Gtk.Label(label=_("Move Playhead to Clip start on keyframe edit init"))))
     row12 = _row(guiutils.get_checkbox_row_box(hide_file_ext_button, Gtk.Label(label=_("Hide file extensions when importing Clips"))))
+    row15 = _row(guiutils.get_two_column_box(Gtk.Label(label=_("Open Clip in Effects Editor")), effects_editor_clip_load, PREFERENCES_LEFT))
     # Apr-2017 - SvdB - For Fast Forward / Reverse options
-    
+
     vbox = Gtk.VBox(False, 2)
     vbox.pack_start(row4, False, False, 0)
     vbox.pack_start(row9, False, False, 0)
-    #vbox.pack_start(row8, False, False, 0)
     vbox.pack_start(row11, False, False, 0)
     vbox.pack_start(row13, False, False, 0)
-    vbox.pack_start(row14, False, False, 0)
     vbox.pack_start(row12, False, False, 0)
+    vbox.pack_start(row15, False, False, 0)
     vbox.pack_start(Gtk.Label(), True, True, 0)
 
     guiutils.set_margins(vbox, 12, 0, 12, 12)
@@ -228,7 +223,8 @@ def _edit_prefs_panel():
     # Jul-2016 - SvdB - Added play_pause_button
     # Apr-2017 - SvdB - Added ffwd / rev values
     return vbox, (gfx_length_spin, cover_delete,
-                  mouse_scroll_action, hide_file_ext_button, hor_scroll_dir, kf_edit_playhead_move)
+                  mouse_scroll_action, hide_file_ext_button, hor_scroll_dir,
+                  effects_editor_clip_load)
 
 def _playback_prefs_panel():
     prefs = editorpersistance.prefs
@@ -236,51 +232,56 @@ def _playback_prefs_panel():
     # Widgets
     auto_center_on_stop = Gtk.CheckButton()
     auto_center_on_stop.set_active(prefs.auto_center_on_play_stop)
-    
+
     # Jul-2016 - SvdB - For play_pause button
     play_pause_button = Gtk.CheckButton()
     # The following test is to make sure play_pause can be used for the initial value. If not found, then leave uninitialized
     if hasattr(prefs, 'play_pause'):
-        play_pause_button.set_active(prefs.play_pause)    
-    
+        play_pause_button.set_active(prefs.play_pause)
+
+    timeline_start_end_button = Gtk.CheckButton()
+    if hasattr(prefs, 'timeline_start_end'):
+        timeline_start_end_button.set_active(prefs.timeline_start_end)
+
     auto_center_on_updown = Gtk.CheckButton()
     auto_center_on_updown.set_active(prefs.center_on_arrow_move)
 
     follow_move_range = Gtk.CheckButton()
     follow_move_range.set_active(prefs.playback_follow_move_tline_range)
-    
+
     # Apr-2017 - SvdB - For FF/Rev speed options
     if hasattr(prefs, 'ffwd_rev_shift'):
-        spin_adj = Gtk.Adjustment(prefs.ffwd_rev_shift, 1, 10, 1)
+        spin_adj = Gtk.Adjustment(value=prefs.ffwd_rev_shift, lower=1, upper=10, step_incr=1)
     else:
-        spin_adj = Gtk.Adjustment(1, 1, 10, 1)
+        spin_adj = Gtk.Adjustment(value=1, lower=1, upper=10, step_incr=1)
     ffwd_rev_shift_spin = Gtk.SpinButton()
     ffwd_rev_shift_spin.set_adjustment(spin_adj)
     ffwd_rev_shift_spin.set_numeric(True)
 
     if hasattr(prefs, 'ffwd_rev_ctrl'):
-        spin_adj = Gtk.Adjustment(prefs.ffwd_rev_ctrl, 1, 10, 1)
+        spin_adj = Gtk.Adjustment(value=prefs.ffwd_rev_ctrl, lower=1, upper=10, step_incr=1)
     else:
-        spin_adj = Gtk.Adjustment(10, 1, 10, 1)
+        spin_adj = Gtk.Adjustment(value=10, lower=1, upper=10, step_incr=1)
     ffwd_rev_ctrl_spin = Gtk.SpinButton()
     ffwd_rev_ctrl_spin.set_adjustment(spin_adj)
     ffwd_rev_ctrl_spin.set_numeric(True)
-    
+
     if hasattr(prefs, 'ffwd_rev_caps'):
-        spin_adj = Gtk.Adjustment(prefs.ffwd_rev_caps, 1, 10, 1)
+        spin_adj = Gtk.Adjustment(value=prefs.ffwd_rev_caps, lower=1, upper=10, step_incr=1)
     else:
-        spin_adj = Gtk.Adjustment(1, 1, 10, 1)
+        spin_adj = Gtk.Adjustment(value=1, lower=1, upper=10, step_incr=1)
     ffwd_rev_caps_spin = Gtk.SpinButton()
     ffwd_rev_caps_spin.set_adjustment(spin_adj)
     ffwd_rev_caps_spin.set_numeric(True)
-    
+
+    loop_clips = Gtk.CheckButton()
+    loop_clips.set_active(prefs.loop_clips)
+
     # Layout
-    #row1 = _row(guiutils.get_checkbox_row_box(auto_play_in_clip_monitor, Gtk.Label(label=_("Autoplay new Clips in Clip Monitor"))))
     row2 = _row(guiutils.get_checkbox_row_box(auto_center_on_stop, Gtk.Label(label=_("Center Current Frame on Playback Stop"))))
     row13 = _row(guiutils.get_checkbox_row_box(auto_center_on_updown, Gtk.Label(label=_("Center Current Frame after Up/Down Arrow"))))
-    # Jul-2016 - SvdB - For play_pause button
     row10 = _row(guiutils.get_checkbox_row_box(play_pause_button, Gtk.Label(label=_("Enable single Play/Pause button"))))
-    # Apr-2017 - SvdB - For Fast Forward / Reverse options
+    row11 = _row(guiutils.get_checkbox_row_box(timeline_start_end_button, Gtk.Label(label=_("Enable To Start and To End buttons"))))
     row14 = _row(guiutils.get_two_column_box(Gtk.Label(label=_("Fast Forward / Reverse Speed for Shift Key:")), ffwd_rev_shift_spin, PREFERENCES_LEFT))
     row14.set_tooltip_text(_("Speed of Forward / Reverse will be multiplied by this value if Shift Key is held (Only using KEYS).\n" \
         "Enabling multiple modifier keys will multiply the set values.\n" \
@@ -291,29 +292,28 @@ def _playback_prefs_panel():
     row15.set_tooltip_text(_("Speed of Forward / Reverse will be multiplied by this value if Ctrl Key is held (Only using KEYS)."))
     row16 = _row(guiutils.get_two_column_box(Gtk.Label(label=_("Fast Forward / Reverse Speed for Caps Lock Key:")), ffwd_rev_caps_spin, PREFERENCES_LEFT))
     row16.set_tooltip_text(_("Speed of Forward / Reverse will be multiplied by this value if Caps Lock is set (Only using KEYS)."))
-
     row17 = _row(guiutils.get_checkbox_row_box(follow_move_range, Gtk.Label(label=_("Move Timeline to follow Playback"))))
-    
+    row18 = _row(guiutils.get_checkbox_row_box(loop_clips, Gtk.Label(label=_("Loop Media Clips on Monitor"))))
+
     vbox = Gtk.VBox(False, 2)
     vbox.pack_start(row17, False, False, 0)
+    vbox.pack_start(row18, False, False, 0)
     vbox.pack_start(row2, False, False, 0)
     vbox.pack_start(row13, False, False, 0)
-    # Jul-2016 - SvdB - For play_pause button
     vbox.pack_start(row10, False, False, 0)
-    # Apr-2017 - SvdB - For ffwd / rev speed
+    vbox.pack_start(row11, False, False, 0)
     vbox.pack_start(row14, False, False, 0)
     vbox.pack_start(row15, False, False, 0)
     vbox.pack_start(row16, False, False, 0)
+
     vbox.pack_start(Gtk.Label(), True, True, 0)
 
     guiutils.set_margins(vbox, 12, 0, 12, 12)
 
-    # Jul-2016 - SvdB - Added play_pause_button
-    # Apr-2017 - SvdB - Added ffwd / rev values
-    return vbox, (auto_center_on_stop, 
-                  play_pause_button, auto_center_on_updown,
-                  ffwd_rev_shift_spin, ffwd_rev_ctrl_spin, ffwd_rev_caps_spin, follow_move_range)
-                  
+    return vbox, (auto_center_on_stop,
+                  play_pause_button, timeline_start_end_button, auto_center_on_updown,
+                  ffwd_rev_shift_spin, ffwd_rev_ctrl_spin, ffwd_rev_caps_spin, follow_move_range, loop_clips)
+
 def _view_prefs_panel():
     prefs = editorpersistance.prefs
 
@@ -321,6 +321,26 @@ def _view_prefs_panel():
     force_english_check = Gtk.CheckButton()
     force_english_check.set_active(prefs.use_english_always)
 
+    force_language_combo = Gtk.ComboBoxText()
+    force_language_combo.append_text(_("None"))
+    force_language_combo.append_text(_("English"))
+    force_language_combo.append_text(_("Chinese, Simplified"))
+    force_language_combo.append_text(_("Chinese, Traditional"))
+    force_language_combo.append_text(_("Czech"))
+    force_language_combo.append_text(_("French"))
+    force_language_combo.append_text(_("German"))
+    force_language_combo.append_text(_("Hungarian"))
+    force_language_combo.append_text(_("Italian"))
+    force_language_combo.append_text(_("Polish"))
+    force_language_combo.append_text(_("Russian"))
+    force_language_combo.append_text(_("Spanish"))
+    force_language_combo.append_text(_("Ukranian"))
+    # THIS NEEDS TO BE UPDATED WHEN LANGUAGES ARE ADDED!!!
+    lang_list = ["None","English","zh_CN","zh_TW","cs","fr","de","hu","it","pl","ru","es","uk"]
+    active_index = lang_list.index(prefs.force_language)
+    force_language_combo.set_active(active_index)
+    force_language_combo.lang_codes = lang_list
+
     display_splash_check = Gtk.CheckButton()
     display_splash_check.set_active(prefs.display_splash_screen)
 
@@ -328,6 +348,11 @@ def _view_prefs_panel():
     show_full_file_names = Gtk.CheckButton()
     show_full_file_names.set_active(prefs.show_full_file_names)
 
+    # --------------------------------- Colorized icons
+    colorized_icons = Gtk.CheckButton()
+    colorized_icons.set_active(prefs.colorized_icons)
+    # ------------------------------------ End of Colorized icons
+
     buttons_combo = Gtk.ComboBoxText()
     buttons_combo.append_text(_("Glass"))
     buttons_combo.append_text(_("Simple"))
@@ -335,11 +360,20 @@ def _view_prefs_panel():
     buttons_combo.set_active( prefs.buttons_style )
 
     dark_combo = Gtk.ComboBoxText()
-    dark_combo.append_text(_("Flowblade Theme"))
+    dark_combo.append_text(_("Flowblade Theme Neutral"))
+    dark_combo.append_text(_("Flowblade Theme Gray"))
+    dark_combo.append_text(_("Flowblade Theme Blue"))
     dark_combo.append_text(_("Dark Theme"))
     dark_combo.append_text(_("Light Theme"))
-    dark_combo.set_active(prefs.theme)
-
+    # The displayed options indeces do not correspond with theme const values.
+    if prefs.theme == appconsts.FLOWBLADE_THEME_GRAY:
+        index = 1
+    elif prefs.theme == appconsts.FLOWBLADE_THEME_NEUTRAL:
+        index = 0
+    else:
+        index = int(prefs.theme) + 2
+    
+    dark_combo.set_active(index)
 
     theme_combo = Gtk.ComboBoxText()
     for theme in gui._THEME_COLORS:
@@ -365,31 +399,44 @@ def _view_prefs_panel():
     tracks_combo = Gtk.ComboBoxText()
     tracks_combo.append_text(_("Normal - 50px, 25px"))
     tracks_combo.append_text(_("Double for HiDPI - 100px, 50px"))
+    # Aug-2019 - SvdB - BB
     tracks_combo.set_active(prefs.double_track_hights)
 
     top_row_layout = Gtk.ComboBoxText()
     top_row_layout.append_text(_("3 panels if width (1450px+) available"))
     top_row_layout.append_text(_("2 panels always"))
     top_row_layout.set_active(prefs.top_row_layout)
-        
+
+    monitors_data = utils.get_display_monitors_size_data()
+    layout_monitor = Gtk.ComboBoxText()
+    combined_w, combined_h = monitors_data[0]
+    layout_monitor.append_text(_("Full Display area: ") + str(combined_w) + " x " + str(combined_h))
+    if len(monitors_data) >= 3:
+        for monitor_index in range(1, len(monitors_data)):
+            monitor_w, monitor_h = monitors_data[monitor_index]
+            layout_monitor.append_text(_("Monitor ") + str(monitor_index) + ": " + str(monitor_w) + " x " + str(monitor_h))
+    layout_monitor.set_active(prefs.layout_display_index)
+
+
     # Layout
     row00 = _row(guiutils.get_two_column_box(Gtk.Label(label=_("Application window mode:")), window_mode_combo, PREFERENCES_LEFT))
-    row0 = _row(guiutils.get_checkbox_row_box(force_english_check, Gtk.Label(label=_("Use English texts on localized OS"))))
+    row9 = _row(guiutils.get_two_column_box(Gtk.Label(label=_("Force Language:")), force_language_combo, PREFERENCES_LEFT))
     row1 = _row(guiutils.get_checkbox_row_box(display_splash_check, Gtk.Label(label=_("Display splash screen"))))
     row2 = _row(guiutils.get_two_column_box(Gtk.Label(label=_("Buttons style:")), buttons_combo, PREFERENCES_LEFT))
     row3 = _row(guiutils.get_two_column_box(Gtk.Label(label=_("Theme request, icons and colors:")), dark_combo, PREFERENCES_LEFT))
     row4 = _row(guiutils.get_two_column_box(Gtk.Label(label=_("Theme detection fail fallback colors:")), theme_combo, PREFERENCES_LEFT))
     row5 = _row(guiutils.get_two_column_box(Gtk.Label(label=_("Default audio levels display:")), audio_levels_combo, PREFERENCES_LEFT))
     row7 = _row(guiutils.get_two_column_box(Gtk.Label(label=_("Tracks Heights:")), tracks_combo, PREFERENCES_LEFT))
-
     # Feb-2017 - SvdB - For full file names
     row6 =  _row(guiutils.get_checkbox_row_box(show_full_file_names, Gtk.Label(label=_("Show Full File names"))))
-
     row8 =  _row(guiutils.get_two_column_box(Gtk.Label(label=_("Top row layout:")), top_row_layout, PREFERENCES_LEFT))
-
+    row10 = _row(guiutils.get_two_column_box(Gtk.Label(label=_("Do GUI layout based on:")), layout_monitor, PREFERENCES_LEFT))
+    row11 =  _row(guiutils.get_checkbox_row_box(colorized_icons, Gtk.Label(label=_("Toolbar color icons"))))
+    
     vbox = Gtk.VBox(False, 2)
     vbox.pack_start(row00, False, False, 0)
-    vbox.pack_start(row0, False, False, 0)
+    vbox.pack_start(row10, False, False, 0)
+    vbox.pack_start(row9, False, False, 0)
     vbox.pack_start(row1, False, False, 0)
     vbox.pack_start(row2, False, False, 0)
     vbox.pack_start(row3, False, False, 0)
@@ -398,14 +445,16 @@ def _view_prefs_panel():
     vbox.pack_start(row7, False, False, 0)
     # Feb-2017 - SvdB - For full file names
     vbox.pack_start(row6, False, False, 0)
-    vbox.pack_start(row8, False, False, 0)
+    vbox.pack_start(row11, False, False, 0)
     vbox.pack_start(Gtk.Label(), True, True, 0)
-
+    
     guiutils.set_margins(vbox, 12, 0, 12, 12)
 
-    # Feb-2017 - SvdB - Added code for full file names
-    return vbox, (force_english_check, display_splash_check, buttons_combo, dark_combo, theme_combo, audio_levels_combo, 
-                  window_mode_combo, show_full_file_names, tracks_combo, top_row_layout)
+    return vbox, (force_language_combo, display_splash_check, buttons_combo, dark_combo, theme_combo, audio_levels_combo,
+                  window_mode_combo, show_full_file_names, tracks_combo, top_row_layout, layout_monitor, colorized_icons)
+
+
+
 
 def _performance_panel():
     # Jan-2017 - SvdB
@@ -418,10 +467,10 @@ def _performance_panel():
 
     warning_icon = Gtk.Image.new_from_stock(Gtk.STOCK_DIALOG_WARNING, Gtk.IconSize.DIALOG)
     warning_label = Gtk.Label(label=_("Changing these values may cause problems with playback and rendering.\nThe safe values are Render Threads:1, Allow Frame Dropping: No."))
-    # Widgets
-    spin_adj = Gtk.Adjustment(prefs.perf_render_threads, 1, multiprocessing.cpu_count(), 1)
-    perf_render_threads = Gtk.SpinButton()
-    perf_render_threads.set_adjustment(spin_adj)
+
+    spin_adj = Gtk.Adjustment(value=prefs.perf_render_threads, lower=1, upper=multiprocessing.cpu_count(), step_incr=1)
+    perf_render_threads = Gtk.SpinButton(adjustment=spin_adj)
+    #perf_render_threads.set_adjustment(spin_adj)
     perf_render_threads.set_numeric(True)
 
     perf_drop_frames = Gtk.CheckButton()
