@@ -165,7 +165,7 @@ def get_transition_editable_properties(compositor):
 
         if p_type == mltfilters.PROP_EXPRESSION:
             """
-            For expressions we can't do straight input output numerical
+            For expressions we can't do straight input->output numerical
             conversion so we need a extending class for expression type.
             """
             args = propertyparse.args_string_to_args_dict(args_str)
@@ -906,6 +906,8 @@ class AffineScaleProperty(EditableProperty):
         if in_norm < 0.001:
             in_norm = 0.001
         out =  1.0 / in_norm
+        
+        return out
 
     def get_in_value(self, out_value):
         """
