@@ -43,7 +43,6 @@ PATTERN_PRODUCER_PATH = None
 LAUNCH_DIR = None
 GMIC_SCRIPTS_DOC = None
 PHANTOM_JAR = None
-NATRON_PROJECTS_XML_DOC = None
 
 def set_paths(root_path):
     # Apr-2017 - SvdB - Added SHORTCUTS_PATH for keyboard shortcuts files
@@ -53,7 +52,7 @@ def set_paths(root_path):
     GPL_3_DOC, ROOT_PARENT, PATTERN_PRODUCER_PATH, TRANSLATIONS_DOC, \
     LAUNCH_DIR, REPLACEMENTS_XML_DOC, GMIC_SCRIPTS_DOC,  \
     PHANTOM_JAR, PHANTOM_DIR, DEVELOPERS_DOC, CONTRIBUTORS_DOC, \
-    SHORTCUTS_PATH, NATRON_PROJECTS_XML_DOC
+    SHORTCUTS_PATH
     
     ROOT_PATH = root_path
     IMAGE_PATH = root_path + "/res/img/"
@@ -73,17 +72,12 @@ def set_paths(root_path):
     ROOT_PARENT = ROOT_PATH.strip("Flowblade")
     PATTERN_PRODUCER_PATH = root_path + "/res/patternproducer/"
     LAUNCH_DIR = root_path + "/launch/"
-    GMIC_SCRIPTS_DOC = root_path + "/res/gmic/gmicscripts.xml"
+    GMIC_SCRIPTS_DOC = root_path + "/res/gmic/gmic2scripts.xml"
     PHANTOM_JAR = root_path + "/phantom2d/Phantom2D.jar"
     # Apr-2017 - SvdB
     SHORTCUTS_PATH = root_path + "/res/shortcuts/"
-    NATRON_PROJECTS_XML_DOC = root_path + "/res/natron/natronprojects.xml"
 
 def apply_dark_theme():
     global IMAGE_PATH
     IMAGE_PATH = ROOT_PATH + "/res/darktheme/"
-
-def set_gmic2(root_path):
-    global GMIC_SCRIPTS_DOC
-    GMIC_SCRIPTS_DOC = root_path + "/res/gmic/gmic2scripts.xml"
     
