@@ -22,7 +22,7 @@
 Module contains class Sequence that is the multitrack media object being edited
 by the application. A project has 1-n of these.
 """
-
+import time
 import mlt
 import os
 
@@ -33,6 +33,7 @@ import mltfilters
 import mlttransitions
 import mltrefhold
 import patternproducer
+import tlinerender
 import utils
 
 # Media types for tracks or clips
@@ -53,6 +54,7 @@ SYNC_LOCKED = appconsts.SYNC_LOCKED # FEATURE NOT AVAILABLE TO USER CURRENTLY!
 LOCKED = appconsts.LOCKED # no edits allowed
 
 # Display heights
+TRACK_HEIGHT_HIGH = appconsts.TRACK_HEIGHT_HIGH # track height in canvas and column
 TRACK_HEIGHT_NORMAL = appconsts.TRACK_HEIGHT_NORMAL # track height in canvas and column
 TRACK_HEIGHT_SMALL = appconsts.TRACK_HEIGHT_SMALL # track height in canvas and column
 
@@ -81,13 +83,13 @@ PROGRAM_OUT_MODE = 0
 VECTORSCOPE_MODE = 1
 RGB_PARADE_MODE = 2
 
-# black clip
+# Black clip
 black_track_clip = None
 
-# Track that all audio is mixed down to combine for output.
+# The track that all audio is mixed down to combine for output.
 AUDIO_MIX_DOWN_TRACK = 0
 
-# Vectorscop and RGB Parade
+# Vectorscope and RGB Parade
 SCOPE_MIX_VALUES = [0.0, 0.2, 0.5, 0.8, 1.0]
 _scope_over_lay_mix = 2
 
@@ -111,6 +113,7 @@ class Sequence:
         self.watermark_filter = None
         self.watermark_file_path = None
         self.seq_len = 0 # used in trim crash hack, remove when fixed
+        self.compositing_mode = appconsts.COMPOSITING_MODE_TOP_DOWN_FREE_MOVE
 
         # MLT objects for a multitrack sequence
         self.init_mlt_objects()
@@ -203,15 +206,26 @@ class Sequence:
 
     def _create_black_track_clip(self):
         # Create 1 fr long black bg clip and set in and out
-        global black_track_clip # btw, why global?
+        global black_track_clip
         
         # This is not an actual bin clip so id can be -1, it is just used to create the producer
         pattern_producer_data = patternproducer.BinColorClip(-1, "black_bg", "#000000000000")
-            
+
         black_track_clip = self.create_pattern_producer(pattern_producer_data)
         black_track_clip.clip_in = 0
         black_track_clip.clip_out = 0
 
+    def _create_white_clip(self, length):
+        
+        # This is not an actual bin clip so id can be -1, it is just used to create the producer
+        pattern_producer_data = patternproducer.BinColorClip(-1, "white_bg", "#ffffffffffff")
+            
+        white_clip = self.create_pattern_producer(pattern_producer_data)
+        white_clip.clip_in = 0
+        white_clip.clip_out = length - 1
+        
+        return white_clip
+        
     def add_track(self, track_type, is_hidden=False):
         """ 
         Creates a MLT playlist object, adds project
@@ -276,15 +290,19 @@ class Sequence:
 
     def _mix_audio_for_track(self, track):
         # Create and add transition to combine track audios
+        #
+        # Audio transition objects are not saved and are thrown away when track count is changed so we don't
+        # need to hold references to them in Sequence object, mltrefhold stuff is just very 
+        # defencsive programming because MLT crashes are most related to deleting stuff, probably not needed at all.
         transition = mlt.Transition(self.profile, "mix")
-        mltrefhold.hold_ref(transition)
+        mltrefhold.hold_ref(transition) # look  to remove
         transition.set("a_track", int(AUDIO_MIX_DOWN_TRACK))
         transition.set("b_track", track.id)
         transition.set("always_active", 1)
         transition.set("combine", 1)
         self.field.plant_transition(transition, int(AUDIO_MIX_DOWN_TRACK), track.id)
 
-        # Create and ad gain filter
+        # Create and add gain filter
         gain_filter = mlt.Filter(self.profile, "volume")
         mltrefhold.hold_ref(gain_filter)
         gain_filter.set("gain", str(track.audio_gain))
@@ -346,7 +364,8 @@ class Sequence:
         self.master_audio_gain = gain
 
     def add_track_pan_filter(self, track, value):
-        # This method is used for master too, and called with tractor then
+        # This method is used for master too, and is called with tractor then.
+        # 'channel' attr is left to -1 meaning that panner affects front balance.
         pan_filter = mlt.Filter(self.profile, "panner")
         mltrefhold.hold_ref(pan_filter)
         pan_filter.set("start", value)
@@ -390,10 +409,9 @@ class Sequence:
         """
         Creates MLT Producer and adds attributes to it, but does 
         not add it to track/playlist object.
-        """        
+        """
         producer = mlt.Producer(self.profile, str(path)) # this runs 0.5s+ on some clips
-        if novalidate == True:
-            producer.set("mlt_service", "avformat-novalidate")
+    
         mltrefhold.hold_ref(producer)
         producer.path = path
         producer.filters = []
@@ -406,7 +424,7 @@ class Sequence:
         producer.media_type = get_media_type(path)
 
         if producer.media_type == FILE_DOES_NOT_EXIST:
-            print "file does not exist"
+            print("file does not exist")
             return None
 
         self.add_clip_attr(producer)
@@ -414,7 +432,7 @@ class Sequence:
         # Img seq ttl value
         producer.ttl = ttl
         if ttl != None:
-            producer.set("ttl", int(ttl))
+            producer.set("ttl", str(ttl))
 
         return producer
 
@@ -471,9 +489,14 @@ class Sequence:
         clip.waveform_data = None
         clip.color = None # None means that clip type default color is displayed
         clip.markers = []
+        clip.container_data = None
         
     def clone_track_clip(self, track, index):
         orig_clip = track.clips[index]
+        if orig_clip.is_blanck_clip == True:
+            # Blank clips are created by adding a blank clip into MLT track and that is not approprite here,
+            # so blank clips are represent by their length as int.
+            return orig_clip.clip_length()
         return self.create_clone_clip(orig_clip)
 
     def create_clone_clip(self, clip):
@@ -498,14 +521,26 @@ class Sequence:
             clone_filter = mltfilters.clone_filter_object(f, self.profile)
             clone_clip.attach(clone_filter.mlt_filter)
             clone_clip.filters.append(clone_filter)
-
+    
+    def copy_filters(self, clip, clone_clip):
+        for f in clip.filters:
+            clone_filter = mltfilters.clone_filter_object(f, self.profile)
+            clone_clip.attach(clone_filter.mlt_filter)
+            clone_clip.filters.append(clone_filter)
+            
     def clone_filters(self, clip):
         clone_filters = []
         for f in clip.filters:
             clone_filter = mltfilters.clone_filter_object(f, self.profile)
             clone_filters.append(clone_filter)
         return clone_filters
-
+    
+    def clone_mute_state(self, clip, clone_clip):
+        # Mute 
+        if clip.mute_filter != None:
+            mute_filter = mltfilters.create_mute_volume_filter(self) 
+            mltfilters.do_clip_mute(clone_clip, mute_filter)
+            
     def get_next_id(self):
         """
         Growing id for newly created clip or transition. 
@@ -514,6 +549,9 @@ class Sequence:
         return self.next_id - 1
 
     def clip_is_in_sequence(self, test_clip):
+        if test_clip == None:
+            return False
+
         for i in range(1, len(self.tracks)):
             track = self.tracks[i]
             for clip in track.clips:
@@ -570,6 +608,11 @@ class Sequence:
         return compositor
 
     def restack_compositors(self):
+        if self.compositing_mode == appconsts.COMPOSITING_MODE_STANDARD_FULL_TRACK:
+            # we should only see this on sequence creation and adding removing tracks for COMPOSITING_MODE_STANDARD_FULL_TRACK
+            # remove this later
+            print("restacking compositors!")
+
         self.sort_compositors()
 
         new_compositors = []
@@ -599,6 +642,9 @@ class Sequence:
         compositor.set_in_and_out(old_compositor.clip_in, old_compositor.clip_out)
         compositor.transition.set_tracks(old_compositor.transition.a_track, old_compositor.transition.b_track)
         compositor.obey_autofollow = old_compositor.obey_autofollow
+        if self.compositing_mode == appconsts.COMPOSITING_MODE_STANDARD_FULL_TRACK:
+            compositor.transition.mlt_transition.set("always_active", str(1))
+            compositor.transition.mlt_transition.set("disable", str(0))
         self._plant_compositor(compositor)
         return compositor
     
@@ -626,7 +672,7 @@ class Sequence:
 
     def add_compositor(self, compositor):
         self.compositors.append(compositor)
-
+        
     def remove_compositor(self, old_compositor):
         try:
             self.compositors.remove(old_compositor)
@@ -636,25 +682,60 @@ class Sequence:
                 if comp.destroy_id == old_compositor.destroy_id:
                     found = True
                     self.compositors.remove(comp)
-                    #edit.old_compositors.append(comp)
                     old_compositor = comp
             if found == False:
                 raise ValueError('compositor not found using destroy_id')
             
         self.field.disconnect_service(old_compositor.transition.mlt_transition)
 
+    def destroy_compositors(self):
+        # This can be called when undo stack destroyd too.
+        for compositor in self.compositors:
+            self.field.disconnect_service(compositor.transition.mlt_transition)
+        self.compositors = []
+
+    def add_full_track_compositors(self):
+        print("Adding full track compositors")
+        
+        for i in range(self.first_video_index, len(self.tracks) - 1):
+            track = self.tracks[i]
+        
+            compositor = self.create_compositor("##affineblend")
+
+            a_track = self.first_video_index
+            b_track = track.id 
+            compositor.transition.set_tracks(a_track, b_track)
+            compositor.set_in_and_out(-1, -1)
+            compositor.transition.mlt_transition.set("always_active", str(1))
+            compositor.origin_clip_id = -1
+
+            self.add_compositor(compositor)
+        
+        self.restack_compositors()
+        print("Adding full track compositors DONE")
+    
     def get_compositor_for_destroy_id(self, destroy_id):
         for comp in self.compositors:
             if comp.destroy_id == destroy_id:
                 return comp
         raise ValueError('compositor for id not found')
 
+    def get_clip_compositors(self, clip):
+        clip_compositors = []
+        for compositor in self.compositors:
+            if compositor.origin_clip_id == clip.id:
+                clip_compositors.append(compositor)
+        return clip_compositors
+        
     def sort_compositors(self):
         """
         Compositor order must be from top to bottom or will not work.
         """
-        self.compositors.sort(_sort_compositors_comparator)
-
+        if self.compositing_mode != appconsts.COMPOSITING_MODE_STANDARD_AUTO_FOLLOW and self.compositing_mode != appconsts.COMPOSITING_MODE_STANDARD_FULL_TRACK:
+            self.compositors.sort(key=_sort_compositors_comparator, reverse=True)
+        else:
+            self.compositors.sort(key=_sort_compositors_comparator)
+        
     def get_track_compositors(self, track_index):
         track_compositors = []
         for comp in self.compositors:
@@ -756,27 +837,46 @@ class Sequence:
         self._unmute_editable()
 
     def update_edit_tracks_length(self):
-        # NEEDED FOR TRIM CRASH HACK, REMOVE IF FIXED
-        self.seq_len = 0  # muuta  arvoksi 1 ???
+        # Needed for timeline renderering updates
+        self.seq_len = 0 
         for i in range(1, len(self.tracks) - 1):
             track_len = self.tracks[i].get_length()
             if track_len > self.seq_len:
                 self.seq_len = track_len
 
-    def update_trim_hack_blank_length(self):
-        # NEEDED FOR TRIM CRASH HACK, REMOVE IF FIXED
+    def update_hidden_track_for_timeline_rendering(self):
+        # Needed for timeline render updates
         self.tracks[-1].clips = []
         self.tracks[-1].clear()
 
         seq_len = self.seq_len
         if seq_len < 1:
             seq_len = 1
-            
-        edit._insert_blank(self.tracks[-1], 0, seq_len)
+        
+        tlinerender.get_renderer().update_hidden_track(self.tracks[-1], seq_len)
+
+    def fix_v1_for_render(self): 
+        # This is a workaround to fix Issue #941 with H248 encoder not being able handle 
+        # blanks and crashing or losing working audio. Underlying reason still 
+        # not known.
+        track_v1 = self.tracks[self.first_video_index]
+        for i in range(0, len(track_v1.clips)):
+            clip = track_v1.clips[i]
+            if clip.is_blanck_clip == False:
+                continue
+            track_v1.remove(i)
+            track_v1.clips.pop(i)
+            length = clip.clip_out - clip.clip_in + 1
+            white_clip = self._create_white_clip(length)
+            edit._insert_clip(track_v1, white_clip, i, white_clip.clip_in, white_clip.clip_out)
+
+        if track_v1.get_length() < self.get_length():
+            length = self.get_length() - track_v1.get_length()
+            white_clip = self._create_white_clip(length)
+            edit.append_clip(track_v1, white_clip, white_clip.clip_in, white_clip.clip_out)
 
     def get_seq_range_frame(self, frame):
-        # NEEDED FOR TRIM CRASH HACK, REMOVE IF FIXED
-        # remove TimeLineFrameScale then too
+        # Needed for timeline renderering updates
         if frame >= (self.seq_len - 1):
             return self.seq_len - 1
         else:
@@ -843,37 +943,42 @@ class Sequence:
         """
         Set black to track length of sequence.
         """
-        global black_track_clip
-        if black_track_clip == None: # This fails for launch with assoc Gnome file because this has not been made yet.
-                                     # This global black_track_clip is brain dead.  
-            self._create_black_track_clip()
-        c_in = 0
-        c_out = self.get_length()
-        black_track_clip.clip_in = c_in
-        black_track_clip.clip_out = c_out
-        black_track_clip.set_in_and_out(c_in, c_out)
+        self._create_black_track_clip()
+        
+        # We are now always creating a new 1 frame long black track clip
+        # and putting it on track0. This is to stay compatible with earlier project files
+        # but to also always have just 1 frame long black track clip.
+        if len(self.tracks[0].clips) > 0:
+            self.tracks[0].clips.pop(0) # py
+            self.tracks[0].remove(0) # mlt
+
+        self.tracks[0].clips.append(black_track_clip) # py
+        self.tracks[0].append(black_track_clip, 0, 0) # mlt
+        
+        # LOOK TO GET RID OF THIS, WE ARE CREATING A NEW BLACK CLIP PER CHANGE OF SEQUENCE!
 
     def get_length(self):
         return self.multitrack.get_length()
 
     def resize_tracks_to_fit(self, allocation):
         x, y, w, panel_height = allocation.x, allocation.y, allocation.width, allocation.height
-        count = 0
+        track_id = 1
         fix_next = True
         while(fix_next):
             tracks_height = self.get_tracks_height()
             if tracks_height < panel_height:
                 fix_next = False
-            elif count + 1 == self.first_video_index:
+            elif track_id == self.first_video_index:
+                # V1 should stay large and everything should still fit
+                track_id += 1
+                continue
+            elif track_id == len(self.tracks) - 2:
                 # This shold not happen because track heights should be set up so that minimized app 
-                # has enough space to display all tracks.
-                # Yet it happens sometimes, meh.
-                print "sequence.resize_tracks_to_fit (): could not make tracks fit in timeline vertical space"
                 fix_next = False
+                print("sequence.resize_tracks_to_fit (): could not make tracks fit in timeline vertical space")
             else:
-                self.tracks[1 + count].height = TRACK_HEIGHT_SMALL
-                self.tracks[len(self.tracks) - 2 - count].height = TRACK_HEIGHT_SMALL
-                count += 1
+                self.tracks[track_id].height = TRACK_HEIGHT_SMALL
+                track_id += 1
 
     def find_next_cut_frame(self, tline_frame):
         """
@@ -907,10 +1012,12 @@ class Sequence:
         """
         Returns frame of next cut in active tracks relative to timeline.
         """
+        if tline_frame == 0:
+            return 0 # Rest of method fails for this special case
+        
         cut_frame = -1
         for i in range(1, len(self.tracks) - 1):
             track = self.tracks[i]
-            #print track.get_producer().get_length()
             
             # Get index and clip start
             index = track.get_clip_index_at(tline_frame)
@@ -1004,6 +1111,19 @@ class Sequence:
                     return clip
 
         return None
+
+    def get_track_and_index_for_id(self, clip_id):
+        """
+        Returns clip or None if not found.
+        """
+        for i in range(1, len(self.tracks)):
+            track = self.tracks[i]
+            for j in range(0, len(track.clips)):
+                clip = track.clips[j]
+                if clip.id == clip_id:
+                    return (track, j)
+
+        return (None, None)
         
     def set_track_mute_state(self, track_index, mute_state):
         track = self.tracks[track_index]
@@ -1042,15 +1162,15 @@ class Sequence:
                     clip.waveform_data = None
 
     def print_all(self):
-        print "------------------------######"
+        print("------------------------######")
         for i in range(0, len(self.tracks)):
-            print "TRACK:", i
+            print("TRACK:", i)
             self.print_track(i)
 
     def print_track(self, track_id):
         track = self.tracks[track_id]
 
-        print "PYTHON"
+        print("PYTHON")
         for i in range(0, len(track.clips)):
             clip = track.clips[i]
             if clip.is_blank():
@@ -1058,27 +1178,28 @@ class Sequence:
             else:
                 msg = clip.name
      
-            print i, ": id:", clip.id, " in:",clip.clip_in," out:", \
-            clip.clip_out, msg
+            print(i, ": id:", clip.id, " in:",clip.clip_in," out:", \
+            clip.clip_out, msg)
 
-        print "MLT"
+        print("MLT")
         for i in range(0, track.count()):
             clip = track.get_clip(i)
-            print i, " in:", clip.get_in()," out:", clip.get_out()
-
+            print(i, " in:", clip.get_in()," out:", clip.get_out())
 
     def print_compositors(self):
         for compositor in self.compositors:
-            print "---"
-            print compositor.name
-            print "a_track:" , compositor.transition.a_track
-            print "b_track:" , compositor.transition.b_track
+            print("---")
+            print(compositor.name)
+            print("a_track:" , compositor.transition.a_track)
+            print("b_track:" , compositor.transition.b_track)
 
 # ------------------------------------------------ module util methods
 def get_media_type(file_path):
     """
     Returns media type of file.
     """
+    return utils.get_media_type(file_path)
+    """
     if os.path.exists(file_path):
         mime_type = utils.get_file_type(file_path)
     else:
@@ -1105,18 +1226,13 @@ def get_media_type(file_path):
         return IMAGE
     
     return UNKNOWN
-
+    """
+    
 def _clip_length(clip):
     return clip.clip_out - clip.clip_in + 1
 
-def _sort_compositors_comparator(a_comp, b_comp):
-    # compositors on top most tracks first
-    if a_comp.transition.b_track > b_comp.transition.b_track:
-        return -1
-    elif a_comp.transition.b_track < b_comp.transition.b_track:
-        return 1
-    else:
-        return 0
+def _sort_compositors_comparator(a_comp):
+    return int(a_comp.transition.b_track)
 
 # ----------------------------- sequence cloning for tracks count change
 def create_sequence_clone_with_different_track_count(old_seq, v_tracks, a_tracks):
@@ -1137,6 +1253,9 @@ def create_sequence_clone_with_different_track_count(old_seq, v_tracks, a_tracks
     track_delta = new_seq.first_video_index - old_seq.first_video_index
     new_seq.clone_compositors_from_sequence(old_seq, track_delta)
 
+    # Copy modes values
+    new_seq.compositing_mode = old_seq.compositing_mode
+        
     # copy next clip id data
     new_seq.next_id = old_seq.next_id
     return new_seq
