@@ -30,7 +30,7 @@ import dbus
 import dbus.service
 from dbus.mainloop.glib import DBusGMainLoop
 import mlt
-import md5
+import hashlib
 import locale
 import os
 from os import listdir
@@ -45,6 +45,7 @@ import time
 import threading
 import unicodedata
 
+import atomicfile
 import appconsts
 import dialogutils
 import editorstate
@@ -60,6 +61,7 @@ import persistance
 import respaths
 import renderconsumer
 import translations
+import userfolders
 import utils
 
 
@@ -122,6 +124,10 @@ class QueueRunnerThread(threading.Thread):
 
             project = persistance.load_project(project_file_path, False)
 
+            project.c_seq.fix_v1_for_render()
+
+            maybe_create_render_folder(render_item.render_path)
+        
             producer = project.c_seq.tractor
             profile = mltprofiles.get_profile(render_item.render_data.profile_name)
             consumer = renderconsumer.get_mlt_render_consumer(render_item.render_path, 
@@ -204,7 +210,7 @@ class QueueRunnerThread(threading.Thread):
 
 class BatchRenderDBUSService(dbus.service.Object):
     def __init__(self):
-        print "dbus service init"
+        print("dbus service init")
         bus_name = dbus.service.BusName('flowblade.movie.editor.batchrender', bus=dbus.SessionBus())
         dbus.service.Object.__init__(self, bus_name, '/flowblade/movie/editor/batchrender')
 
@@ -255,11 +261,9 @@ def add_render_item(flowblade_project, render_path, args_vals_list, mark_in, mar
     else:
         launch_batch_rendering()
 
-    #print "Render queue item for rendering file into " + render_path + " with identifier " + identifier + " added."
-
 # ------------------------------------------------------- file utils
 def init_dirs_if_needed():
-    user_dir = utils.get_hidden_user_dir_path()
+    user_dir = userfolders.get_cache_dir()
 
     if not os.path.exists(user_dir + BATCH_DIR):
         os.mkdir(user_dir + BATCH_DIR)
@@ -269,10 +273,10 @@ def init_dirs_if_needed():
         os.mkdir(get_projects_dir())
 
 def get_projects_dir():
-    return utils.get_hidden_user_dir_path() + PROJECTS_DIR
+    return userfolders.get_cache_dir() + PROJECTS_DIR
 
 def get_datafiles_dir():
-    return utils.get_hidden_user_dir_path() + DATAFILES_DIR
+    return userfolders.get_cache_dir() + DATAFILES_DIR
 
 def get_identifier_from_path(file_path):
     start = file_path.rfind("/")
@@ -280,21 +284,22 @@ def get_identifier_from_path(file_path):
     return file_path[start + 1:end]
 
 def _get_pid_file_path():
-    user_dir = utils.get_hidden_user_dir_path()
+    user_dir = userfolders.get_cache_dir()
     return user_dir + PID_FILE
     
-def destroy_for_identifier(identifier):
+def destroy_for_identifier(identifier, destroy_project_file=True):
     try:
         item_path = get_datafiles_dir() + identifier + ".renderitem"
         os.remove(item_path)
     except:
         pass
     
-    try:
-        project_path = get_projects_dir() + identifier + ".flb"
-        os.remove(project_path)
-    except:
-        pass    
+    if destroy_project_file == True:
+        try:
+            project_path = get_projects_dir() + identifier + ".flb"
+            os.remove(project_path)
+        except:
+            pass
 
 def copy_project(render_item, file_name):
     try:
@@ -304,26 +309,33 @@ def copy_project(render_item, file_name):
         secondary_txt = _("Error message: ") + str(e)
         dialogutils.warning_message(primary_txt, secondary_txt, batch_window.window)
 
+def maybe_create_render_folder(render_path):
+    folder = os.path.dirname(render_path)
+    if not os.path.exists(folder):
+        os.mkdir(folder)
+        
 # --------------------------------------------------------------- app thread and data objects
 def launch_batch_rendering():
     bus = dbus.SessionBus()
     if bus.name_has_owner('flowblade.movie.editor.batchrender'):
-        print "flowblade.movie.editor.batchrender dbus service exists, batch rendering already running"
+        print("flowblade.movie.editor.batchrender dbus service exists, batch rendering already running")
         _show_single_instance_info()
     else:
-        FLOG = open(utils.get_hidden_user_dir_path() + "log_batch_render", 'w')
+        FLOG = open(userfolders.get_cache_dir() + "log_batch_render", 'w')
         subprocess.Popen([sys.executable, respaths.LAUNCH_DIR + "flowbladebatch"], stdin=FLOG, stdout=FLOG, stderr=FLOG)
 
 def main(root_path, force_launch=False):
-    init_dirs_if_needed()
-    
     gtk_version = "%s.%s.%s" % (Gtk.get_major_version(), Gtk.get_minor_version(), Gtk.get_micro_version())
     editorstate.gtk_version = gtk_version
     try:
         editorstate.mlt_version = mlt.LIBMLT_VERSION
     except:
         editorstate.mlt_version = "0.0.99" # magic string for "not found"
-        
+
+    # Get XDG paths etc.
+    userfolders.init()
+    init_dirs_if_needed()
+    
     # Set paths.
     respaths.set_paths(root_path)
 
@@ -342,9 +354,11 @@ def main(root_path, force_launch=False):
     editorpersistance.load()
     if editorpersistance.prefs.theme != appconsts.LIGHT_THEME:
         Gtk.Settings.get_default().set_property("gtk-application-prefer-dark-theme", True)
-        if editorpersistance.prefs.theme == appconsts.FLOWBLADE_THEME:
-            gui.apply_gtk_css()
-            
+        if editorpersistance.prefs.theme == appconsts.FLOWBLADE_THEME \
+            or editorpersistance.prefs.theme == appconsts.FLOWBLADE_THEME_GRAY \
+            or editorpersistance.prefs.theme == appconsts.FLOWBLADE_THEME_NEUTRAL:
+            gui.apply_gtk_css(editorpersistance.prefs.theme)
+        
     repo = mlt.Factory().init()
     processutils.prepare_mlt_repo(repo)
     
@@ -399,7 +413,7 @@ def _display_single_instance_window():
     dialog = Gtk.Dialog("",
                         None,
                         Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT,
-                        (_("OK").encode('utf-8'), Gtk.ResponseType.OK))
+                        (_("OK"), Gtk.ResponseType.OK))
 
     dialog.vbox.pack_start(align, True, True, 0)
     dialogutils.set_outer_margins(dialog.vbox)
@@ -434,26 +448,31 @@ class RenderQueue:
     def load_render_items(self):
         self.queue = []
         self.error_status = None
-        user_dir = utils.get_hidden_user_dir_path()
+        user_dir = userfolders.get_cache_dir()
         data_files_dir = user_dir + DATAFILES_DIR
         data_files = [ f for f in listdir(data_files_dir) if isfile(join(data_files_dir,f)) ]
         for data_file_name in data_files:
+            render_item = None
             try:
                 data_file_path = data_files_dir + data_file_name
-                data_file = open(data_file_path)
-                render_item = pickle.load(data_file)
+                render_item = utils.unpickle(data_file_path)
                 self.queue.append(render_item)
             except Exception as e:
+                print (str(e))
                 if self.error_status == None:
                     self.error_status = []
                 self.error_status.append((data_file_name,  _(" datafile load failed with ") + str(e)))
+                continue
+
             try:
-                render_file = open(render_item.get_project_filepath())
+                render_file = open(render_item.get_project_filepath(), 'rb')
             except Exception as e:
                 if self.error_status == None:
                     self.error_status = []
-                self.error_status.append((render_item.get_project_filepath(), _(" project file load failed with ") + str(e)))
+                self.error_status.append((data_file_name, _(" project file load failed with ") + str(e)))
 
+        print(self.error_status)
+         
         if self.error_status != None:
             for file_path, error_str in self.error_status:
                 identifier = get_identifier_from_path(file_path)
@@ -492,7 +511,7 @@ class RenderQueue:
             except:
                 path_counts[render_item.render_path] = 1
         
-        for k,v in path_counts.iteritems():
+        for k,v in path_counts.items():
             if v > 1:
                 same_paths[k] = v
         
@@ -520,11 +539,11 @@ class BatchRenderItemData:
     def generate_identifier(self):
         id_str = self.project_name + self.timestamp.ctime()
         try:
-            idfier = md5.new(id_str).hexdigest()
+            idfier = hashlib.md5(id_str.encode('utf-8')).hexdigest()
         except:
             ascii_pname = unicodedata.normalize('NFKD', self.project_name).encode('ascii','ignore')
             id_str = str(ascii_pname) + self.timestamp.ctime()
-            idfier = md5.new(id_str).hexdigest()
+            idfier = hashlib.md5(id_str.encode('utf-8')).hexdigest()
         return idfier
 
     def matches_identifier(self, identifier):
@@ -535,12 +554,14 @@ class BatchRenderItemData:
 
     def save(self):
         item_path = get_datafiles_dir() + self.generate_identifier() + ".renderitem"
-        item_write_file = file(item_path, "wb")
-        pickle.dump(self, item_write_file)
+        with atomicfile.AtomicFileWriter(item_path, "wb") as afw:
+            item_write_file = afw.get_file()
+            pickle.dump(self, item_write_file)
 
     def save_as_single_render_item(self, item_path):
-        item_write_file = file(item_path, "wb")
-        pickle.dump(self, item_write_file)
+        with atomicfile.AtomicFileWriter(item_path, "wb") as afw:
+            item_write_file = afw.get_file()
+            pickle.dump(self, item_write_file)
 
     def delete_from_queue(self):
         identifier = self.generate_identifier()
@@ -617,6 +638,8 @@ def get_render_range(render_item):
     else: # both start and end defined
         start_frame = render_item.mark_in
         end_frame = render_item.mark_out
+        if render_item.length - 2 < end_frame:
+            end_frame = render_item.length - 2
         wait_for_stop_render = False
     
     return (start_frame, end_frame, wait_for_stop_render)
@@ -776,8 +799,10 @@ class BatchRenderWindow:
             primary_txt = _("Multiple items with same render target file!")
             
             secondary_txt = _("Later items will render on top of earlier items if this queue is rendered.\n") + \
-                            _("Delete or unqueue some items with same paths:\n\n")
-            for k,v in same_paths.iteritems():
+                            _("Possible fixes:\n\n") + \
+                            "\u2022" + " " + _("Change item render file path from right click popup menu.\n") + \
+                            "\u2022" + " " + _("Delete or unqueue some items with same paths.\n\n")
+            for k,v in same_paths.items():
                 secondary_txt = secondary_txt + str(v) + _(" items with path: ") + str(k) + "\n"
             dialogutils.warning_message(primary_txt, secondary_txt, batch_window.window)
             return
@@ -896,6 +921,7 @@ class RenderQueueView(Gtk.VBox):
         self.text_col_2.set_expand(False)
         self.text_col_2.pack_start(self.text_rend_2, True)
         self.text_col_2.add_attribute(self.text_rend_2, "text", 2)
+        self.text_col_2.set_min_width(90)
 
         self.text_col_3.set_expand(False)
         self.text_col_3.pack_start(self.text_rend_3, True)
@@ -956,6 +982,8 @@ class RenderQueueView(Gtk.VBox):
             file_name = run_save_project_as_dialog(render_item.project_name)
             if file_name != None:
                 copy_project(render_item, file_name)
+        elif msg == "changepath":
+            show_change_render_item_path_dialog(_change_render_item_path_callback, render_item)
 
     def fill_data_model(self, render_queue):
         self.storemodel.clear()        
@@ -973,8 +1001,8 @@ class RenderQueueView(Gtk.VBox):
 def run_save_project_as_dialog(project_name):
     dialog = Gtk.FileChooserDialog(_("Save Render Item Project As"), None, 
                                    Gtk.FileChooserAction.SAVE, 
-                                   (_("Cancel").encode('utf-8'), Gtk.ResponseType.REJECT,
-                                    _("Save").encode('utf-8'), Gtk.ResponseType.ACCEPT), None)
+                                   (_("Cancel"), Gtk.ResponseType.REJECT,
+                                    _("Save"), Gtk.ResponseType.ACCEPT), None)
     dialog.set_action(Gtk.FileChooserAction.SAVE)
     project_name = project_name.rstrip(".flb")
     dialog.set_current_name(project_name + "_FROM_BATCH.flb")
@@ -1005,6 +1033,16 @@ def show_render_properties_panel(render_item):
     start_str = utils.get_tc_string_with_fps(start_frame, render_item.render_data.fps)
     end_str = utils.get_tc_string_with_fps(end_frame, render_item.render_data.fps)
     
+    
+    if hasattr(render_item.render_data, "proxy_mode"):
+        if render_item.render_data.proxy_mode == appconsts.USE_ORIGINAL_MEDIA:
+            proxy_mode = _("Using Original Media")
+        else:
+            proxy_mode = _("Using Proxy Media")
+    else:
+        proxy_mode = _("N/A")
+
+    
     LEFT_WIDTH = 200
     render_item.get_display_name()
     row0 = guiutils.get_two_column_box(guiutils.bold_label(_("Encoding:")), Gtk.Label(label=enc_desc), LEFT_WIDTH)
@@ -1016,6 +1054,7 @@ def show_render_properties_panel(render_item):
     row6 = guiutils.get_two_column_box(guiutils.bold_label(_("Frames Per Second:")), Gtk.Label(label=str(render_item.render_data.fps)), LEFT_WIDTH)
     row7 = guiutils.get_two_column_box(guiutils.bold_label(_("Render Profile Name:")), Gtk.Label(label=str(render_item.render_data.profile_name)), LEFT_WIDTH)
     row8 = guiutils.get_two_column_box(guiutils.bold_label(_("Render Profile:")), Gtk.Label(label=render_item.render_data.profile_desc), LEFT_WIDTH)
+    row8 = guiutils.get_two_column_box(guiutils.bold_label(_("Proxy Mode:")), Gtk.Label(label=proxy_mode), LEFT_WIDTH)
 
     vbox = Gtk.VBox(False, 2)
     vbox.pack_start(Gtk.Label(label=render_item.get_display_name()), False, False, 0)
@@ -1033,11 +1072,66 @@ def show_render_properties_panel(render_item):
 
     title = _("Render Properties")
     dialogutils.panel_ok_dialog(title, vbox)
+
+def show_change_render_item_path_dialog(callback, render_item):
+    cancel_str = _("Cancel")
+    ok_str = _("Change Path")
+    dialog = Gtk.Dialog(_("Change Render Item Path"),
+                        batch_window.window,
+                        Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT,
+                        (cancel_str, Gtk.ResponseType.CANCEL,
+                        ok_str, Gtk.ResponseType.YES))
+
+    INPUT_LABELS_WITDH = 150
     
+    folder, f_name = os.path.split(render_item.render_path)
+
+    out_folder = Gtk.FileChooserButton(_("Select target folder"))
+    out_folder.set_action(Gtk.FileChooserAction.SELECT_FOLDER)
+    out_folder.set_current_folder(folder)
+    
+    folder_row = guiutils.get_two_column_box(Gtk.Label(label=_("Folder:")), out_folder, INPUT_LABELS_WITDH)
+    
+    file_name = Gtk.Entry()
+    file_name.set_text(f_name)
+
+    name_pack = Gtk.HBox(False, 4)
+    name_pack.pack_start(file_name, True, True, 0)
+
+    name_row = guiutils.get_two_column_box(Gtk.Label(label=_("Render file name:")), name_pack, INPUT_LABELS_WITDH)
+ 
+    vbox = Gtk.VBox(False, 2)
+    vbox.pack_start(folder_row, False, False, 0)
+    vbox.pack_start(name_row, False, False, 0)
+    vbox.pack_start(guiutils.pad_label(12, 25), False, False, 0)
+    
+    alignment = guiutils.set_margins(vbox, 12, 12, 12, 12)
+
+    dialog.vbox.pack_start(alignment, True, True, 0)
+    dialogutils.set_outer_margins(dialog.vbox)
+    dialogutils.default_behaviour(dialog)
+    dialog.connect('response', callback, (out_folder, file_name, render_item))
+    dialog.show_all()
+
+def _change_render_item_path_callback(dialog, response_id, data):
+
+    out_folder, file_name, render_item = data
+    if response_id == Gtk.ResponseType.YES:
+
+        render_path = out_folder.get_filename() + "/" + file_name.get_text()
+        render_item.render_path = render_path
+        destroy_for_identifier(render_item.generate_identifier(), False)
+        render_item.save()
+        batch_window.reload_queue()
+        dialog.destroy()
+    else:
+        dialog.destroy()
+        
 def display_render_item_popup_menu(callback, event):
     menu = render_item_menu
     guiutils.remove_children(menu)
-
+    
+    menu.add(_get_menu_item(_("Change Item Render File Path..."), callback,"changepath"))
     menu.add(_get_menu_item(_("Save Item Project As..."), callback,"saveas"))
     menu.add(_get_menu_item(_("Render Properties"), callback,"renderinfo")) 
     _add_separetor(menu)
@@ -1060,7 +1154,7 @@ def _get_menu_item(text, callback, data, sensitive=True):
 
 # --------------------------------------------------- single item render
 def add_single_render_item(flowblade_project, render_path, args_vals_list, mark_in, mark_out, render_data):
-    hidden_dir = utils.get_hidden_user_dir_path()
+    hidden_dir = userfolders.get_cache_dir()
         
     timestamp = datetime.datetime.now()
 
@@ -1095,12 +1189,13 @@ class SingleRenderLaunchThread(threading.Thread):
     
     def run(self):      
         # Launch render process and wait for it to end
-        FLOG = open(utils.get_hidden_user_dir_path() + "log_single_render", 'w')
+        FLOG = open(userfolders.get_cache_dir() + "log_single_render", 'w')
         process = subprocess.Popen([sys.executable, respaths.LAUNCH_DIR + "flowbladesinglerender"], stdin=FLOG, stdout=FLOG, stderr=FLOG)
         process.wait()
 
     
 def single_render_main(root_path):
+    
     # called from .../launch/flowbladesinglerender script
     gtk_version = "%s.%s.%s" % (Gtk.get_major_version(), Gtk.get_minor_version(), Gtk.get_micro_version())
     editorstate.gtk_version = gtk_version
@@ -1108,7 +1203,10 @@ def single_render_main(root_path):
         editorstate.mlt_version = mlt.LIBMLT_VERSION
     except:
         editorstate.mlt_version = "0.0.99" # magic string for "not found"
-        
+    
+    # Get XDG paths etc.
+    userfolders.init()
+    
     # Set paths.
     respaths.set_paths(root_path)
 
@@ -1128,8 +1226,10 @@ def single_render_main(root_path):
     editorpersistance.load()
     if editorpersistance.prefs.theme != appconsts.LIGHT_THEME:
         Gtk.Settings.get_default().set_property("gtk-application-prefer-dark-theme", True)
-        if editorpersistance.prefs.theme == appconsts.FLOWBLADE_THEME:
-            gui.apply_gtk_css()
+        if editorpersistance.prefs.theme == appconsts.FLOWBLADE_THEME \
+            or editorpersistance.prefs.theme == appconsts.FLOWBLADE_THEME_GRAY \
+            or editorpersistance.prefs.theme == appconsts.FLOWBLADE_THEME_NEUTRAL:
+            gui.apply_gtk_css(editorpersistance.prefs.theme)
 
     repo = mlt.Factory().init()
     processutils.prepare_mlt_repo(repo)
@@ -1164,12 +1264,11 @@ class SingleRenderThread(threading.Thread):
         threading.Thread.__init__(self)
     
     def run(self):      
-        hidden_dir = utils.get_hidden_user_dir_path()
+        hidden_dir = userfolders.get_cache_dir()
 
         try:
             data_file_path = hidden_dir + CURRENT_RENDER_RENDER_ITEM
-            data_file = open(data_file_path)
-            render_item = pickle.load(data_file)
+            render_item = utils.unpickle(data_file_path)
             self.error_status = None
         except Exception as e:
             if self.error_status == None:
@@ -1186,12 +1285,17 @@ class SingleRenderThread(threading.Thread):
 
         project = persistance.load_project(project_file_path, False)
 
+        project.c_seq.fix_v1_for_render()
+
         producer = project.c_seq.tractor
         profile = mltprofiles.get_profile(render_item.render_data.profile_name)
         
         vcodec = self.get_vcodec(render_item)
         vformat = self.get_argval(render_item, "f")
         
+        # We just autocreate folder if for some reason it has been deleted.
+        maybe_create_render_folder(render_item.render_path)
+        
         if self.is_frame_sequence_render(vcodec) == True and vformat == None:
             # Frame sequence render
             consumer = renderconsumer.get_img_seq_render_consumer_codec_ext(render_item.render_path,
