@@ -17,12 +17,13 @@
     You should have received a copy of the GNU General Public License
     along with Flowblade Movie Editor.  If not, see <http://www.gnu.org/licenses/>.
 """
-
+import array
+import cairo
 import copy
 import os
 import pickle
+from PIL import Image, ImageFilter
 import threading
-
 import time
 
 from gi.repository import Gtk, Gdk, GdkPixbuf
@@ -30,6 +31,7 @@ from gi.repository import GLib, GObject
 from gi.repository import Pango
 from gi.repository import PangoCairo
 
+import atomicfile
 import toolsdialogs
 from editorstate import PLAYER
 import editorstate
@@ -60,7 +62,7 @@ TEXT_LAYER_LIST_WIDTH = 300
 TEXT_LAYER_LIST_HEIGHT = 150
 
 TEXT_VIEW_WIDTH = 300
-TEXT_VIEW_HEIGHT = 275
+TEXT_VIEW_HEIGHT = 200
 
 DEFAULT_FONT_SIZE = 25
 
@@ -69,10 +71,15 @@ FACE_BOLD = "Bold"
 FACE_ITALIC = "Italic"
 FACE_BOLD_ITALIC = "Bold Italic"
 
+DEFAULT_FONT_SIZE = 40
+
 ALIGN_LEFT = 0
 ALIGN_CENTER = 1
 ALIGN_RIGHT = 2
 
+VERTICAL = 0
+HORIZONTAL = 1
+
 def show_titler():
     global _titler_data
     if _titler_data == None:
@@ -105,9 +112,17 @@ def titler_destroy():
         _titler_data = None
 
 def reset_titler():
-    global _titler_data
-    _titler_data = None
-        
+    global _titler, _keep_titler_data, _titler_data
+
+    if _titler != None:
+        temp_keep_val = _keep_titler_data
+        _keep_titler_data = True
+        titler_destroy()
+        _keep_titler_data = temp_keep_val
+        show_titler()
+    else:
+        _titler_data = None
+
 # ------------------------------------------------------------- data
 class TextLayer:
     """
@@ -120,13 +135,16 @@ class TextLayer:
         self.angle = 0.0 # future feature
         self.font_family = "Times New Roman"
         self.font_face = FACE_REGULAR
-        self.font_size = 15
+        self.font_size = DEFAULT_FONT_SIZE
         self.fill_on = True
         self.color_rgba = (1.0, 1.0, 1.0, 1.0) 
         self.alignment = ALIGN_LEFT
         self.pixel_size = (100, 100)
         self.spacing = 5
-        
+
+        self.gradient_color_rgba = None
+        self.gradient_direction = VERTICAL
+
         self.outline_on = False
         self.outline_color_rgba = (0.3, 0.3, 0.3, 1.0) 
         self.outline_width = 2
@@ -136,9 +154,9 @@ class TextLayer:
         self.shadow_opacity = 100
         self.shadow_xoff = 3
         self.shadow_yoff = 3
-        self.shadow_blur = 0.0 # not impl yet, for future so that we don't need to break save format again
+        self.shadow_blur = 0.0
         
-        self.pango_layout = None # PangoTextLayout(self)
+        self.pango_layout = None # PangoTextLayout object
 
         self.layer_attributes = None # future feature 
         self.visible = True
@@ -170,16 +188,25 @@ class TitlerData:
         return self.layers.index(self.active_layer)
     
     def save(self, save_file_path):
-        save_data = copy.deepcopy(self)
+        save_data = copy.copy(self)
         for layer in save_data.layers:
             layer.pango_layout = None
-        write_file = file(save_file_path, "wb")
-        pickle.dump(save_data, write_file)
-   
+        with atomicfile.AtomicFileWriter(save_file_path, "wb") as afw:
+            write_file = afw.get_file()
+            pickle.dump(save_data, write_file)
+        self.create_pango_layouts() # we just destroyed these because they don't pickle, they need to be recreated.
+
     def create_pango_layouts(self):
         for layer in self.layers:
             layer.pango_layout = PangoTextLayout(layer)
-            
+
+    def data_compatibility_update(self):
+        # We added new stuff for 2.8 and need to update data created with older versions.
+        for layer in self.layers:
+            if hasattr(layer, "gradient_color_rgba") == False:
+                layer.gradient_color_rgba = None
+                layer.gradient_direction = VERTICAL
+
 # ---------------------------------------------------------- editor
 class Titler(Gtk.Window):
     def __init__(self):
@@ -187,11 +214,11 @@ class Titler(Gtk.Window):
         self.set_title(_("Titler"))
         self.connect("delete-event", lambda w, e:close_titler())
         
-        if editorstate.screen_size_small_height() == True:
+        if editorstate.SCREEN_HEIGHT < 865:
             global TEXT_LAYER_LIST_HEIGHT, TEXT_VIEW_HEIGHT, VIEW_EDITOR_HEIGHT
-            TEXT_LAYER_LIST_HEIGHT = 150
-            TEXT_VIEW_HEIGHT = 180
-            VIEW_EDITOR_HEIGHT = 450
+            TEXT_LAYER_LIST_HEIGHT = 130
+            TEXT_VIEW_HEIGHT = 130
+            VIEW_EDITOR_HEIGHT = 350
 
         if editorstate.screen_size_small_height() == True:
             global VIEW_EDITOR_WIDTH
@@ -255,7 +282,7 @@ class Titler(Gtk.Window):
         font_map = PangoCairo.font_map_get_default()
         unsorted_families = font_map.list_families()
         if len(unsorted_families) == 0:
-            print "No font families found in system! Titler will not work."
+            print("No font families found in system! Titler will not work.")
         self.font_families = sorted(unsorted_families, key=lambda family: family.get_name())
         self.font_family_indexes_for_name = {}
         combo = Gtk.ComboBoxText()
@@ -267,8 +294,7 @@ class Titler(Gtk.Window):
         combo.set_active(0)
         self.font_select = combo
         self.font_select.connect("changed", self._edit_value_changed)
-    
-        adj = Gtk.Adjustment(float(DEFAULT_FONT_SIZE), float(1), float(300), float(1))
+        adj = Gtk.Adjustment(value=float(DEFAULT_FONT_SIZE), lower=float(1), upper=float(300), step_incr=float(1))
         self.size_spin = Gtk.SpinButton()
         self.size_spin.set_adjustment(adj)
         self.size_spin.connect("changed", self._edit_value_changed)
@@ -278,7 +304,8 @@ class Titler(Gtk.Window):
         font_main_row.pack_start(self.font_select, True, True, 0)
         font_main_row.pack_start(guiutils.pad_label(5, 5), False, False, 0)
         font_main_row.pack_start(self.size_spin, False, False, 0)
-
+        guiutils.set_margins(font_main_row, 0,4,0,0)
+        
         self.bold_font = Gtk.ToggleButton()
         self.italic_font = Gtk.ToggleButton()
         bold_icon = Gtk.Image.new_from_stock(Gtk.STOCK_BOLD, 
@@ -329,14 +356,13 @@ class Titler(Gtk.Window):
         buttons_box.pack_start(self.fill_on, False, False, 0)
         buttons_box.pack_start(Gtk.Label(), True, True, 0)
 
-        outline_label = Gtk.Label(_("<b>Outline</b>"))
-        outline_label.set_use_markup(True)
+        # ------------------------------------------- Outline Panel
         outline_size = Gtk.Label(_("Size:"))
         
         self.out_line_color_button = Gtk.ColorButton.new_with_rgba(Gdk.RGBA(red=0.3, green=0.3, blue=0.3, alpha=1.0))
         self.out_line_color_button.connect("color-set", self._edit_value_changed)
-        
-        adj2 = Gtk.Adjustment(float(3), float(1), float(50), float(1))
+
+        adj2 = Gtk.Adjustment(value=float(3), lower=float(1), upper=float(50), step_incr=float(1))
         self.out_line_size_spin = Gtk.SpinButton()
         self.out_line_size_spin.set_adjustment(adj2)
         self.out_line_size_spin.connect("changed", self._edit_value_changed)
@@ -347,8 +373,6 @@ class Titler(Gtk.Window):
         self.outline_on.connect("toggled", self._edit_value_changed)
         
         outline_box = Gtk.HBox()
-        outline_box.pack_start(outline_label, False, False, 0)
-        outline_box.pack_start(guiutils.pad_label(15, 1), False, False, 0)
         outline_box.pack_start(outline_size, False, False, 0)
         outline_box.pack_start(guiutils.pad_label(2, 1), False, False, 0)
         outline_box.pack_start(self.out_line_size_spin, False, False, 0)
@@ -358,26 +382,29 @@ class Titler(Gtk.Window):
         outline_box.pack_start(self.outline_on, False, False, 0)
         outline_box.pack_start(Gtk.Label(), True, True, 0)
 
-        shadow_label = Gtk.Label(_("<b>Shadow</b>"))
-        shadow_label.set_use_markup(True)
+        # -------------------------------------------- Shadow panel 
         shadow_opacity_label = Gtk.Label(_("Opacity:"))
         shadow_xoff = Gtk.Label(_("X Off:"))
         shadow_yoff = Gtk.Label(_("Y Off:"))
+        shadow_blur_label = Gtk.Label(_("Blur:"))
         
         self.shadow_opa_spin = Gtk.SpinButton()
-        adj3 = Gtk.Adjustment(float(100), float(1), float(100), float(1))
+ 
+        adj3 = Gtk.Adjustment(value=float(100), lower=float(1), upper=float(100), step_incr=float(1))
         self.shadow_opa_spin.set_adjustment(adj3)
         self.shadow_opa_spin.connect("changed", self._edit_value_changed)
         self.shadow_opa_spin.connect("key-press-event", self._key_pressed_on_widget)
 
         self.shadow_xoff_spin = Gtk.SpinButton()
-        adj4 = Gtk.Adjustment(float(3), float(1), float(100), float(1))
+
+        adj4 = Gtk.Adjustment(value=float(3), lower=float(1), upper=float(100), step_incr=float(1))
         self.shadow_xoff_spin.set_adjustment(adj4)
         self.shadow_xoff_spin.connect("changed", self._edit_value_changed)
         self.shadow_xoff_spin.connect("key-press-event", self._key_pressed_on_widget)
 
         self.shadow_yoff_spin = Gtk.SpinButton()
-        adj5 = Gtk.Adjustment(float(3), float(1), float(100), float(1))
+
+        adj5 = Gtk.Adjustment(value=float(3), lower=float(1), upper=float(100), step_incr=float(1))
         self.shadow_yoff_spin.set_adjustment(adj5)
         self.shadow_yoff_spin.connect("changed", self._edit_value_changed)
         self.shadow_yoff_spin.connect("key-press-event", self._key_pressed_on_widget)
@@ -389,9 +416,12 @@ class Titler(Gtk.Window):
         self.shadow_color_button = Gtk.ColorButton.new_with_rgba(Gdk.RGBA(red=0.3, green=0.3, blue=0.3, alpha=1.0))
         self.shadow_color_button.connect("color-set", self._edit_value_changed)
 
+        self.shadow_blur_spin = Gtk.SpinButton()
+        adj6 = Gtk.Adjustment(value=float(0), lower=float(0), upper=float(20), step_incr=float(1))
+        self.shadow_blur_spin.set_adjustment(adj6)
+        self.shadow_blur_spin.connect("changed", self._edit_value_changed)
+
         shadow_box_1 = Gtk.HBox()
-        shadow_box_1.pack_start(shadow_label, False, False, 0)
-        shadow_box_1.pack_start(guiutils.pad_label(15, 1), False, False, 0)
         shadow_box_1.pack_start(shadow_opacity_label, False, False, 0)
         shadow_box_1.pack_start(self.shadow_opa_spin, False, False, 0)
         shadow_box_1.pack_start(guiutils.pad_label(15, 1), False, False, 0)
@@ -399,6 +429,7 @@ class Titler(Gtk.Window):
         shadow_box_1.pack_start(guiutils.pad_label(2, 1), False, False, 0)
         shadow_box_1.pack_start(self.shadow_on, False, False, 0)
         shadow_box_1.pack_start(Gtk.Label(), True, True, 0)
+        guiutils.set_margins(shadow_box_1, 0,4,0,0)
 
         shadow_box_2 = Gtk.HBox()
         shadow_box_2.pack_start(shadow_xoff, False, False, 0)
@@ -407,7 +438,42 @@ class Titler(Gtk.Window):
         shadow_box_2.pack_start(shadow_yoff, False, False, 0)
         shadow_box_2.pack_start(self.shadow_yoff_spin, False, False, 0)
         shadow_box_2.pack_start(Gtk.Label(), True, True, 0)
+
+        shadow_box_3 = Gtk.HBox()
+        shadow_box_3.pack_start(shadow_blur_label, False, False, 0)
+        shadow_box_3.pack_start(self.shadow_blur_spin, False, False, 0)
+        shadow_box_3.pack_start(Gtk.Label(), True, True, 0)
+
+        # ------------------------------------ Gradient panel
+        self.gradient_color_button = Gtk.ColorButton.new_with_rgba(Gdk.RGBA(red=0.0, green=0.0, blue=0.8, alpha=1.0))
+        self.gradient_color_button.connect("color-set", self._edit_value_changed)
+        self.gradient_on = Gtk.CheckButton()
+        self.gradient_on.set_active(True)
+        self.gradient_on.connect("toggled", self._edit_value_changed)
+
+        direction_label = Gtk.Label(_("Gradient Direction:"))
+        self.direction_combo = Gtk.ComboBoxText()
+        self.direction_combo.append_text(_("Vertical"))
+        self.direction_combo.append_text(_("Horizontal"))
+        self.direction_combo.set_active(0)
+        self.direction_combo.connect("changed", self._edit_value_changed)
+         
+        gradient_box_row1 = Gtk.HBox()
+        gradient_box_row1.pack_start(self.gradient_on, False, False, 0)
+        gradient_box_row1.pack_start(self.gradient_color_button, False, False, 0)
+        gradient_box_row1.pack_start(Gtk.Label(), True, True, 0)
+
+        gradient_box_row2 = Gtk.HBox()
+        gradient_box_row2.pack_start(direction_label, False, False, 0)
+        gradient_box_row2.pack_start(self.direction_combo, False, False, 0)
+        gradient_box_row2.pack_start(Gtk.Label(), True, True, 0)
         
+        gradient_box = Gtk.VBox()
+        gradient_box.pack_start(gradient_box_row1, False, False, 0)
+        gradient_box.pack_start(gradient_box_row2, False, False, 0)
+        gradient_box.pack_start(Gtk.Label(), True, True, 0)
+                
+        # ---------------------------------------------------- Save and Load buttons        
         load_layers = Gtk.Button(_("Load Layers"))
         load_layers.connect("clicked", lambda w:self._load_layers_pressed())
         save_layers = Gtk.Button(_("Save Layers"))
@@ -419,18 +485,21 @@ class Titler(Gtk.Window):
         layers_save_buttons_row.pack_start(save_layers, False, False, 0)
         layers_save_buttons_row.pack_start(load_layers, False, False, 0)
         layers_save_buttons_row.pack_start(Gtk.Label(), True, True, 0)
-        
-        adj = Gtk.Adjustment(float(0), float(0), float(3000), float(1))
+
+        # ---------------------------------------------------- X, Y pos input
+        adj = Gtk.Adjustment(value=float(0), lower=float(0), upper=float(3000), step_incr=float(1))
         self.x_pos_spin = Gtk.SpinButton()
         self.x_pos_spin.set_adjustment(adj)
         self.x_pos_spin.connect("changed", self._position_value_changed)
         self.x_pos_spin.connect("key-press-event", self._key_pressed_on_widget)
-        adj = Gtk.Adjustment(float(0), float(0), float(3000), float(1))
+
+        adj = Gtk.Adjustment(value=float(0), lower=float(0), upper=float(3000), step_incr=float(1))
         self.y_pos_spin = Gtk.SpinButton()
         self.y_pos_spin.set_adjustment(adj)
         self.y_pos_spin.connect("changed", self._position_value_changed)
         self.y_pos_spin.connect("key-press-event", self._key_pressed_on_widget)
-        adj = Gtk.Adjustment(float(0), float(0), float(3000), float(1))
+
+        adj = Gtk.Adjustment(value=float(0), lower=float(0), upper=float(3000), step_incr=float(1))
         self.rotation_spin = Gtk.SpinButton()
         self.rotation_spin.set_adjustment(adj)
         self.rotation_spin.connect("changed", self._position_value_changed)
@@ -441,6 +510,7 @@ class Titler(Gtk.Window):
                                        Gtk.IconSize.BUTTON)
         undo_pos.set_image(undo_icon)
 
+        # ------------------------------------------------- Timeline controls
         next_icon = Gtk.Image.new_from_file(respaths.IMAGE_PATH + "next_frame_s.png")
         prev_icon = Gtk.Image.new_from_file(respaths.IMAGE_PATH + "prev_frame_s.png")
         prev_frame = Gtk.Button()
@@ -451,7 +521,9 @@ class Titler(Gtk.Window):
         next_frame.connect("clicked", lambda w:self._next_frame_pressed())
 
         self.scale_selector = vieweditor.ScaleSelector(self)
+        self.view_editor.scale_select = self.scale_selector
 
+        # ------------------------------------------------------ Panels
         timeline_box = Gtk.HBox()
         timeline_box.pack_start(self.tc_display.widget, False, False, 0)
         timeline_box.pack_start(guiutils.pad_label(12, 12), False, False, 0)
@@ -471,8 +543,6 @@ class Titler(Gtk.Window):
         positions_box.pack_start(guiutils.pad_label(40, 5), False, False, 0)
         positions_box.pack_start(Gtk.Label(label="Y:"), False, False, 0)
         positions_box.pack_start(self.y_pos_spin, False, False, 0)
-        #positions_box.pack_start(Gtk.Label(label=_("Angle")), False, False, 0)
-        #positions_box.pack_start(self.rotation_spin, False, False, 0)
         positions_box.pack_start(guiutils.pad_label(40, 5), False, False, 0)
         positions_box.pack_start(center_h, False, False, 0)
         positions_box.pack_start(center_v, False, False, 0)
@@ -480,27 +550,41 @@ class Titler(Gtk.Window):
 
         controls_panel_1 = Gtk.VBox()
         controls_panel_1.pack_start(add_del_box, False, False, 0)
-        controls_panel_1.pack_start(self.layer_list, False, False, 0)
+        controls_panel_1.pack_start(self.layer_list, True, True, 0)
         controls_panel_1.pack_start(layers_save_buttons_row, False, False, 0)
 
         controls_panel_2 = Gtk.VBox()
-        controls_panel_2.pack_start(scroll_frame, True, True, 0)
         controls_panel_2.pack_start(font_main_row, False, False, 0)
         controls_panel_2.pack_start(buttons_box, False, False, 0)
-        controls_panel_2.pack_start(guiutils.pad_label(40, 1), False, False, 0)
-        controls_panel_2.pack_start(outline_box, False, False, 0)
-        controls_panel_2.pack_start(guiutils.pad_label(40, 1), False, False, 0)
-        controls_panel_2.pack_start(shadow_box_1, False, False, 0)
-        controls_panel_2.pack_start(shadow_box_2, False, False, 0)
+
+        controls_panel_3 = Gtk.VBox()
+        controls_panel_3.pack_start(outline_box, False, False, 0)
+
+        controls_panel_4 = Gtk.VBox()
+        controls_panel_4.pack_start(shadow_box_1, False, False, 0)
+        controls_panel_4.pack_start(shadow_box_2, False, False, 0)
+        controls_panel_4.pack_start(shadow_box_3, False, False, 0)
+
+        controls_panel_5 = Gtk.VBox()
+        controls_panel_5.pack_start(gradient_box, False, False, 0)
+
+        notebook = Gtk.Notebook()
+        notebook.append_page(guiutils.set_margins(controls_panel_2,8,8,8,8), Gtk.Label(label=_("Font")))
+        notebook.append_page(guiutils.set_margins(controls_panel_3,8,8,8,8), Gtk.Label(label=_("Outline")))
+        notebook.append_page(guiutils.set_margins(controls_panel_4,8,8,8,8), Gtk.Label(label=_("Shadow")))
+        notebook.append_page(guiutils.set_margins(controls_panel_5,8,8,8,8), Gtk.Label(label=_("Gradient")))
         
         controls_panel = Gtk.VBox()
-        controls_panel.pack_start(guiutils.get_named_frame(_("Active Layer"),controls_panel_2), True, True, 0)
-        controls_panel.pack_start(guiutils.get_named_frame(_("Layers"),controls_panel_1), False, False, 0)
+        controls_panel.pack_start(guiutils.get_named_frame(_("Layer Text"), scroll_frame), True, True, 0)
+        controls_panel.pack_start(guiutils.set_margins(notebook, 0,0,10,4), False, False, 0)
+        controls_panel.pack_start(guiutils.pad_label(1, 24), False, False, 0)
+        controls_panel.pack_start(guiutils.get_named_frame(_("Layers"),controls_panel_1), True, True, 0)
  
         view_editor_editor_buttons_row = Gtk.HBox()
         view_editor_editor_buttons_row.pack_start(positions_box, False, False, 0)
         view_editor_editor_buttons_row.pack_start(Gtk.Label(), True, True, 0)
 
+        # ------------------------------------------------------- Editor buttons
         keep_label = Gtk.Label(label=_("Keep Layers When Closed"))
         self.keep_layers_check = Gtk.CheckButton()
         self.keep_layers_check.set_active(_keep_titler_data)
@@ -518,20 +602,23 @@ class Titler(Gtk.Window):
         
         editor_buttons_row = Gtk.HBox()
         editor_buttons_row.pack_start(Gtk.Label(), True, True, 0)
-        editor_buttons_row.pack_start(keep_label, False, False, 0)
         editor_buttons_row.pack_start(self.keep_layers_check, False, False, 0)
+        editor_buttons_row.pack_start(guiutils.pad_label(4, 1), False, False, 0)
+        editor_buttons_row.pack_start(keep_label, False, False, 0)
         editor_buttons_row.pack_start(guiutils.pad_label(24, 2), False, False, 0)
-        editor_buttons_row.pack_start(open_label, False, False, 0)
         editor_buttons_row.pack_start(self.open_in_current_check, False, False, 0)
-        editor_buttons_row.pack_start(guiutils.pad_label(24, 2), False, False, 0)
+        editor_buttons_row.pack_start(guiutils.pad_label(4, 1), False, False, 0)
+        editor_buttons_row.pack_start(open_label, False, False, 0)
+        editor_buttons_row.pack_start(guiutils.pad_label(32, 2), False, False, 0)
         editor_buttons_row.pack_start(exit_b, False, False, 0)
         editor_buttons_row.pack_start(save_titles_b, False, False, 0)
         
+        # ------------------------------------------------------ window layout
         editor_panel = Gtk.VBox()
         editor_panel.pack_start(self.view_editor, True, True, 0)
         editor_panel.pack_start(timeline_box, False, False, 0)
         editor_panel.pack_start(guiutils.get_in_centering_alignment(view_editor_editor_buttons_row), False, False, 0)
-        editor_panel.pack_start(guiutils.pad_label(2, 24), True, True, 0)
+        editor_panel.pack_start(guiutils.pad_label(2, 24), False, False, 0)
         editor_panel.pack_start(editor_buttons_row, False, False, 0)
 
         editor_row = Gtk.HBox()
@@ -546,6 +633,7 @@ class Titler(Gtk.Window):
         self._update_gui_with_active_layer_data()
         self.show_all()
 
+        # -------------------------------------------------- window state listeners
         self.connect("size-allocate", lambda w, e:self.window_resized())
         self.connect("window-state-event", lambda w, e:self.window_resized())
     
@@ -561,12 +649,17 @@ class Titler(Gtk.Window):
             text_layer.mouse_released_listener  = self._editor_layer_mouse_released
             text_layer.set_rect_pos(layer.x, layer.y)
             text_layer.update_rect = True
+            text_layer.visible = True
             self.view_editor.add_layer(text_layer)
 
+        for layer in _titler_data.layers:
+            layer.visible = True
+
         self._activate_layer(0)
         self.layer_list.fill_data_model()
         self.view_editor.edit_area.queue_draw()
-
+        self._select_layer(0)
+        
     def show_current_frame(self):
         frame = PLAYER().current_frame()
         length = PLAYER().producer.get_length()
@@ -644,12 +737,13 @@ class Titler(Gtk.Window):
             try:
                 filenames = dialog.get_filenames()
                 load_path = filenames[0]
-                f = open(load_path)
-                new_data = pickle.load(f)
+                new_data = utils.unpickle(load_path)
+                new_data.data_compatibility_update()
                 global _titler_data
                 _titler_data = new_data
                 self.load_titler_data()
-            except:
+            except Exception as e:
+                print("Titler._load_layers_dialog_callback", e)
                 dialog.destroy()
                 # INFOWINDOW
                 return
@@ -680,7 +774,7 @@ class Titler(Gtk.Window):
             self._update_active_layout()
             return True
 
-            # update layer for enter on x, y, angle
+        # update layer for enter on x, y, angle
         if ((event.keyval == Gdk.KEY_Return) and ((widget == self.x_pos_spin) or
             (widget == self.y_pos_spin) or (widget == self.rotation_spin))):
             self.x_pos_spin.update()
@@ -707,20 +801,22 @@ class Titler(Gtk.Window):
         view_editor_layer.mouse_released_listener  = self._editor_layer_mouse_released
         self.view_editor.edit_layers.append(view_editor_layer)
         
+        layer_index = len(_titler_data.layers) - 1
         self.layer_list.fill_data_model()
-        self._activate_layer(len(_titler_data.layers) - 1)
+        self._activate_layer(layer_index, True)
+        self._select_layer(layer_index)
         
     def _del_layer_pressed(self):
         # we always need 1 layer
         if len(_titler_data.layers) < 2:
             return
 
-        #active_index = _titler_data.get_active_layer_index()
         _titler_data.layers.remove(_titler_data.active_layer)
         self.view_editor.edit_layers.remove(self.view_editor.active_layer)
         self.layer_list.fill_data_model()
         self._activate_layer(0)
-
+        self._select_layer(0)
+        
     def _layer_visibility_toggled(self, layer_index):
         toggled_visible = (self.view_editor.edit_layers[layer_index].visible == False)
         self.view_editor.edit_layers[layer_index].visible = toggled_visible
@@ -780,17 +876,27 @@ class Titler(Gtk.Window):
         _titler_data.active_layer = _titler_data.layers[layer_index]
         self._update_gui_with_active_layer_data()
         _titler_data.active_layer.update_pango_layout()
+        self._select_layer(layer_index)
 
-    def _activate_layer(self, layer_index):
+    def _activate_layer(self, layer_index, is_new_layer=False):
         global _titler_data
         _titler_data.active_layer = _titler_data.layers[layer_index]
         
-        self._update_gui_with_active_layer_data()
+        if not is_new_layer:
+            self._update_gui_with_active_layer_data() # Update GUI with layer data
+        else:
+            self._update_active_layout_font_properties() # Update layer font properties with current GUI values.
+            self._update_gui_with_active_layer_data() # Update GUI with layer data
+            
         _titler_data.active_layer.update_pango_layout()
         self.view_editor.activate_layer(layer_index)
         self.view_editor.active_layer.update_rect = True
         self.view_editor.edit_area.queue_draw()
 
+    def _select_layer(self, layer_index):
+        self.layer_list.treeview.get_selection().select_path(Gtk.TreePath.new_from_indices([layer_index]))
+        self.layer_list.queue_draw()
+
     def _editor_layer_mouse_released(self):
         p = self.view_editor.active_layer.edit_point_shape.edit_points[0]
         
@@ -806,6 +912,7 @@ class Titler(Gtk.Window):
 
     def _text_changed(self, widget):
         self._update_active_layout()
+        self._select_layer(_titler_data.get_active_layer_index())
 
     def _position_value_changed(self, widget):
         # mouse release when layer is moved causes this method to be called,
@@ -822,7 +929,7 @@ class Titler(Gtk.Window):
     def _edit_value_changed(self, widget):
         self._update_active_layout()
 
-    def _update_active_layout(self, fill_layers_data_if_needed=True):
+    def _update_active_layout(self):
         if self.block_updates:
             return
 
@@ -835,7 +942,16 @@ class Titler(Gtk.Window):
             update_layers_list = False
 
         _titler_data.active_layer.text = text
-        
+
+        self._update_active_layout_font_properties()
+
+        # We only wnat to update layer list data model when this called after user typing 
+        if update_layers_list:
+            self.layer_list.fill_data_model()
+
+        self.view_editor.edit_area.queue_draw()
+
+    def _update_active_layout_font_properties(self):
         family = self.font_families[self.font_select.get_active()]
         _titler_data.active_layer.font_family = family.get_name()
 
@@ -871,7 +987,6 @@ class Titler(Gtk.Window):
         _titler_data.active_layer.outline_on = self.outline_on.get_active()
         _titler_data.active_layer.outline_width = self.out_line_size_spin.get_value()
 
-        
         # SHADOW
         color = self.shadow_color_button.get_color()
         r, g, b = utils.hex_to_rgb(color.to_string())
@@ -882,22 +997,28 @@ class Titler(Gtk.Window):
         _titler_data.active_layer.shadow_opacity = self.shadow_opa_spin.get_value()
         _titler_data.active_layer.shadow_xoff = self.shadow_xoff_spin.get_value()
         _titler_data.active_layer.shadow_yoff = self.shadow_yoff_spin.get_value()
-                
+        _titler_data.active_layer.shadow_blur = self.shadow_blur_spin.get_value()
+        
+        # GRADIENT
+        if self.gradient_on.get_active() == True:
+            color = self.gradient_color_button.get_color()
+            r, g, b = utils.hex_to_rgb(color.to_string())
+            new_color = (r/65535.0, g/65535.0, b/65535.0, 1.0)  
+            _titler_data.active_layer.gradient_color_rgba = new_color
+        else:
+            _titler_data.active_layer.gradient_color_rgba = None
+        _titler_data.active_layer.gradient_direction = self.direction_combo.get_active() # Combo indexes correspond with values of VERTICAL and HORIZONTAL
+        
+        
         self.view_editor.active_layer.update_rect = True
         _titler_data.active_layer.update_pango_layout()
 
-        # We only wnat to update layer list data model when this called after user typing 
-        if update_layers_list:
-            self.layer_list.fill_data_model()
-
-        self.view_editor.edit_area.queue_draw()
-
     def _update_gui_with_active_layer_data(self):
         if _filling_layer_list:
             return
         
         # This a bit hackish, but works. Finding a method that blocks all
-        # gui events from being added to queue would be nice.
+        # gui events from being added to event queue would be nice.
         self.block_updates = True
         
         # TEXT
@@ -959,7 +1080,20 @@ class Titler(Gtk.Window):
         self.shadow_xoff_spin.set_value(layer.shadow_xoff)
         self.shadow_yoff_spin.set_value(layer.shadow_yoff)
         self.shadow_on.set_active(layer.shadow_on)
+        self.shadow_blur_spin.set_value(layer.shadow_blur)
         
+        # GRADIENT
+        if layer.gradient_color_rgba != None:
+            r, g, b, a = layer.gradient_color_rgba
+            button_color = Gdk.RGBA(r, g, b, 1.0)
+            self.gradient_color_button.set_rgba(button_color)
+            self.gradient_on.set_active(True)
+        else:
+            button_color = Gdk.RGBA(0.0, 0.0, 0.6, 1.0)
+            self.gradient_color_button.set_rgba(button_color)
+            self.gradient_on.set_active(False)
+        self.direction_combo.set_active(layer.gradient_direction)
+                
         self.block_updates = False
 
 
@@ -982,7 +1116,8 @@ class PangoTextLayout:
         self.alignment = self._get_pango_alignment_for_layer(layer)
         self.pixel_size = layer.pixel_size
         self.fill_on = layer.fill_on
-        
+        self.gradient_color_rgba = layer.gradient_color_rgba
+
         self.outline_color_rgba = layer.outline_color_rgba
         self.outline_on = layer.outline_on
         self.outline_width = layer.outline_width
@@ -992,34 +1127,91 @@ class PangoTextLayout:
         self.shadow_opacity = layer.shadow_opacity
         self.shadow_xoff = layer.shadow_xoff
         self.shadow_yoff = layer.shadow_yoff
+        self.shadow_blur = layer.shadow_blur
+        
+        self.gradient_color_rgba = layer.gradient_color_rgba
+        self.gradient_direction = layer.gradient_direction
 
     # called from vieweditor draw vieweditor-> editorlayer->here
-    def draw_layout(self, cr, x, y, rotation, xscale, yscale):
+    def draw_layout(self, cr, x, y, rotation, xscale, yscale, view_editor):
         cr.save()
         
         layout = PangoCairo.create_layout(cr)
         layout.set_text(self.text, -1)
         layout.set_font_description(self.font_desc)
         layout.set_alignment(self.alignment)
-    
         self.pixel_size = layout.get_pixel_size()
 
         # Shadow
         if self.shadow_on:
             cr.save()
+
+            # Get colors.
             r, g, b = self.shadow_color_rgb
             a = self.shadow_opacity / 100.0
-            cr.set_source_rgba(r, g, b, a)
-            cr.move_to(x + self.shadow_xoff, y + self.shadow_yoff)
-            cr.scale(xscale, yscale)
-            cr.rotate(rotation)
-            PangoCairo.update_layout(cr, layout)
-            PangoCairo.show_layout(cr, layout)
-            cr.restore()
-        
+
+            # Blurred shadow need s own ImageSurface
+            if self.shadow_blur != 0.0:
+                blurred_img = cairo.ImageSurface(cairo.FORMAT_ARGB32, view_editor.profile_w,  view_editor.profile_h)
+                cr_blurred = cairo.Context(blurred_img)
+                transform_cr = cr_blurred # Set draw transform_cr to cotext for newly created image.
+            else:
+                transform_cr = cr # Set draw transform_cr to out context.
+
+            # Transform and set color.
+            transform_cr.set_source_rgba(r, g, b, a)
+            effective_shadow_xoff = self.shadow_xoff * xscale
+            effective_shadow_yoff = self.shadow_yoff * yscale
+            transform_cr.move_to(x + effective_shadow_xoff, y + effective_shadow_yoff)
+            transform_cr.scale(xscale, yscale)
+            transform_cr.rotate(rotation)
+
+            # If no blur just draw layout on out context.
+            if self.shadow_blur == 0.0:
+                PangoCairo.update_layout(cr, layout)
+                PangoCairo.show_layout(cr, layout)
+                cr.restore()
+            else:
+                # If we have blur - draw shadow, blur it and then draw on out context.
+                PangoCairo.update_layout(cr_blurred, layout)
+                PangoCairo.show_layout(cr_blurred, layout)
+
+                img2 = Image.frombuffer("RGBA", (blurred_img.get_width(), blurred_img.get_height()), blurred_img.get_data(), "raw", "RGBA", 0, 1)
+                effective_blur = xscale * self.shadow_blur # This is not going to be exact
+                                                           # on non-100% scales but let's try to get approximation. 
+                img2 = img2.filter(ImageFilter.GaussianBlur(radius=int(effective_blur)))
+                imgd = img2.tobytes()
+                a = array.array('B',imgd)
+
+                stride = blurred_img.get_width() * 4
+                draw_surface = cairo.ImageSurface.create_for_data (a, cairo.FORMAT_ARGB32,
+                                                              blurred_img.get_width(), blurred_img.get_height(), stride)
+                cr.restore()
+                cr.set_source_surface(draw_surface, 0, 0)
+                cr.paint()
+
         # Text
         if self.fill_on:
-            cr.set_source_rgba(*self.color_rgba)
+            if self.gradient_color_rgba == None:
+                cr.set_source_rgba(*self.color_rgba)
+            else:
+                w, h = self.pixel_size
+                w = float(w) * xscale
+                h = float(h) * yscale
+                if self.gradient_direction == HORIZONTAL:
+                    grad = cairo.LinearGradient (x, 0, x + w, 0)
+                else:
+                    grad = cairo.LinearGradient (0, y, 0, y + h)
+                
+                r, g, b, a = self.color_rgba
+                rg, gg, bg, ag =  self.gradient_color_rgba 
+                    
+                CLIP_COLOR_GRAD_1 = (0,  r, g, b, 1)
+                CLIP_COLOR_GRAD_2 = (1,  rg, gg, bg, 1)
+                grad.add_color_stop_rgba(*CLIP_COLOR_GRAD_1)
+                grad.add_color_stop_rgba(*CLIP_COLOR_GRAD_2)
+                cr.set_source(grad)
+
             cr.move_to(x, y)
             cr.scale(xscale, yscale)
             cr.rotate(rotation)
